@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * Dehacked file support
@@ -45,7 +45,7 @@
 //      comments in this file
 //      New string names all start with an extra s_ to avoid conflicts,
 //      but are otherwise identical to the original including uppercase.
-//      This is partly to keep the changes simple and partly for easier
+//      This is partly to keep the changes simple and partly for easier 
 //      identification of the locations in which they're used.
 //
 //      Printed strings for translation
@@ -86,10 +86,10 @@ extern const char *s_QLPROMPT; // = QLPROMPT;
 "you can't start a new game\n"\
 "while in a network game.\n\n"PRESSKEY
 */
-extern char *s_NEWGAME; // = NEWGAME;
+extern const char *s_NEWGAME; // = NEWGAME;
 
 // CPhipps - message given when asked if to restart the level
-extern char *s_RESTARTLEVEL;
+extern const char *s_RESTARTLEVEL;
 
 /*
 #define NIGHTMARE     \
@@ -864,7 +864,7 @@ extern const char* s_C5TEXT; // = C5TEXT;
 extern const char* s_C6TEXT; // = C6TEXT;
 
 
-// after map 06
+// after map 06 
 
 /*
 #define P1TEXT  \
@@ -1124,4 +1124,8 @@ extern const char* startup5;
 // from g_game.c, prefix for savegame name like "boomsav"
 extern const char* savegamename;
 
+extern boolean deh_loaded; // sf
+
+void D_BuildBEXTables(void);
+
 #endif
