@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -34,6 +34,7 @@
 #pragma implementation "doomstat.h"
 #endif
 #include "doomstat.h"
+#include "c_runcmd.h"
 
 // Game Mode - identify IWAD as shareware, retail etc.
 GameMode_t gamemode = indetermined;
@@ -48,17 +49,25 @@ boolean modifiedgame;
 //-----------------------------------------------------------------------------
 
 // CPhipps - compatibility vars
-complevel_t compatibility_level, default_compatibility_level;
+complevel_t default_compatibility_level = -1;
+complevel_t compatibility_level = -1;
 
-int comp[COMP_TOTAL], default_comp[COMP_TOTAL];    // killough 10/98
+int comp[COMP_TOTAL];
+int default_comp[COMP_TOTAL] = {
+  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0 ,0,
+  0, 0, 0, 0, 0, 0, 0, 0, 0 ,0,
+  0, 0
+};    // killough 10/98
 
 // v1.1-like pitched sounds
-int pitched_sounds;        // killough
+int pitched_sounds = 0;        // killough
 
-int     default_translucency; // config file says           // phares
+int     default_translucency = 1; // config file says           // phares
 boolean general_translucency; // true if translucency is ok // phares
 
-int demo_insurance, default_demo_insurance;        // killough 1/16/98
+int demo_insurance;
+int default_demo_insurance = 2;        // killough 1/16/98
 
 int  allow_pushers = 1;      // MT_PUSH Things              // phares 3/10/98
 int  default_allow_pushers;  // killough 3/1/98: make local to each game
@@ -67,40 +76,44 @@ int  variable_friction = 1;      // ice & mud               // phares 3/10/98
 int  default_variable_friction;  // killough 3/1/98: make local to each game
 
 int  weapon_recoil;              // weapon recoil                   // phares
-int  default_weapon_recoil;      // killough 3/1/98: make local to each game
+int  default_weapon_recoil = 1;  // killough 3/1/98: make local to each game
 
 int player_bobbing;  // whether player bobs or not          // phares 2/25/98
-int default_player_bobbing;  // killough 3/1/98: make local to each game
+int default_player_bobbing = 1;  // killough 3/1/98: make local to each game
 
 int monsters_remember;          // killough 3/1/98
-int default_monsters_remember;
+int default_monsters_remember = 1;
 
 int monster_infighting=1;       // killough 7/19/98: monster<=>monster attacks
 int default_monster_infighting=1;
 
-int monster_friction=1;       // killough 10/98: monsters affected by friction
+int monster_friction=1;       // killough 10/98: monsters affected by friction 
 int default_monster_friction=1;
 
 #ifdef DOGS
-int dogs, default_dogs;         // killough 7/19/98: Marine's best friend :)
-int dog_jumping, default_dog_jumping;   // killough 10/98
+int dogs, default_dogs = 0;         // killough 7/19/98: Marine's best friend :)
+int dog_jumping, default_dog_jumping = 1;   // killough 10/98
 #endif
 
 // killough 8/8/98: distance friends tend to move towards players
 int distfriend = 128, default_distfriend = 128;
 
 // killough 9/8/98: whether monsters are allowed to strafe or retreat
-int monster_backing, default_monster_backing;
+int monster_backing;
+int default_monster_backing = 1;
 
 // killough 9/9/98: whether monsters are able to avoid hazards (e.g. crushers)
-int monster_avoid_hazards, default_monster_avoid_hazards;
+int monster_avoid_hazards;
+int default_monster_avoid_hazards = 1;
 
 // killough 9/9/98: whether monsters help friends
-int help_friends, default_help_friends;
+int help_friends;
+int default_help_friends = 1;
 
-int flashing_hom;     // killough 10/98
+int flashing_hom = 0;     // killough 10/98
 
-int doom_weapon_toggles; // killough 10/98
+int doom_weapon_toggles = 1; // killough 10/98
 
-int monkeys, default_monkeys;
+int monkeys;
+int default_monkeys = 1;
 
