@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -37,6 +37,7 @@
 #include "s_sound.h"
 #include "sounds.h"
 #include "d_deh.h"  // Ty 03/22/98 - externalizations
+#include "p_info.h"
 #include "f_finale.h" // CPhipps - hmm...
 
 // Stage of animation:
@@ -89,8 +90,8 @@ void F_StartFinale (void)
     case registered:
     case retail:
     {
-      S_ChangeMusic(mus_victor, true);
-
+      S_ChangeMusicNum(mus_victor, true);
+      
       switch (gameepisode)
       {
         case 1:
@@ -99,7 +100,7 @@ void F_StartFinale (void)
              break;
         case 2:
              finaleflat = bgflatE2;
-             finaletext = s_E2TEXT; // Ty 03/23/98 - Same stuff for each
+             finaletext = s_E2TEXT; // Ty 03/23/98 - Same stuff for each 
              break;
         case 3:
              finaleflat = bgflatE3;
@@ -115,11 +116,11 @@ void F_StartFinale (void)
       }
       break;
     }
-
+    
     // DOOM II and missions packs with E1, M34
     case commercial:
     {
-      S_ChangeMusic(mus_read_m, true);
+      S_ChangeMusicNum(mus_read_m, true);
 
       // Ty 08/27/98 - added the gamemission logic
       switch (gamemap)
@@ -160,15 +161,21 @@ void F_StartFinale (void)
       }
       break;
       // Ty 08/27/98 - end gamemission logic
-    }
+    } 
 
     // Indeterminate.
     default:  // Ty 03/30/98 - not externalized
-         S_ChangeMusic(mus_read_m, true);
+         S_ChangeMusicNum(mus_read_m, true);
          finaleflat = "F_SKY1"; // Not used anywhere else.
          finaletext = s_C1TEXT;  // FIXME - other text, music?
          break;
   }
+  
+  if(info_intertext)
+  {
+        finaleflat = info_backdrop ? info_backdrop : "F_SKY1";
+        finaletext = info_intertext;
+  }
 
   finalestage = 0;
   finalecount = 0;
@@ -180,7 +187,7 @@ boolean F_Responder (event_t *event)
 {
   if (finalestage == 2)
     return F_CastResponder (event);
-
+        
   return false;
 }
 
@@ -189,7 +196,7 @@ boolean F_Responder (event_t *event)
 
 static float Get_TextSpeed(void)
 {
-  return midstage ? NEWTEXTSPEED : (midstage=acceleratestage) ?
+  return midstage ? NEWTEXTSPEED : (midstage=acceleratestage) ? 
     acceleratestage=0, NEWTEXTSPEED : TEXTSPEED;
 }
 
@@ -220,7 +227,7 @@ void F_Ticker(void)
 
   // advance animation
   finalecount++;
-
+ 
   if (finalestage == 2)
     F_CastTicker();
 
@@ -230,7 +237,7 @@ void F_Ticker(void)
       /* killough 2/28/98: changed to allow acceleration */
       if (finalecount > strlen(finaletext)*speed +
           (midstage ? NEWTEXTWAIT : TEXTWAIT) ||
-          (midstage && acceleratestage)) {
+          (midstage && acceleratestage)) {       
         if (gamemode != commercial)       // Doom 1 / Ultimate Doom episode end
           {                               // with enough time, it's automatic
             finalecount = 0;
@@ -263,49 +270,19 @@ void F_Ticker(void)
 // text can be increased, and there's still time to read what's     //   |
 // written.                                                         // phares
 // CPhipps - reformatted
-
-#include "hu_stuff.h"
-extern patchnum_t hu_font[HU_FONTSIZE];
-
+// proff - using V_WriteTextXYGap
 
 void F_TextWrite (void)
 {
+  char *temp;
+  int count = (int)((float)(finalecount - 10)/Get_TextSpeed()); // phares
+
   V_DrawBackground(finaleflat, 0);
-  { // draw some of the text onto the screen
-    int         cx = 10;
-    int         cy = 10;
-    const char* ch = finaletext; // CPhipps - const
-    int         count = (int)((float)(finalecount - 10)/Get_TextSpeed()); // phares
-    int         w;
-
-    if (count < 0)
-      count = 0;
-
-    for ( ; count ; count-- ) {
-      int       c = *ch++;
-
-      if (!c)
-  break;
-      if (c == '\n') {
-  cx = 10;
-  cy += 11;
-  continue;
-      }
-
-      c = toupper(c) - HU_FONTSTART;
-      if (c < 0 || c> HU_FONTSIZE) {
-  cx += 4;
-  continue;
-      }
-
-      w = SHORT (hu_font[c].width);
-      if (cx+w > SCREENWIDTH)
-  break;
-      // CPhipps - patch drawing updated
-      V_DrawNumPatch(cx, cy, 0, hu_font[c].lumpnum, CR_DEFAULT, VPT_STRETCH);
-      cx+=w;
-    }
-  }
+  temp = strdup(finaletext);
+  if (count < 0)
+    count = 0;
+  temp[count] = 0;
+  V_WriteTextXYGap(temp, 10, 10, 0, 3);
 }
 
 //
@@ -338,7 +315,7 @@ static const castinfo_t castorder[] = { // CPhipps - static const, initialised h
   { &s_CC_SPIDER,  MT_SPIDER },
   { &s_CC_CYBER,   MT_CYBORG },
   { &s_CC_HERO,    MT_PLAYER },
-  { NULL,         0}
+  { NULL,         0} 
   };
 
 int             castnum;
@@ -362,11 +339,11 @@ void F_StartCast (void)
   caststate = &states[mobjinfo[castorder[castnum].type].seestate];
   casttics = caststate->tics;
   castdeath = false;
-  finalestage = 2;
+  finalestage = 2;    
   castframes = 0;
   castonmelee = 0;
   castattacking = false;
-  S_ChangeMusic(mus_evil, true);
+  S_ChangeMusicNum(mus_evil, true);
 }
 
 
@@ -377,10 +354,10 @@ void F_CastTicker (void)
 {
   int st;
   int sfx;
-
+      
   if (--casttics > 0)
     return;                 // not time to change state yet
-
+              
   if (caststate->tics == -1 || caststate->nextstate == S_NULL)
   {
     // switch from deathstate to next monster
@@ -401,7 +378,7 @@ void F_CastTicker (void)
     st = caststate->nextstate;
     caststate = &states[st];
     castframes++;
-
+      
     // sound hacks....
     switch (st)
     {
@@ -433,11 +410,11 @@ void F_CastTicker (void)
       case S_PAIN_ATK3:     sfx = sfx_sklatk; break;
       default: sfx = 0; break;
     }
-
+            
     if (sfx)
       S_StartSound (NULL, sfx);
   }
-
+      
   if (castframes == 12)
   {
     // go into attack frame
@@ -457,7 +434,7 @@ void F_CastTicker (void)
           &states[mobjinfo[castorder[castnum].type].missilestate];
     }
   }
-
+      
   if (castattacking)
   {
     if (castframes == 24
@@ -469,7 +446,7 @@ void F_CastTicker (void)
       caststate = &states[mobjinfo[castorder[castnum].type].seestate];
     }
   }
-
+      
   casttics = caststate->tics;
   if (casttics == -1)
       casttics = 15;
@@ -484,10 +461,10 @@ boolean F_CastResponder (event_t* ev)
 {
   if (ev->type != ev_keydown)
     return false;
-
+                
   if (castdeath)
     return true;                    // already in dying frames
-
+                
   // go into death frame
   castdeath = true;
   caststate = &states[mobjinfo[castorder[castnum].type].deathstate];
@@ -496,59 +473,14 @@ boolean F_CastResponder (event_t* ev)
   castattacking = false;
   if (mobjinfo[castorder[castnum].type].deathsound)
     S_StartSound (NULL, mobjinfo[castorder[castnum].type].deathsound);
-
+        
   return true;
 }
 
 
 static void F_CastPrint (const char* text) // CPhipps - static, const char*
 {
-  const char* ch; // CPhipps - const
-  int         c;
-  int         cx;
-  int         w;
-  int         width;
-
-  // find width
-  ch = text;
-  width = 0;
-
-  while (ch)
-  {
-    c = *ch++;
-    if (!c)
-      break;
-    c = toupper(c) - HU_FONTSTART;
-    if (c < 0 || c> HU_FONTSIZE)
-    {
-      width += 4;
-      continue;
-    }
-
-    w = SHORT (hu_font[c].width);
-    width += w;
-  }
-
-  // draw it
-  cx = 160-width/2;
-  ch = text;
-  while (ch)
-  {
-    c = *ch++;
-    if (!c)
-      break;
-    c = toupper(c) - HU_FONTSTART;
-    if (c < 0 || c> HU_FONTSIZE)
-    {
-      cx += 4;
-      continue;
-    }
-
-    w = SHORT (hu_font[c].width);
-    // CPhipps - patch drawing updated
-    V_DrawNumPatch(cx, 180, 0, hu_font[c].lumpnum, CR_DEFAULT, VPT_STRETCH);
-    cx+=w;
-  }
+  V_WriteText(text, 160-V_StringWidth(text, 0)/2, 180, 0);
 }
 
 
@@ -562,13 +494,13 @@ void F_CastDrawer (void)
   spriteframe_t*      sprframe;
   int                 lump;
   boolean             flip;
-
+    
   // erase the entire screen to a background
   // CPhipps - patch drawing updated
   V_DrawNamePatch(0,0,0, bgcastcall, CR_DEFAULT, VPT_STRETCH); // Ty 03/30/98 bg texture extern
 
   F_CastPrint (*(castorder[castnum].name));
-
+    
   // draw the current frame in the middle of the screen
   sprdef = &sprites[caststate->sprite];
   sprframe = &sprdef->spriteframes[ caststate->frame & FF_FRAMEMASK];
@@ -576,8 +508,8 @@ void F_CastDrawer (void)
   flip = (boolean)sprframe->flip[0];
 
   // CPhipps - patch drawing updated
-  V_DrawNumPatch(160, 170, 0, lump+firstspritelump, CR_DEFAULT,
-     VPT_STRETCH | (flip ? VPT_FLIP : 0));
+  V_DrawNumPatch(160, 170, 0, lump+firstspritelump, CR_DEFAULT, 
+		 VPT_STRETCH | (flip ? VPT_FLIP : 0));
 }
 
 //
@@ -592,7 +524,6 @@ static void F_BunnyScroll (void)
   int         stage;
   static int  laststage;
 
-  V_MarkRect (0, 0, SCREENWIDTH, SCREENHEIGHT);
   {
     int scrolled = 320 - (finalecount-230)/2;
     if (scrolled <= 0) {
@@ -600,21 +531,9 @@ static void F_BunnyScroll (void)
     } else if (scrolled >= 320) {
       V_DrawNamePatch(0, 0, 0, pfub1, CR_DEFAULT, VPT_STRETCH);
     } else {
-#define SCRN 2
-
-#ifdef GL_DOOM
-      V_DrawNamePatch(320-scrolled, 0, SCRN, pfub1, CR_DEFAULT, VPT_STRETCH);
-      V_DrawNamePatch(-scrolled, 0, SCRN, pfub2, CR_DEFAULT, VPT_STRETCH);
-#else
       int realscrolled = (SCREENWIDTH * scrolled) / 320;
-
-      V_AllocScreen(SCRN);
-      V_DrawNamePatch(0, 0, SCRN, pfub2, CR_DEFAULT, VPT_STRETCH);
-      V_CopyRect(realscrolled, 0, SCRN, SCREENWIDTH-realscrolled, SCREENHEIGHT, 0, 0, 0, VPT_NONE);
-      V_DrawNamePatch(0, 0, SCRN, pfub1, CR_DEFAULT, VPT_STRETCH);
-      V_CopyRect(0, 0, SCRN, realscrolled, SCREENHEIGHT, SCREENWIDTH-realscrolled, 0, 0, VPT_NONE);
-      V_FreeScreen(SCRN);
-#endif
+      V_DrawNamePatch(320-scrolled, 0, 0, pfub1, CR_DEFAULT, VPT_STRETCH);
+      V_DrawNamePatch(-scrolled, 0, 0, pfub2, CR_DEFAULT, VPT_STRETCH);
     }
   }
 
@@ -627,7 +546,7 @@ static void F_BunnyScroll (void)
     laststage = 0;
     return;
   }
-
+      
   stage = (finalecount-1180) / 5;
   if (stage > 6)
     stage = 6;
@@ -636,7 +555,7 @@ static void F_BunnyScroll (void)
     S_StartSound (NULL, sfx_pistol);
     laststage = stage;
   }
-
+      
   sprintf (name,"END%i",stage);
   // CPhipps - patch drawing updated
   V_DrawNamePatch((320-13*8)/2, (200-8*8)/2, 0, name, CR_DEFAULT, VPT_STRETCH);
