@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -6,9 +6,9 @@
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2001 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -36,7 +36,6 @@
 #pragma interface
 #endif
 
-extern int ms_to_next_tick;
 int I_GetTime_RealTime(void);     /* killough */
 
 unsigned long I_GetRandomTimeSeed(void); /* cphipps */
@@ -44,7 +43,7 @@ unsigned long I_GetRandomTimeSeed(void); /* cphipps */
 void I_uSleep(unsigned long usecs);
 
 /* cphipps - I_GetVersionString
- * Returns a version string in the given buffer
+ * Returns a version string in the given buffer 
  */
 const char* I_GetVersionString(char* buf, size_t sz);
 
@@ -53,6 +52,12 @@ const char* I_GetVersionString(char* buf, size_t sz);
  */
 const char* I_SigString(char* buf, size_t sz, int signum);
 
+/* cph 2001/11/18 - wrapper for read(2) which deals with partial reads */
+void I_Read(int fd, void* buf, size_t sz);
+
+/* cph 2001/11/18 - Move W_Filelength to i_system.c */
+int I_Filelength(int handle);
+
 char *I_DoomExeDir(void); // killough 2/16/98: path to executable's dir
 
 char* I_FindFile(const char* wfname, const char* ext);
