@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -50,11 +50,12 @@
 #include "doomtype.h"
 #include "lprintf.h"
 #include "i_main.h"
+#include "c_io.h"
 
 int cons_error_mask = -1-LO_INFO; /* all but LO_INFO when redir'd */
 int cons_output_mask = -1;        /* all output enabled */
 
-/* cphipps - enlarged message buffer and made non-static
+/* cphipps - enlarged message buffer and made non-static 
  * We still have to be careful here, this function can be called after exit
  */
 #define MAX_MESSAGE_SIZE 2048
@@ -69,7 +70,6 @@ char szConName[] = "PrBoomConWinClass";
 char Lines[(80+2)*25+1];
 char *Last = NULL;
 boolean console_inited=FALSE;
-static boolean should_exit = 0;
 
 static CALLBACK ConWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
 {
@@ -77,127 +77,113 @@ static CALLBACK ConWndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
   HDC dc;
 
   switch (iMsg) {
-  case WM_KEYDOWN:
-    if (wParam == VK_ESCAPE)
-      should_exit = 1;
-    break;
   case WM_CLOSE:
     return 1;
     break;
   case WM_PAINT:
-    if (dc = BeginPaint (con_hWnd, &paint))
+	  if (dc = BeginPaint (con_hWnd, &paint))
     {
-      if (Last)
+		  if (Last)
       {
-        char *row;
-        int line, last;
+			  char *row;
+			  int line, last;
 
-        line = paint.rcPaint.top / OemHeight;
-        last = paint.rcPaint.bottom / OemHeight;
-        for (row = Lines + (line*(80+2)); line < last; line++)
+			  line = paint.rcPaint.top / OemHeight;
+			  last = paint.rcPaint.bottom / OemHeight;
+			  for (row = Lines + (line*(80+2)); line < last; line++)
         {
-          if (row[1]>0)
-            TextOut (dc, 0, line * OemHeight, &row[2], row[1]);
-          row += 80 + 2;
-        }
-      }
-      EndPaint (con_hWnd, &paint);
-    }
+				  if (row[1]>0)
+				    TextOut (dc, 0, line * OemHeight, &row[2], row[1]);
+				  row += 80 + 2;
+			  }
+		  }
+		  EndPaint (con_hWnd, &paint);
+	  }
     return 0;
     break;
   default:
-    break;
+    return(DefWindowProc(hwnd,iMsg,wParam,lParam));
   }
-  return(DefWindowProc(hwnd,iMsg,wParam,lParam));
 }
 
 static void I_PrintStr (int xp, const char *cp, int count, BOOL scroll) {
-  RECT rect;
+	RECT rect;
   HDC conDC;
 
   if ((!con_hWnd) || (!console_inited))
     return;
-  if (count)
+	if (count)
   {
     conDC=GetDC(con_hWnd);
-    TextOut (conDC, xp * OemWidth, ConHeight - OemHeight, cp, count);
+		TextOut (conDC, xp * OemWidth, ConHeight - OemHeight, cp, count);
     ReleaseDC(con_hWnd,conDC);
   }
-  if (scroll) {
-    rect.left = 0;
-    rect.top = 0;
-    rect.right = ConWidth;
-    rect.bottom = ConHeight;
-    ScrollWindowEx (con_hWnd, 0, -OemHeight, NULL, &rect, NULL, NULL, SW_ERASE|SW_INVALIDATE);
-    UpdateWindow (con_hWnd);
-  }
+	if (scroll) {
+		rect.left = 0;
+		rect.top = 0;
+		rect.right = ConWidth;
+		rect.bottom = ConHeight;
+		ScrollWindowEx (con_hWnd, 0, -OemHeight, NULL, &rect, NULL, NULL, SW_ERASE|SW_INVALIDATE);
+		UpdateWindow (con_hWnd);
+	}
 }
 
 static int I_ConPrintString (const char *outline)
 {
-  const char *cp, *newcp;
-  static int xp = 0;
-  int newxp;
-  BOOL scroll;
+	const char *cp, *newcp;
+	static int xp = 0;
+	int newxp;
+	BOOL scroll;
 
   if (!console_inited)
     return 0;
-  cp = outline;
-  while (*cp) {
-    for (newcp = cp, newxp = xp;
-      *newcp != '\n' && *newcp != '\0' && newxp < 80;
-       newcp++) {
-      if (*newcp == '\x08') {
-        newxp--;
-        break;
-      }
-      else if (*newcp == '\t') {
-        newxp = ((newxp + 8) / 8) * 8;
-        break;
-      }
-      else
-        newxp++;
-    }
+	cp = outline;
+	while (*cp) {
+		for (newcp = cp, newxp = xp;
+			*newcp != '\n' && *newcp != '\0' && newxp < 80;
+			 newcp++, newxp++) {
+			if (*newcp == '\x8') {
+				if (xp) xp--;
+				newxp = xp;
+				cp++;
+			}
+		}
 
-    if (*cp) {
-      const char *poop;
-      int x;
+		if (*cp) {
+			const char *poop;
+			int x;
 
-      for (x = xp, poop = cp; poop < newcp; poop++, x++) {
+			for (x = xp, poop = cp; poop < newcp; poop++, x++) {
         Last[x+2] = ((*poop) < 32) ? 32 : (*poop);
-      }
+			}
 
-      if (*newcp == '\t')
-        for (x = xp; x < newxp; x++)
-          Last[x+2] = ' ';
+			if (Last[1] < xp + (newcp - cp))
+				Last[1] = xp + (newcp - cp);
 
-      if (Last[1] < xp + (newcp - cp))
-        Last[1] = xp + (newcp - cp);
+			if (*newcp == '\n' || xp == 80) {
+				if (*newcp != '\n') {
+					Last[0] = 1;
+				}
+				memmove (Lines, Lines + (80 + 2), (80 + 2) * (25 - 1));
+				Last[0] = 0;
+				Last[1] = 0;
+				newxp = 0;
+				scroll = TRUE;
+			} else {
+				scroll = FALSE;
+			}
+			I_PrintStr (xp, cp, newcp - cp, scroll);
 
-      if (*newcp == '\n' || xp == 80) {
-        if (*newcp != '\n') {
-          Last[0] = 1;
-        }
-        memmove (Lines, Lines + (80 + 2), (80 + 2) * (25 - 1));
-        Last[0] = 0;
-        Last[1] = 0;
-        newxp = 0;
-        scroll = TRUE;
-      } else {
-        scroll = FALSE;
-      }
-      I_PrintStr (xp, cp, newcp - cp, scroll);
+			xp = newxp;
 
-      xp = newxp;
+			if (*newcp == '\n')
+				cp = newcp + 1;
+			else
+				cp = newcp;
+		}
+	}
 
-      if ((*newcp == '\n') || (*newcp == '\x08') || (*newcp == '\t'))
-        cp = newcp + 1;
-      else
-        cp = newcp;
-    }
-  }
-
-  return strlen (outline);
+	return strlen (outline);
 }
 
 void I_ConTextAttr(unsigned char a)
@@ -221,28 +207,14 @@ void I_ConTextAttr(unsigned char a)
   if (a & BACKGROUND_RED) r=col;
   if (a & BACKGROUND_GREEN) g=col;
   if (a & BACKGROUND_BLUE) b=col;
-  SetBkColor(conDC, PALETTERGB(r,g,b));
+ 	SetBkColor(conDC, PALETTERGB(r,g,b));
   ReleaseDC(con_hWnd,conDC);
 }
 
-void I_UpdateConsole(void)
-{
-  MSG msg;
-
-  UpdateWindow(con_hWnd);
-  while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) > 0)
-  {
-    TranslateMessage(&msg);
-    DispatchMessage(&msg);
-  }
-  if (should_exit)
-    exit(0);
-}
-
 static void Init_Console(void)
 {
   memset(Lines,0,25*(80+2)+1);
-  Last = Lines + (25 - 1) * (80 + 2);
+	Last = Lines + (25 - 1) * (80 + 2);
   console_inited=TRUE;
 }
 
@@ -250,8 +222,8 @@ int Init_ConsoleWin(void)
 {
     HDC conDC;
     WNDCLASS wndclass;
-    TEXTMETRIC metrics;
-    RECT cRect;
+	  TEXTMETRIC metrics;
+	  RECT cRect;
     int width,height;
     int scr_width,scr_height;
     HINSTANCE hInstance;
@@ -288,23 +260,23 @@ int Init_ConsoleWin(void)
              NULL, NULL, hInstance, NULL);
     conDC=GetDC(con_hWnd);
     OemFont = GetStockObject(OEM_FIXED_FONT);
-    SelectObject(conDC, OemFont);
-    GetTextMetrics(conDC, &metrics);
-    OemWidth = metrics.tmAveCharWidth;
-    OemHeight = metrics.tmHeight;
-    GetClientRect(con_hWnd, &cRect);
+	  SelectObject(conDC, OemFont);
+	  GetTextMetrics(conDC, &metrics);
+	  OemWidth = metrics.tmAveCharWidth;
+	  OemHeight = metrics.tmHeight;
+	  GetClientRect(con_hWnd, &cRect);
     width += (OemWidth * 80) - cRect.right;
     height += (OemHeight * 25) - cRect.bottom;
     // proff 11/09/98: Added code for centering console
     scr_width = GetSystemMetrics(SM_CXFULLSCREEN);
     scr_height = GetSystemMetrics(SM_CYFULLSCREEN);
     MoveWindow(con_hWnd, (scr_width-width)/2, (scr_height-height)/2, width, height, TRUE);
-    GetClientRect(con_hWnd, &cRect);
-    ConWidth = cRect.right;
-    ConHeight = cRect.bottom;
+	  GetClientRect(con_hWnd, &cRect);
+	  ConWidth = cRect.right;
+	  ConHeight = cRect.bottom;
     SetTextColor(conDC, RGB(192,192,192));
-    SetBkColor(conDC, RGB(0,0,0));
-    SetBkMode(conDC, OPAQUE);
+  	SetBkColor(conDC, RGB(0,0,0));
+		SetBkMode(conDC, OPAQUE);
     ReleaseDC(con_hWnd,conDC);
     ShowWindow(con_hWnd, SW_SHOW);
     UpdateWindow(con_hWnd);
@@ -330,11 +302,12 @@ int lprintf(OutputLevels pri, const char *s, ...)
   va_start(v,s);
 #ifdef HAVE_VSNPRINTF
   vsnprintf(msg,sizeof(msg),s,v);         /* print message in buffer  */
-#else
+#else 
   vsprintf(msg,s,v);
 #endif
   va_end(v);
 
+#ifndef DREAMCAST
   if (lvl&cons_output_mask)               /* mask output as specified */
   {
     r=fprintf(stdout,"%s",msg);
@@ -344,7 +317,10 @@ int lprintf(OutputLevels pri, const char *s, ...)
   }
   if (!isatty(1) && lvl&cons_error_mask)  /* if stdout redirected     */
     r=fprintf(stderr,"%s",msg);           /* select output at console */
-
+#else // DREAMCAST
+  r=printf("%s",msg);
+#endif // DREAMCAST
+  C_Printf("%s",msg);
   return r;
 }
 
@@ -352,7 +328,7 @@ int lprintf(OutputLevels pri, const char *s, ...)
  * I_Error
  *
  * cphipps - moved out of i_* headers, to minimise source files that depend on
- * the low-level headers. All this does is print the error, then call the
+ * the low-level headers. All this does is print the error, then call the 
  * low-level safe exit function.
  * killough 3/20/98: add const
  */
@@ -368,12 +344,17 @@ void I_Error(const char *error, ...)
   vsprintf(errmsg,error,argptr);
 #endif
   va_end(argptr);
+#ifndef DREAMCAST
   fprintf(stderr,"%s\n",errmsg);
 #ifdef _MSC_VER
   {
-    Init_ConsoleWin();
+    //Init_ConsoleWin();
     MessageBox(con_hWnd,errmsg,"PrBoom",MB_OK | MB_TASKMODAL | MB_TOPMOST);
   }
 #endif
+#else // DREAMCAST
+  printf("%s",errmsg);
+#endif // DREAMCAST
+
   I_SafeExit(-1);
 }
