@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -6,9 +6,9 @@
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2002 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -30,19 +30,20 @@
  *-----------------------------------------------------------------------------*/
 
 #include "doomstat.h"
+#include "c_runcmd.h"
 #include "g_game.h"
 #include "r_data.h"
 #include "p_inter.h"
+#include "p_tick.h"
 #include "m_cheat.h"
 #include "m_argv.h"
 #include "s_sound.h"
 #include "sounds.h"
 #include "dstrings.h"
 #include "r_main.h"
-#include "p_map.h"
 #include "d_deh.h"  // Ty 03/27/98 - externalized strings
 
-#define plyr (players+consoleplayer)     /* the console player */
+#define plyr (&players[consoleplayer])     /* the console player */
 
 //-----------------------------------------------------------------------------
 //
@@ -142,7 +143,7 @@ struct cheat_s cheat[] = {
   {"idbeholds",  "Berserk",           not_net | not_demo,
    cheat_pw,  pw_strength        },
 
-  {"idbeholdi",  "Invisibility",      not_net | not_demo,
+  {"idbeholdi",  "Invisibility",      not_net | not_demo,  
    cheat_pw,  pw_invisibility    },
 
   {"idbeholdr",  "Radiation Suit",    not_net | not_demo,
@@ -178,7 +179,7 @@ struct cheat_s cheat[] = {
   {"tnthom",     NULL,                always,
    cheat_hom      },     // killough 2/07/98: HOM autodetector
 
-  {"tntkey",     NULL,                not_net | not_demo,
+  {"tntkey",     NULL,                not_net | not_demo, 
    cheat_tntkey   },     // killough 2/16/98: generalized key cheats
 
   {"tntkeyr",    NULL,                not_net | not_demo,
@@ -190,13 +191,13 @@ struct cheat_s cheat[] = {
   {"tntkeyb",    NULL,                not_net | not_demo,
    cheat_tntkeyx  },
 
-  {"tntkeyrc",   NULL,                not_net | not_demo,
+  {"tntkeyrc",   NULL,                not_net | not_demo, 
    cheat_tntkeyxx, it_redcard    },
 
   {"tntkeyyc",   NULL,                not_net | not_demo,
    cheat_tntkeyxx, it_yellowcard },
 
-  {"tntkeybc",   NULL,                not_net | not_demo,
+  {"tntkeybc",   NULL,                not_net | not_demo, 
    cheat_tntkeyxx, it_bluecard   },
 
   {"tntkeyrs",   NULL,                not_net | not_demo,
@@ -233,7 +234,7 @@ struct cheat_s cheat[] = {
    cheat_pitch},         // killough 2/21/98: pitched sound toggle
 
   // killough 2/21/98: reduce RSI injury by adding simpler alias sequences:
-  {"tntran",     NULL,                always,
+  {"tntran",     NULL,                always, 
    cheat_tnttran    },   // killough 2/21/98: same as tnttran
 
   {"tntamo",     NULL,                not_net | not_demo,
@@ -248,7 +249,7 @@ struct cheat_s cheat[] = {
   {"tntice",     NULL,                not_net | not_demo,
    cheat_friction   },   // phares 3/10/98: toggle variable friction effects
 
-  {"tntpush",    NULL,                not_net | not_demo,
+  {"tntpush",    NULL,                not_net | not_demo, 
    cheat_pushers    },   // phares 3/10/98: toggle pushers
 
   {NULL}                 // end-of-list marker
@@ -260,38 +261,38 @@ static void cheat_mus(buf)
 char buf[3];
 {
   int musnum;
-
+  
   //jff 3/20/98 note: this cheat allowed in netgame/demorecord
 
   //jff 3/17/98 avoid musnum being negative and crashing
   if (!isdigit(buf[0]) || !isdigit(buf[1]))
     return;
 
-  plyr->message = s_STSTR_MUS; // Ty 03/27/98 - externalized
-
+  doom_printf(s_STSTR_MUS); // Ty 03/27/98 - externalized
+  
   if (gamemode == commercial)
     {
       musnum = mus_runnin + (buf[0]-'0')*10 + buf[1]-'0' - 1;
-
+          
       //jff 4/11/98 prevent IDMUS00 in DOOMII and IDMUS36 or greater
       if (musnum < mus_runnin ||  ((buf[0]-'0')*10 + buf[1]-'0') > 35)
-        plyr->message = s_STSTR_NOMUS; // Ty 03/27/98 - externalized
+        doom_printf(s_STSTR_NOMUS); // Ty 03/27/98 - externalized
       else
         {
-          S_ChangeMusic(musnum, 1);
+          S_ChangeMusicNum(musnum, 1);
           idmusnum = musnum; //jff 3/17/98 remember idmus number for restore
         }
     }
   else
     {
       musnum = mus_e1m1 + (buf[0]-'1')*9 + (buf[1]-'1');
-
+          
       //jff 4/11/98 prevent IDMUS0x IDMUSx0 in DOOMI and greater than introa
       if (buf[0] < '1' || buf[1] < '1' || ((buf[0]-'1')*9 + buf[1]-'1') > 31)
-        plyr->message = s_STSTR_NOMUS; // Ty 03/27/98 - externalized
+        doom_printf(s_STSTR_NOMUS); // Ty 03/27/98 - externalized
       else
         {
-          S_ChangeMusic(musnum, 1);
+          S_ChangeMusicNum(musnum, 1);
           idmusnum = musnum; //jff 3/17/98 remember idmus number for restore
         }
     }
@@ -302,7 +303,7 @@ static void cheat_choppers()
 {
   plyr->weaponowned[wp_chainsaw] = true;
   plyr->powers[pw_invulnerability] = true;
-  plyr->message = s_STSTR_CHOPPERS; // Ty 03/27/98 - externalized
+  doom_printf(s_STSTR_CHOPPERS); // Ty 03/27/98 - externalized
 }
 
 static void cheat_god()
@@ -312,12 +313,12 @@ static void cheat_god()
     {
       if (plyr->mo)
         plyr->mo->health = god_health;  // Ty 03/09/98 - deh
-
+          
       plyr->health = god_health;
-      plyr->message = s_STSTR_DQDON; // Ty 03/27/98 - externalized
+      doom_printf(s_STSTR_DQDON); // Ty 03/27/98 - externalized
     }
-  else
-    plyr->message = s_STSTR_DQDOFF; // Ty 03/27/98 - externalized
+  else 
+    doom_printf(s_STSTR_DQDOFF); // Ty 03/27/98 - externalized
 }
 
 // CPhipps - new health and armour cheat codes
@@ -327,7 +328,7 @@ static void cheat_health()
     if (plyr->mo)
       plyr->mo->health = mega_health;
     plyr->health = mega_health;
-    plyr->message = s_STSTR_BEHOLDX; // Ty 03/27/98 - externalized
+    doom_printf(s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
   }
 }
 
@@ -335,7 +336,7 @@ static void cheat_megaarmour()
 {
   plyr->armorpoints = idfa_armor;      // Ty 03/09/98 - deh
   plyr->armortype = idfa_armor_class;  // Ty 03/09/98 - deh
-  plyr->message = s_STSTR_BEHOLDX; // Ty 03/27/98 - externalized
+  doom_printf(s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
 }
 
 static void cheat_fa()
@@ -351,18 +352,18 @@ static void cheat_fa()
 
   plyr->armorpoints = idfa_armor;      // Ty 03/09/98 - deh
   plyr->armortype = idfa_armor_class;  // Ty 03/09/98 - deh
-
+        
   // You can't own weapons that aren't in the game // phares 02/27/98
   for (i=0;i<NUMWEAPONS;i++)
     if (!(((i == wp_plasma || i == wp_bfg) && gamemode == shareware) ||
           (i == wp_supershotgun && gamemode != commercial)))
       plyr->weaponowned[i] = true;
-
+        
   for (i=0;i<NUMAMMO;i++)
     if (i!=am_cell || gamemode!=shareware)
       plyr->ammo[i] = plyr->maxammo[i];
 
-  plyr->message = s_STSTR_FAADDED;
+  doom_printf(s_STSTR_FAADDED);
 }
 
 static void cheat_k()
@@ -372,7 +373,7 @@ static void cheat_k()
     if (!plyr->cards[i])     // only print message if at least one key added
       {                      // however, caller may overwrite message anyway
         plyr->cards[i] = true;
-        plyr->message = "Keys Added";
+        doom_printf("Keys Added");
       }
 }
 
@@ -380,7 +381,7 @@ static void cheat_kfa()
 {
   cheat_k();
   cheat_fa();
-  plyr->message = STSTR_KFAADDED;
+  doom_printf(STSTR_KFAADDED);
 }
 
 static void cheat_noclip()
@@ -388,8 +389,8 @@ static void cheat_noclip()
   // Simplified, accepting both "noclip" and "idspispopd".
   // no clipping mode cheat
 
-  plyr->message = (plyr->cheats ^= CF_NOCLIP) & CF_NOCLIP ?
-    s_STSTR_NCON : s_STSTR_NCOFF; // Ty 03/27/98 - externalized
+  doom_printf((plyr->cheats ^= CF_NOCLIP) & CF_NOCLIP ? 
+    s_STSTR_NCON : s_STSTR_NCOFF); // Ty 03/27/98 - externalized
 }
 
 // 'behold?' power-up cheats (modified for infinite duration -- killough)
@@ -403,13 +404,13 @@ static void cheat_pw(pw)
       if (pw != pw_strength)
         plyr->powers[pw] = -1;      // infinite duration -- killough
     }
-  plyr->message = s_STSTR_BEHOLDX; // Ty 03/27/98 - externalized
+  doom_printf(s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
 }
 
 // 'behold' power-up menu
 static void cheat_behold()
 {
-  plyr->message = s_STSTR_BEHOLD; // Ty 03/27/98 - externalized
+  doom_printf(s_STSTR_BEHOLD); // Ty 03/27/98 - externalized 
 }
 
 // 'clev' change-level cheat
@@ -420,7 +421,7 @@ char buf[3];
 
   if (gamemode == commercial)
     {
-      epsd = 1; //jff was 0, but espd is 1-based
+      epsd = 1; //jff was 0, but espd is 1-based 
       map = (buf[0] - '0')*10 + buf[1] - '0';
     }
   else
@@ -441,16 +442,16 @@ char buf[3];
 
   idmusnum = -1; //jff 3/17/98 revert to normal level music on IDCLEV
 
-  plyr->message = s_STSTR_CLEV; // Ty 03/27/98 - externalized
+  doom_printf(s_STSTR_CLEV); // Ty 03/27/98 - externalized
 
-  G_DeferedInitNew(gameskill, epsd, map);
+  G_DeferedInitNewNum(gameskill, epsd, map);
 }
 
 // 'mypos' for player position
 // killough 2/7/98: simplified using dprintf and made output more user-friendly
 static void cheat_mypos()
 {
-  doom_printf("Position (%d,%d,%d)\tAngle %-.0f",
+  doom_printf("Position (%d,%d,%d)\tAngle %-.0f", 
           players[consoleplayer].mo->x >> FRACBITS,
           players[consoleplayer].mo->y >> FRACBITS,
           players[consoleplayer].mo->z >> FRACBITS,
@@ -469,16 +470,16 @@ static void cheat_comp()
 {
   // CPhipps - modified for new compatibility system
   compatibility_level++; compatibility_level %= MAX_COMPATIBILITY_LEVEL;
-  doom_printf("New compatibility level:\n%s",
-        comp_lev_str[compatibility_level]);
+  doom_printf("New compatibility level:\n%s", 
+	      comp_lev_str[compatibility_level]);
 }
 
 // variable friction cheat
 static void cheat_friction()
 {
-  plyr->message =                       // Ty 03/27/98 - *not* externalized
-    (variable_friction = !variable_friction) ? "Variable Friction enabled" :
-                                               "Variable Friction disabled";
+  doom_printf(                      // Ty 03/27/98 - *not* externalized
+    (variable_friction = !variable_friction) ? "Variable Friction enabled" : 
+                                               "Variable Friction disabled");
 }
 
 
@@ -486,16 +487,16 @@ static void cheat_friction()
 // phares 3/10/98
 static void cheat_pushers()
 {
-  plyr->message =                      // Ty 03/27/98 - *not* externalized
-    (allow_pushers = !allow_pushers) ? "Pushers enabled" : "Pushers disabled";
+  doom_printf(                     // Ty 03/27/98 - *not* externalized
+    (allow_pushers = !allow_pushers) ? "Pushers enabled" : "Pushers disabled");
 }
 
 // translucency cheat
 static void cheat_tnttran()
 {
-  plyr->message =                      // Ty 03/27/98 - *not* externalized
+  doom_printf(                     // Ty 03/27/98 - *not* externalized
     (general_translucency = !general_translucency) ? "Translucency enabled" :
-                                                     "Translucency disabled";
+                                                     "Translucency disabled");
 
   // killough 3/1/98, 4/11/98: cache translucency map on a demand basis
   if (general_translucency && !main_tranmap)
@@ -511,16 +512,15 @@ static void cheat_massacre()    // jff 2/01/98 kill all monsters
   // fixed lost soul bug (LSs left behind when PEs are killed)
 
   int killcount=0;
-  thinker_t *currentthinker=&thinkercap;
+  thinker_t *currentthinker = NULL;
   extern void A_PainDie(mobj_t *);
 
   // killough 7/20/98: kill friendly monsters only if no others to kill
   uint_64_t mask = MF_FRIEND;
-  P_MapStart();
   do
-    while ((currentthinker=currentthinker->next)!=&thinkercap)
+    while ((currentthinker = P_NextThinker(currentthinker,th_all)) != NULL)
     if (currentthinker->function == P_MobjThinker &&
-  !(((mobj_t *) currentthinker)->flags & mask) && // killough 7/20/98
+	!(((mobj_t *) currentthinker)->flags & mask) && // killough 7/20/98
         (((mobj_t *) currentthinker)->flags & MF_COUNTKILL ||
          ((mobj_t *) currentthinker)->type == MT_SKULL))
       { // killough 3/6/98: kill even if PE is dead
@@ -536,7 +536,6 @@ static void cheat_massacre()    // jff 2/01/98 kill all monsters
           }
       }
   while (!killcount && mask ? mask=0, 1 : 0); // killough 7/20/98
-  P_MapEnd();
   // killough 3/22/98: make more intelligent about plural
   // Ty 03/27/98 - string(s) *not* externalized
   doom_printf("%d Monster%s Killed", killcount, killcount==1 ? "" : "s");
@@ -555,41 +554,41 @@ static void cheat_ddt()
 static void cheat_hom()
 {
   extern int autodetect_hom;           // Ty 03/27/98 - *not* externalized
-  plyr->message = (autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
-    "HOM Detection Off";
+  doom_printf((autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
+    "HOM Detection Off");
 }
 
 // killough 3/6/98: -fast parameter toggle
 static void cheat_fast()
 {
-  plyr->message = (fastparm = !fastparm) ? "Fast Monsters On" :
-    "Fast Monsters Off";  // Ty 03/27/98 - *not* externalized
+  doom_printf((fastparm = !fastparm) ? "Fast Monsters On" : 
+    "Fast Monsters Off");  // Ty 03/27/98 - *not* externalized
   G_SetFastParms(fastparm); // killough 4/10/98: set -fast parameter correctly
 }
 
 // killough 2/16/98: keycard/skullkey cheat functions
 static void cheat_tntkey()
 {
-  plyr->message = "Red, Yellow, Blue";  // Ty 03/27/98 - *not* externalized
+  doom_printf("Red, Yellow, Blue");  // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_tntkeyx()
 {
-  plyr->message = "Card, Skull";        // Ty 03/27/98 - *not* externalized
+  doom_printf("Card, Skull");        // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_tntkeyxx(key)
 {
-  plyr->message = (plyr->cards[key] = !plyr->cards[key]) ?
-    "Key Added" : "Key Removed";  // Ty 03/27/98 - *not* externalized
+  doom_printf((plyr->cards[key] = !plyr->cards[key]) ? 
+    "Key Added" : "Key Removed");  // Ty 03/27/98 - *not* externalized
 }
 
 // killough 2/16/98: generalized weapon cheats
 
 static void cheat_tntweap()
 {                                   // Ty 03/27/98 - *not* externalized
-  plyr->message = gamemode==commercial ?           // killough 2/28/98
-    "Weapon number 1-9" : "Weapon number 1-8";
+  doom_printf(gamemode==commercial ?           // killough 2/28/98
+    "Weapon number 1-9" : "Weapon number 1-8");
 }
 
 static void cheat_tntweapx(buf)
@@ -606,11 +605,11 @@ char buf[3];
   else
     if (w >= 0 && w < NUMWEAPONS) {
       if ((plyr->weaponowned[w] = !plyr->weaponowned[w]))
-        plyr->message = "Weapon Added";  // Ty 03/27/98 - *not* externalized
-      else
+        doom_printf("Weapon Added");  // Ty 03/27/98 - *not* externalized
+      else 
         {
           int P_SwitchWeapon(player_t *player);
-          plyr->message = "Weapon Removed"; // Ty 03/27/98 - *not* externalized
+          doom_printf("Weapon Removed"); // Ty 03/27/98 - *not* externalized
           if (w==plyr->readyweapon)         // maybe switch if weapon removed
             plyr->pendingweapon = P_SwitchWeapon(plyr);
         }
@@ -620,7 +619,7 @@ char buf[3];
 // killough 2/16/98: generalized ammo cheats
 static void cheat_tntammo()
 {
-  plyr->message = "Ammo 1-4, Backpack";  // Ty 03/27/98 - *not* externalized
+  doom_printf("Ammo 1-4, Backpack");  // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_tntammox(buf)
@@ -629,35 +628,35 @@ char buf[1];
   int a = *buf - '1';
   if (*buf == 'b')  // Ty 03/27/98 - strings *not* externalized
     if ((plyr->backpack = !plyr->backpack))
-      for (plyr->message = "Backpack Added",   a=0 ; a<NUMAMMO ; a++)
+      for (doom_printf("Backpack Added"),   a=0 ; a<NUMAMMO ; a++)
         plyr->maxammo[a] <<= 1;
     else
-      for (plyr->message = "Backpack Removed", a=0 ; a<NUMAMMO ; a++)
+      for (doom_printf("Backpack Removed"), a=0 ; a<NUMAMMO ; a++)
         {
           if (plyr->ammo[a] > (plyr->maxammo[a] >>= 1))
             plyr->ammo[a] = plyr->maxammo[a];
         }
   else
     if (a>=0 && a<NUMAMMO)  // Ty 03/27/98 - *not* externalized
-      { // killough 5/5/98: switch plasma and rockets for now -- KLUDGE
+      { // killough 5/5/98: switch plasma and rockets for now -- KLUDGE 
         a = a==am_cell ? am_misl : a==am_misl ? am_cell : a;  // HACK
-        plyr->message = (plyr->ammo[a] = !plyr->ammo[a]) ?
-          plyr->ammo[a] = plyr->maxammo[a], "Ammo Added" : "Ammo Removed";
+        doom_printf((plyr->ammo[a] = !plyr->ammo[a]) ? 
+          plyr->ammo[a] = plyr->maxammo[a], "Ammo Added" : "Ammo Removed");
       }
 }
 
 static void cheat_smart()
 {
   extern int monsters_remember;  // Ty 03/27/98 - *not* externalized
-  plyr->message = (monsters_remember = !monsters_remember) ?
-    "Smart Monsters Enabled" : "Smart Monsters Disabled";
+  doom_printf((monsters_remember = !monsters_remember) ? 
+    "Smart Monsters Enabled" : "Smart Monsters Disabled");
 }
 
 static void cheat_pitch()
 {
   extern int pitched_sounds;  // Ty 03/27/98 - *not* externalized
-  plyr->message=(pitched_sounds = !pitched_sounds) ? "Pitch Effects Enabled" :
-    "Pitch Effects Disabled";
+  doom_printf((pitched_sounds = !pitched_sounds) ? "Pitch Effects Enabled" :
+    "Pitch Effects Disabled");
 }
 
 //-----------------------------------------------------------------------------
@@ -729,7 +728,7 @@ boolean M_FindCheats(int key)
           ret = 1;                        // responder has eaten key
         }
       else
-        if (!matchedbefore)               // allow only one cheat at a time
+        if (!matchedbefore)               // allow only one cheat at a time 
           {
             matchedbefore = ret = 1;      // responder has eaten key
             cheat[i].func(cheat[i].arg);  // call cheat handler
@@ -737,3 +736,28 @@ boolean M_FindCheats(int key)
     }
   return ret;
 }
+
+CONSOLE_COMMAND(noclip, cf_notnet|cf_level)
+{
+	cheat_noclip();
+}
+
+CONSOLE_COMMAND(god, cf_notnet|cf_level)
+{
+	cheat_god();
+}
+
+/*
+CONSOLE_NETCMD(nuke, cf_server|cf_level, netcmd_nuke)
+{
+	cheat_massacre();
+}
+*/
+
+void Cheat_AddCommands()
+{
+   C_AddCommand(god);
+   C_AddCommand(noclip);
+   //C_AddCommand(nuke);
+}
+
