@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  *-----------------------------------------------------------------------------
@@ -34,25 +34,25 @@
 
 typedef enum
 {
-  MUSDATACOR,    // MUS data corrupt
-  TOOMCHAN,      // Too many channels
-  MEMALLOC,      // Memory allocation error
-  MUSDATAMT,     // MUS file empty
-  BADMUSCTL,     // MUS event 5 or 7 found
-  BADSYSEVT,     // MUS system event not in 10-14 range
-  BADCTLCHG,     // MUS control change larger than 9
-  TRACKOVF,      // MIDI track exceeds allocation
-  BADMIDHDR,     // bad midi header detected
+  MUSDATACOR,    // MUS data corrupt 
+  TOOMCHAN,      // Too many channels 
+  MEMALLOC,      // Memory allocation error 
+  MUSDATAMT,     // MUS file empty 
+  BADMUSCTL,     // MUS event 5 or 7 found 
+  BADSYSEVT,     // MUS system event not in 10-14 range 
+  BADCTLCHG,     // MUS control change larger than 9 
+  TRACKOVF,      // MIDI track exceeds allocation 
+  BADMIDHDR,     // bad midi header detected 
 } error_code_t;
 
-// some names for integers of various sizes, all unsigned
-typedef unsigned char UBYTE;  // a one-byte int
-typedef unsigned short UWORD; // a two-byte int
+// some names for integers of various sizes, all unsigned 
+typedef unsigned char UBYTE;  // a one-byte int 
+typedef unsigned short UWORD; // a two-byte int 
 // proff: changed from unsigned int to unsigned long to avoid warning
-typedef unsigned long ULONG;   // a four-byte int (assumes int 4 bytes)
+typedef unsigned long ULONG;   // a four-byte int (assumes int 4 bytes) 
 
 #ifndef MSDOS /* proff: This is from allegro.h */
-#define MIDI_TRACKS           32
+#define MIDI_TRACKS           32       
 
 typedef struct MIDI                    /* a midi file */
 {
@@ -60,12 +60,11 @@ typedef struct MIDI                    /* a midi file */
    struct {
       unsigned char *data;             /* MIDI message stream */
       int len;                         /* length of the track data */
-   } track[MIDI_TRACKS];
+   } track[MIDI_TRACKS]; 
 } MIDI;
 #endif /* !MSDOS */
 
 extern int mmus2mid(const UBYTE *mus,MIDI *mid, UWORD division, int nocomp);
-extern void free_mididata(MIDI *mid);
 extern int MIDIToMidi(MIDI *mididata,UBYTE **mid,int *midlen);
 extern int MidiToMIDI(UBYTE *mid,MIDI *mididata);
 
