@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -6,9 +6,9 @@
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2001 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,15 +21,17 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
- * DESCRIPTION:
+ * DESCRIPTION: 
  *   Door animation code (opening/closing)
  *
  *-----------------------------------------------------------------------------*/
 
+#include "z_zone.h"
 #include "doomstat.h"
+#include "g_game.h"
 #include "p_spec.h"
 #include "p_tick.h"
 #include "s_sound.h"
@@ -47,11 +49,11 @@
 //
 // T_VerticalDoor
 //
-// Passed a door structure containing all info about the door.
+// Passed a door structure containing all info about the door. 
 // See P_SPEC.H for fields.
 // Returns nothing.
 //
-// jff 02/08/98 all cases with labels beginning with gen added to support
+// jff 02/08/98 all cases with labels beginning with gen added to support 
 // generalized line type behaviors.
 
 void T_VerticalDoor (vldoor_t* door)
@@ -61,7 +63,7 @@ void T_VerticalDoor (vldoor_t* door)
   // Is the door waiting, going up, or going down?
   switch(door->direction)
   {
-    case 0:
+    case plat_stop:
       // Door is waiting
       if (!--door->topcountdown)  // downcount and check
       {
@@ -69,24 +71,24 @@ void T_VerticalDoor (vldoor_t* door)
         {
           case blazeRaise:
           case genBlazeRaise:
-            door->direction = -1; // time to go back down
+            door->direction = plat_down; // time to go back down
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdcls);
             break;
 
           case normal:
           case genRaise:
-            door->direction = -1; // time to go back down
+            door->direction = plat_down; // time to go back down
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_dorcls);
             break;
 
           case close30ThenOpen:
           case genCdO:
-            door->direction = 1;  // time to go back up
+            door->direction = plat_up;  // time to go back up
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
             break;
 
           case genBlazeCdO:
-            door->direction = 1;  // time to go back up
+            door->direction = plat_up;  // time to go back up
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
             break;
 
@@ -103,7 +105,7 @@ void T_VerticalDoor (vldoor_t* door)
         switch(door->type)
         {
           case raiseIn5Mins:
-            door->direction = 1;  // time to raise then
+            door->direction = plat_up;  // time to raise then
             door->type = normal;  // door acts just like normal 1 DR door now
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
             break;
@@ -114,7 +116,7 @@ void T_VerticalDoor (vldoor_t* door)
       }
       break;
 
-    case -1:
+    case plat_down:
       // Door is moving down
       res = T_MovePlane
             (
@@ -144,7 +146,7 @@ void T_VerticalDoor (vldoor_t* door)
           case blazeClose:
           case genBlazeRaise:
           case genBlazeClose:
-            door->sector->ceilingdata = NULL;  //jff 2/22/98
+            door->sector->ceilingdata = NULL;  //jff 2/22/98 
             P_RemoveThinker (&door->thinker);  // unlink and free
             // killough 4/15/98: remove double-closing sound of blazing doors
             if (comp[comp_blazing])
@@ -161,13 +163,13 @@ void T_VerticalDoor (vldoor_t* door)
 
           // close then open doors start waiting
           case close30ThenOpen:
-            door->direction = 0;
+            door->direction = plat_stop;
             door->topcountdown = TICRATE*30;
             break;
 
           case genCdO:
           case genBlazeCdO:
-            door->direction = 0;
+            door->direction = plat_stop;
             door->topcountdown = door->topwait; // jff 5/8/98 insert delay
             break;
 
@@ -188,23 +190,15 @@ void T_VerticalDoor (vldoor_t* door)
           case close:          // Close types do not bounce, merely wait
             break;
 
-          case blazeRaise:
-          case genBlazeRaise:
-            door->direction = 1;
-	    if (!comp[comp_blazing]) {
-	      S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
-	      break;
-	    }
-
           default:             // other types bounce off the obstruction
-            door->direction = 1;
+              door->direction = plat_up;
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
             break;
         }
       }
       break;
 
-    case 1:
+    case plat_up:
       // Door is moving up
       res = T_MovePlane
             (
@@ -233,7 +227,7 @@ void T_VerticalDoor (vldoor_t* door)
           case normal:
           case genRaise:
           case genBlazeRaise:
-            door->direction = 0; // wait at top with delay
+            door->direction = plat_stop; // wait at top with delay
             door->topcountdown = door->topwait;
             break;
 
@@ -253,7 +247,7 @@ void T_VerticalDoor (vldoor_t* door)
         }
 
         /* jff 1/31/98 turn lighting on in tagged sectors of manual doors
-   * killough 10/98: replaced with gradual lighting code */
+	 * killough 10/98: replaced with gradual lighting code */
       }
       break;
   }
@@ -270,20 +264,15 @@ void T_VerticalDoor (vldoor_t* door)
 //
 // Handle opening a tagged locked door
 //
-// Passed the line activating the door, the type of door,
+// Passed the line activating the door, the type of door, 
 // and the thing that activated the line
 // Returns true if a thinker created
 //
-int EV_DoLockedDoor
-( line_t* line,
-  vldoor_e  type,
-  mobj_t* thing )
+int EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing)
 {
-  player_t* p;
+  player_t *p = thing->player;
 
-  // only players can open locked doors
-  p = thing->player;
-  if (!p)
+  if (!p)          // only players can open locked doors
     return 0;
 
   // check type of linedef, and if key is possessed to open it
@@ -293,7 +282,8 @@ int EV_DoLockedDoor
     case 133:
       if (!p->cards[it_bluecard] && !p->cards[it_blueskull])
       {
-        p->message = s_PD_BLUEO;             // Ty 03/27/98 - externalized
+                //sf: player_printf
+          player_printf(p, s_PD_BLUEO);       // Ty 03/27/98 - externalized
         S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
@@ -303,7 +293,7 @@ int EV_DoLockedDoor
     case 135:
       if (!p->cards[it_redcard] && !p->cards[it_redskull])
       {
-        p->message = s_PD_REDO;              // Ty 03/27/98 - externalized
+          player_printf(p, s_PD_REDO);             // Ty 03/27/98 - externalized
         S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
@@ -313,11 +303,11 @@ int EV_DoLockedDoor
     case 137:
       if (!p->cards[it_yellowcard] && !p->cards[it_yellowskull])
       {
-        p->message = s_PD_YELLOWO;           // Ty 03/27/98 - externalized
+          player_printf(p, s_PD_YELLOWO);             // Ty 03/27/98 - externalized
         S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
-      break;
+      break;  
   }
 
   // got the key, so open the door
@@ -330,20 +320,16 @@ int EV_DoLockedDoor
 //
 // Handle opening a tagged door
 //
-// Passed the line activating the door and the type of door
+// Passed the line activating the door and the type of door 
 // Returns true if a thinker created
 //
-int EV_DoDoor
-( line_t* line,
-  vldoor_e  type )
+
+int EV_DoDoor(line_t *line, vldoor_e type)
 {
-  int   secnum,rtn;
+  int secnum = -1, rtn = 0;
   sector_t* sec;
   vldoor_t* door;
 
-  secnum = -1;
-  rtn = 0;
-
   // open all doors with the same tag as the activating line
   while ((secnum = P_FindSectorFromLineTag(line,secnum)) >= 0)
   {
@@ -351,7 +337,7 @@ int EV_DoDoor
     // if the ceiling already moving, don't start the door action
     if (P_SectorActive(ceiling_special,sec)) //jff 2/22/98
         continue;
-
+ 
     // new door thinker
     rtn = 1;
     door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
@@ -365,50 +351,50 @@ int EV_DoDoor
     door->speed = VDOORSPEED;
     door->line = line; // jff 1/31/98 remember line that triggered us
     door->lighttag = 0; /* killough 10/98: no light effects with tagged doors */
-
+    
     // setup door parameters according to type of door
     switch(type)
     {
       case blazeClose:
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
-        door->direction = -1;
+          door->direction = plat_down;
         door->speed = VDOORSPEED * 4;
         S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdcls);
         break;
-
+      
       case close:
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
-        door->direction = -1;
+          door->direction = plat_down;
         S_StartSound((mobj_t *)&door->sector->soundorg,sfx_dorcls);
         break;
-
+      
       case close30ThenOpen:
         door->topheight = sec->ceilingheight;
-        door->direction = -1;
+          door->direction = plat_down;
         S_StartSound((mobj_t *)&door->sector->soundorg,sfx_dorcls);
         break;
-
+      
       case blazeRaise:
       case blazeOpen:
-        door->direction = 1;
+          door->direction = plat_up;
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
         door->speed = VDOORSPEED * 4;
         if (door->topheight != sec->ceilingheight)
           S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
         break;
-
+      
       case normal:
       case open:
-        door->direction = 1;
+          door->direction = plat_up;
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
         if (door->topheight != sec->ceilingheight)
           S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
         break;
-
+      
       default:
         break;
     }
@@ -416,9 +402,120 @@ int EV_DoDoor
   return rtn;
 }
 
+//
+// EV_OpenDoor
+//
+// sf: for FraggleScript functions
+// allows greater control over how the door behaves
+//
+
+void EV_OpenDoor(int sectag, int speed, int wait_time)
+{
+  vldoor_e door_type;
+  int secnum = -1;
+  vldoor_t *door;
+
+  if(speed < 1) speed = 1;
+  
+  // find out door type first
+
+  if(wait_time)               // door closes afterward
+    {
+      if(speed >= 4)              // blazing ?
+	door_type = blazeRaise;
+      else
+	door_type = normal;
+    }
+  else
+    {
+      if(speed >= 4)              // blazing ?
+	door_type = blazeOpen;
+      else
+	door_type = open;
+    }
+
+  // open door in all the sectors with the specified tag
+
+  while ((secnum = P_FindSectorFromTag(sectag, secnum)) >= 0)
+    {
+      sector_t *sec = &sectors[secnum];
+      // if the ceiling already moving, don't start the door action
+      if (P_SectorActive(ceiling_special,sec)) //jff 2/22/98
+        continue;
+
+      // new door thinker
+      door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
+      P_AddThinker(&door->thinker);
+      sec->ceilingdata = door;
+
+      door->thinker.function = T_VerticalDoor;
+      door->sector = sec;
+      door->type = door_type;
+      door->topwait = wait_time;
+      door->speed = VDOORSPEED * speed;
+      door->line = NULL;   // not triggered by a line
+      door->lighttag = 0;  // no lighting effect
+      door->topheight = P_FindLowestCeilingSurrounding(sec) - 4*FRACUNIT;
+      door->direction = plat_up;
+
+      if (door->topheight != sec->ceilingheight)
+	S_StartSound((mobj_t *)&door->sector->soundorg,
+		     speed >= 4 ? sfx_bdopn : sfx_doropn);
+    }
+}
 
 //
-// EV_VerticalDoor
+// EV_CloseDoor
+//
+// sf: also for FraggleScript functions
+//
+
+void EV_CloseDoor(int sectag, int speed)
+{
+  vldoor_e door_type;
+  int secnum = -1;
+  vldoor_t *door;
+
+  if(speed < 1) speed = 1;
+  
+  // find out door type first
+
+  if(speed >= 4)              // blazing ?
+    door_type = blazeClose;
+  else
+    door_type = close;
+  
+  // open door in all the sectors with the specified tag
+
+  while ((secnum = P_FindSectorFromTag(sectag, secnum)) >= 0)
+    {
+      sector_t *sec = &sectors[secnum];
+      // if the ceiling already moving, don't start the door action
+      if (P_SectorActive(ceiling_special,sec)) //jff 2/22/98
+        continue;
+
+      // new door thinker
+      door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
+      P_AddThinker(&door->thinker);
+      sec->ceilingdata = door;
+
+      door->thinker.function = T_VerticalDoor;
+      door->sector = sec;
+      door->type = door_type;
+      door->speed = VDOORSPEED * speed;
+      door->line = NULL;   // not triggered by a line
+      door->lighttag = 0;  // no lighting effect
+      door->topheight = P_FindLowestCeilingSurrounding(sec) - 4*FRACUNIT;
+      door->direction = plat_down;
+
+      S_StartSound((mobj_t *)&door->sector->soundorg,
+		   speed >= 4 ? sfx_bdcls : sfx_dorcls);
+    }  
+}
+
+
+//
+// EV_VerticalDoor 
 //
 // Handle opening a door manually, no tag value
 //
@@ -427,9 +524,8 @@ int EV_DoDoor
 //
 // jff 2/12/98 added int return value, fixed all returns
 //
-int EV_VerticalDoor
-( line_t* line,
-  mobj_t* thing )
+
+int EV_VerticalDoor(line_t *line, mobj_t *thing)
 {
   player_t* player;
   int   secnum;
@@ -438,7 +534,7 @@ int EV_VerticalDoor
 
   //  Check for locks
   player = thing->player;
-
+  
   switch(line->special)
   {
     case 26: // Blue Lock
@@ -447,31 +543,31 @@ int EV_VerticalDoor
         return 0;
       if (!player->cards[it_bluecard] && !player->cards[it_blueskull])
       {
-          player->message = s_PD_BLUEK;         // Ty 03/27/98 - externalized
+          player_printf(player, s_PD_BLUEK);             // Ty 03/27/98 - externalized
           S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
       break;
-
+  
     case 27: // Yellow Lock
     case 34:
       if ( !player )
           return 0;
       if (!player->cards[it_yellowcard] && !player->cards[it_yellowskull])
       {
-          player->message = s_PD_YELLOWK;       // Ty 03/27/98 - externalized
+          player_printf(player, s_PD_YELLOWK);             // Ty 03/27/98 - externalized
           S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
       break;
-
+  
     case 28: // Red Lock
     case 33:
       if ( !player )
           return 0;
       if (!player->cards[it_redcard] && !player->cards[it_redskull])
       {
-          player->message = s_PD_REDK;          // Ty 03/27/98 - externalized
+          player_printf(player, s_PD_REDK);             // Ty 03/27/98 - externalized
           S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
@@ -480,7 +576,7 @@ int EV_VerticalDoor
     default:
       break;
   }
-
+  
   // if the wrong side of door is pushed, give oof sound
   if (line->sidenum[1]==-1)                     // killough
   {
@@ -493,11 +589,8 @@ int EV_VerticalDoor
   secnum = sec-sectors;
 
   /* if door already has a thinker, use it
-   * cph 2001/04/05 -
-   * Ok, this is a disaster area. We're assuming that sec->ceilingdata
-   *  is a vldoor_t! What if this door is controlled by both DR lines
-   *  and by switches? I don't know how to fix that.
-   * Secondly, original Doom didn't distinguish floor/lighting/ceiling
+   * cph 2001/04/05 - 
+   * Original Doom didn't distinguish floor/lighting/ceiling 
    *  actions, so we need to do the same in demo compatibility mode.
    */
   door = sec->ceilingdata;
@@ -505,26 +598,27 @@ int EV_VerticalDoor
     if (!door) door = sec->floordata;
     if (!door) door = sec->lightingdata;
   }
-  if (door)
-  {
-    switch(line->special)
-    {
-      case  1: // only for "raise" doors, not "open"s
-      case  26:
-      case  27:
-      case  28:
-      case  117:
-        if (door->direction == -1)
-          door->direction = 1;  // go back up
-        else
-        {
-          if (!thing->player)
-            return 0;           // JDC: bad guys never close doors
-
-          door->direction = -1; // start going down immediately
-        }
-        return 1;
+  if (door) {
+    /* If the current action is a T_VerticalDoor and we're back in
+     * EV_VerticalDoor, it must have been a repeatable line, so I've dropped
+     * that check. For old demos we have to emulate the old buggy behavior and
+     * mess up non-T_VerticalDoor actions.
+     */
+    if (compatibility_level < prboom_4_compatibility || 
+        door->thinker.function == T_VerticalDoor) {
+      /* An already moving repeatable door which is being re-pressed, or a
+       * monster is trying to open a closing door - so change direction
+       */
+      if (door->direction == -1) {
+        door->direction = 1; return 1; /* go back up */
+      } else if (player) {
+        door->direction = -1; return 1; /* go back down */
+      }
     }
+    /* Either we're in prboom >=v2.3 and it's not a door, or it's a door but
+     * we're a monster and don't want to shut it; exit with no action.
+     */
+    return 0;
   }
 
   // emit proper sound
@@ -535,23 +629,18 @@ int EV_VerticalDoor
       S_StartSound((mobj_t *)&sec->soundorg,sfx_bdopn);
       break;
 
-    case 1:   // normal door sound
-    case 31:
-      S_StartSound((mobj_t *)&sec->soundorg,sfx_doropn);
-      break;
-
-    default:  // locked door sound
+    default:  // normal or locked door sound
       S_StartSound((mobj_t *)&sec->soundorg,sfx_doropn);
       break;
   }
-
+  
   // new door thinker
   door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
   P_AddThinker (&door->thinker);
   sec->ceilingdata = door; //jff 2/22/98
   door->thinker.function = T_VerticalDoor;
   door->sector = sec;
-  door->direction = 1;
+  door->direction = plat_up;
   door->speed = VDOORSPEED;
   door->topwait = VDOORWAIT;
   door->line = line; // jff 1/31/98 remember line that triggered us
@@ -591,7 +680,7 @@ int EV_VerticalDoor
       door->lighttag = 0;   // killough 10/98
       break;
   }
-
+  
   // find the top and bottom of the movement range
   door->topheight = P_FindLowestCeilingSurrounding(sec);
   door->topheight -= 4*FRACUNIT;
@@ -626,7 +715,7 @@ void P_SpawnDoorCloseIn30 (sector_t* sec)
 
   door->thinker.function = T_VerticalDoor;
   door->sector = sec;
-  door->direction = 0;
+  door->direction = plat_stop;
   door->type = normal;
   door->speed = VDOORSPEED;
   door->topcountdown = 30 * 35;
@@ -642,14 +731,13 @@ void P_SpawnDoorCloseIn30 (sector_t* sec)
 // Passed the sector of the door, whose type specified the door action
 // Returns nothing
 //
-void P_SpawnDoorRaiseIn5Mins
-( sector_t* sec,
-  int   secnum )
+
+void P_SpawnDoorRaiseIn5Mins(sector_t *sec, int secnum)
 {
   vldoor_t* door;
 
   door = Z_Malloc ( sizeof(*door), PU_LEVSPEC, 0);
-
+  
   P_AddThinker (&door->thinker);
 
   sec->ceilingdata = door; //jff 2/22/98
