@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -138,9 +138,16 @@ void P_MakeDivline(const line_t *li, divline_t *dl)
 
 fixed_t CONSTFUNC P_InterceptVector(const divline_t *v2, const divline_t *v1)
 {
+ if (compatibility_level < prboom_4_compatibility) {
   fixed_t den = FixedMul(v1->dy>>8, v2->dx) - FixedMul(v1->dx>>8, v2->dy);
   return den ? FixedDiv((FixedMul((v1->x-v2->x)>>8, v1->dy) +
                          FixedMul((v2->y-v1->y)>>8, v1->dx)), den) : 0;
+ } else {
+  int_64_t den = (int_64_t)v1->dy * v2->dx - (int_64_t)v1->dx * v2->dy;
+  den >>= 16;
+  if (!den) return 0;
+  return (fixed_t)(((int_64_t)(v1->x - v2->x) * v1->dy - (int_64_t)(v1->y - v2->y) * v1->dx) / den);
+ }
 }
 
 //
@@ -216,7 +223,7 @@ void P_UnsetThingPosition (mobj_t *thing)
       mobj_t **sprev = thing->sprev;
       mobj_t  *snext = thing->snext;
       if ((*sprev = snext))  // unlink from sector list
-        snext->sprev = sprev;
+	      snext->sprev = sprev;
 
         // phares 3/14/98
         //
@@ -250,7 +257,7 @@ void P_UnsetThingPosition (mobj_t *thing)
 
       mobj_t *bnext, **bprev = thing->bprev;
       if (bprev && (*bprev = bnext = thing->bnext))  // unlink from block map
-        bnext->bprev = bprev;
+      	bnext->bprev = bprev;
     }
 }
 
@@ -275,7 +282,7 @@ void P_SetThingPosition(mobj_t *thing)
       mobj_t **link = &ss->sector->thinglist;
       mobj_t *snext = *link;
       if ((thing->snext = snext))
-        snext->sprev = &thing->snext;
+      	snext->sprev = &thing->snext;
       thing->sprev = link;
       *link = thing;
 
@@ -305,14 +312,14 @@ void P_SetThingPosition(mobj_t *thing)
       int blocky = (thing->y - bmaporgy)>>MAPBLOCKSHIFT;
       if (blockx>=0 && blockx < bmapwidth && blocky>=0 && blocky < bmapheight)
         {
-        // killough 8/11/98: simpler scheme using pointer-to-pointer prev
-        // pointers, allows head nodes to be treated like everything else
+    	  // killough 8/11/98: simpler scheme using pointer-to-pointer prev
+	      // pointers, allows head nodes to be treated like everything else
 
         mobj_t **link = &blocklinks[blocky*bmapwidth+blockx];
-        mobj_t *bnext = *link;
+	      mobj_t *bnext = *link;
         if ((thing->bnext = bnext))
-          bnext->bprev = &thing->bnext;
-        thing->bprev = link;
+	        bnext->bprev = &thing->bnext;
+	      thing->bprev = link;
         *link = thing;
       }
       else        // thing is off the map
