@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -37,6 +37,7 @@
  *
  *-----------------------------------------------------------------------------*/
 
+#include "z_zone.h"
 #include "doomstat.h"
 #include "p_spec.h"
 #include "p_tick.h"
@@ -56,6 +57,7 @@
 #include "d_deh.h"
 #include "r_plane.h"
 #include "lprintf.h"
+#include "t_script.h"
 
 //
 // Animating textures and planes
@@ -74,7 +76,14 @@ typedef struct
 //
 //      source animation definition
 //
-//
+#ifdef _MSC_VER
+#define GCC_PACKED
+#else //_MSC_VER
+/* we might want to make this sensitive to whether configure has detected
+   a build with GCC */
+#define GCC_PACKED __attribute__((packed))
+#endif //_MSC_VER
+
 #ifdef _MSC_VER // proff: This is the same as __attribute__ ((packed)) in GNUC
 #pragma pack(push)
 #pragma pack(1)
@@ -86,11 +95,11 @@ typedef struct
 
 typedef struct
 {
-  signed char istexture; //jff 3/23/98 make char for comparison // cph - make signed
-  char        endname[9];           //  if false, it is a flat
-  char        startname[9];
-  int         speed;
-} PACKEDATTR animdef_t; //jff 3/23/98 pack to read from memory
+  signed char istexture GCC_PACKED; //jff 3/23/98 make char for comparison // cph - make signed
+  char        endname[9] GCC_PACKED;           //  if false, it is a flat
+  char        startname[9] GCC_PACKED;
+  int         speed GCC_PACKED;
+} GCC_PACKED animdef_t; //jff 3/23/98 pack to read from memory
 
 #if defined(__MWERKS__)
 #pragma options align=reset
@@ -206,10 +215,8 @@ void P_InitPicAnims (void)
 //
 // Note: if side=1 is specified, it must exist or results undefined
 //
-side_t* getSide
-( int           currentSector,
-  int           line,
-  int           side )
+
+side_t *getSide(int currentSector, int line, int side)
 {
   return &sides[ (sectors[currentSector].lines[line])->sidenum[side] ];
 }
@@ -224,10 +231,8 @@ side_t* getSide
 //
 // Note: if side=1 is specified, it must exist or results undefined
 //
-sector_t* getSector
-( int           currentSector,
-  int           line,
-  int           side )
+
+sector_t *getSector(int currentSector, int line, int side)
 {
   return sides[ (sectors[currentSector].lines[line])->sidenum[side] ].sector;
 }
@@ -242,9 +247,8 @@ sector_t* getSector
 // modified to return actual two-sidedness rather than presence
 // of 2S flag unless compatibility optioned
 //
-int twoSided
-( int   sector,
-  int   line )
+
+int twoSided(int sector, int line)
 {
   //jff 1/26/98 return what is actually needed, whether the line
   //has two sidedefs, rather than whether the 2S flag is set
@@ -263,9 +267,8 @@ int twoSided
 //
 // Note: returns NULL if not two-sided line, or both sides refer to sector
 //
-sector_t* getNextSector
-( line_t*       line,
-  sector_t*     sec )
+
+sector_t *getNextSector(line_t *line, sector_t *sec)
 {
   //jff 1/26/98 check unneeded since line->backsector already
   //returns NULL if the line is not two sided, and does so from
@@ -492,7 +495,7 @@ fixed_t P_FindLowestCeilingSurrounding(sector_t* sec)
   fixed_t             height = INT_MAX;
 
   /* jff 3/12/98 avoid ovf in height calculations */
-  if (!comp[comp_model]) height = 32000*FRACUNIT;
+  if (!comp[comp_model]) height = 32000*FRACUNIT; 
 
   for (i=0 ;i < sec->linecount ; i++)
   {
@@ -528,7 +531,7 @@ fixed_t P_FindHighestCeilingSurrounding(sector_t* sec)
   /* jff 1/26/98 Fix initial value for floor to not act differently
    * in sections of wad that are below 0 units
    * jff 3/12/98 avoid ovf in height calculations */
-  if (!comp[comp_model]) height = -32000*FRACUNIT;
+  if (!comp[comp_model]) height = -32000*FRACUNIT; 
 
   for (i=0 ;i < sec->linecount ; i++)
   {
@@ -734,6 +737,17 @@ int P_FindLineFromLineTag(const line_t *line, int start)
   return start;
 }
 
+// sf: same thing but from just a number
+
+int P_FindSectorFromTag(const int tag, int start)
+{
+  start = start >= 0 ? sectors[start].nexttag :
+    sectors[(unsigned) tag % (unsigned) numsectors].firsttag;
+  while (start >= 0 && sectors[start].tag != tag)
+    start = sectors[start].nexttag;
+  return start;
+}
+
 // Hash the sector tags across the sectors and linedefs.
 static void P_InitTagLists(void)
 {
@@ -767,9 +781,8 @@ static void P_InitTagLists(void)
 // in a surrounding sector less than that passed. If no smaller light
 // level exists, the light level passed is returned.
 //
-int P_FindMinSurroundingLight
-( sector_t*     sector,
-  int           max )
+
+int P_FindMinSurroundingLight(sector_t *sector, int max)
 {
   int         i;
   int         min;
@@ -804,9 +817,8 @@ int P_FindMinSurroundingLight
 // jff 02/05/98 routine added to test for unlockability of
 //  generalized locked doors
 //
-boolean P_CanUnlockGenDoor
-( line_t* line,
-  player_t* player)
+
+boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
 {
   // does this line special distinguish between skulls and keys?
   int skulliscard = (line->special & LockedNKeys)>>LockedNKeysShift;
@@ -825,7 +837,7 @@ boolean P_CanUnlockGenDoor
         !player->cards[it_yellowskull]
       )
       {
-        player->message = s_PD_ANY; // Ty 03/27/98 - externalized
+          player_printf(player, s_PD_ANY);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -837,7 +849,7 @@ boolean P_CanUnlockGenDoor
         (!skulliscard || !player->cards[it_redskull])
       )
       {
-        player->message = skulliscard? s_PD_REDK : s_PD_REDC; // Ty 03/27/98 - externalized
+          player_printf(player, skulliscard? s_PD_REDK : s_PD_REDC);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -849,7 +861,7 @@ boolean P_CanUnlockGenDoor
         (!skulliscard || !player->cards[it_blueskull])
       )
       {
-        player->message = skulliscard? s_PD_BLUEK : s_PD_BLUEC; // Ty 03/27/98 - externalized
+          player_printf(player, skulliscard ? s_PD_BLUEK : s_PD_BLUEC);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -861,7 +873,7 @@ boolean P_CanUnlockGenDoor
         (!skulliscard || !player->cards[it_yellowskull])
       )
       {
-        player->message = skulliscard? s_PD_YELLOWK : s_PD_YELLOWC; // Ty 03/27/98 - externalized
+          player_printf(player, skulliscard? s_PD_YELLOWK : s_PD_YELLOWC);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -873,7 +885,7 @@ boolean P_CanUnlockGenDoor
         (!skulliscard || !player->cards[it_redcard])
       )
       {
-        player->message = skulliscard? s_PD_REDK : s_PD_REDS; // Ty 03/27/98 - externalized
+          player_printf(player, skulliscard? s_PD_REDK : s_PD_REDS);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -885,7 +897,7 @@ boolean P_CanUnlockGenDoor
         (!skulliscard || !player->cards[it_bluecard])
       )
       {
-        player->message = skulliscard? s_PD_BLUEK : s_PD_BLUES; // Ty 03/27/98 - externalized
+          player_printf(player, skulliscard? s_PD_BLUEK : s_PD_BLUES);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -897,7 +909,7 @@ boolean P_CanUnlockGenDoor
         (!skulliscard || !player->cards[it_yellowcard])
       )
       {
-        player->message = skulliscard? s_PD_YELLOWK : s_PD_YELLOWS; // Ty 03/27/98 - externalized
+          player_printf(player, skulliscard? s_PD_YELLOWK : s_PD_YELLOWS);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -916,7 +928,7 @@ boolean P_CanUnlockGenDoor
         )
       )
       {
-        player->message = s_PD_ALL6; // Ty 03/27/98 - externalized
+          player_printf(player, s_PD_ALL6);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -933,7 +945,7 @@ boolean P_CanUnlockGenDoor
         )
       )
       {
-        player->message = s_PD_ALL3; // Ty 03/27/98 - externalized
+          player_printf(player, s_PD_ALL3);
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -1045,7 +1057,16 @@ int P_CheckTag(line_t *line)
 
     case 48:                // Scrolling walls
     case 85:
-      return 1;   // zero tag allowed
+        // sf: scripting
+    case 272:   // WR
+    case 273:
+    case 274:   // W1
+    case 275:
+    case 276:   // SR
+    case 277:   // S1
+    case 278:   // GR
+    case 279:   // G1
+      return 1;
 
     default:
       break;
@@ -1136,11 +1157,7 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
     int (*linefunc)(line_t *line)=NULL;
 
     // check each range of generalized linedefs
-    if ((unsigned)line->special >= GenEnd)
-    {
-      // Out of range for GenFloors
-    }
-    else if ((unsigned)line->special >= GenFloorBase)
+    if ((unsigned)line->special >= GenFloorBase)
     {
       if (!thing->player)
         if ((line->special & FloorChange) || !(line->special & FloorModel))
@@ -1404,7 +1421,7 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
       // EXIT!
       // killough 10/98: prevent zombies from exiting levels
       if (!(thing->player && thing->player->health <= 0 && !comp[comp_zombie]))
-  G_ExitLevel ();
+	G_ExitLevel ();
       break;
 
     case 53:
@@ -1490,7 +1507,7 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
       // killough 10/98: prevent zombies from exiting levels
       // CPhipps - change for lxdoom's compatibility handling
       if (!(thing->player && thing->player->health <= 0 && !comp[comp_zombie]))
-  G_SecretExitLevel ();
+	G_SecretExitLevel ();
       break;
 
     case 125:
@@ -1675,6 +1692,29 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
     case 129:
       // Raise Floor Turbo
       EV_DoFloor(line,raiseFloorTurbo);
+      break;
+
+        // scripting ld types
+
+        // repeatable
+
+    case 273:  // console command (1sided)
+      if(side) break;
+
+    case 272:  // console command (2sided)
+      t_trigger = thing;
+      T_RunScript(line->tag);
+      break;
+
+        // once-only triggers
+
+    case 275:  // console command (1sided)
+      if(side) break;
+
+    case 274:  // console command (2sided)
+      t_trigger = thing;
+      T_RunScript(line->tag);
+      line->special = 0;        // clear trigger
       break;
 
       // Extended walk triggers
@@ -1997,9 +2037,7 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
 // of the line, should the sector already be in motion when the line is
 // impacted. Change is qualified by demo_compatibility.
 //
-void P_ShootSpecialLine
-( mobj_t*       thing,
-  line_t*       line )
+void P_ShootSpecialLine(mobj_t *thing, line_t *line)
 {
   //jff 02/04/98 add check here for generalized linedef
   if (!demo_compatibility)
@@ -2009,11 +2047,7 @@ void P_ShootSpecialLine
     int (*linefunc)(line_t *line)=NULL;
 
     // check each range of generalized linedefs
-    if ((unsigned)line->special >= GenEnd)
-    {
-      // Out of range for GenFloors
-    }
-    else if ((unsigned)line->special >= GenFloorBase)
+    if ((unsigned)line->special >= GenFloorBase)
     {
       if (!thing->player)
         if ((line->special & FloorChange) || !(line->special & FloorModel))
@@ -2144,6 +2178,14 @@ void P_ShootSpecialLine
     //jff 1/30/98 added new gun linedefs here
     // killough 1/31/98: added demo_compatibility check, added inner switch
 
+        // sf: scripting
+    case 278:
+    case 279:
+      t_trigger = thing;
+      T_RunScript(line->tag);
+      if(line->special == 279) line->special = 0;       // clear if G1
+      break;
+
     default:
       if (!demo_compatibility)
         switch (line->special)
@@ -2182,9 +2224,7 @@ void P_ShootSpecialLine
 //
 void P_PlayerInSpecialSector (player_t* player)
 {
-  sector_t*   sector;
-
-  sector = player->mo->subsector->sector;
+  sector_t *sector = player->mo->subsector->sector;
 
   // Falling, not all the way down yet?
   // Sector specials don't apply in mid-air
@@ -3260,3 +3300,5 @@ static void P_SpawnPushers(void)
 // phares 3/20/98: End of Pusher effects
 //
 ////////////////////////////////////////////////////////////////////////////
+
+
