@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -6,9 +6,9 @@
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2000,2002 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -29,11 +29,16 @@
  *
  *-----------------------------------------------------------------------------*/
 
+#include "z_zone.h"
 #include "doomstat.h"
 #include "p_user.h"
 #include "p_spec.h"
 #include "p_tick.h"
-#include "p_map.h"
+#include "t_script.h"
+
+#ifdef COMPILE_VIDD
+#include "vidd/vidd.h"
+#endif
 
 int leveltime;
 
@@ -45,13 +50,11 @@ int leveltime;
 // but the first element must be thinker_t.
 //
 
-// Both the head and tail of the thinker list.
-thinker_t thinkercap;
+/* killough 8/29/98: we maintain several separate threads, each containing
+ * a special class of thinkers, to allow more efficient searches. 
+ */
 
-// killough 8/29/98: we maintain several separate threads, each containing
-// a special class of thinkers, to allow more efficient searches.
-
-thinker_t thinkerclasscap[NUMTHCLASS];
+thinker_t thinkerclasscap[NUMTHCLASS]; // Dim on actual size - POPE
 
 //
 // P_InitThinkers
@@ -69,26 +72,31 @@ void P_InitThinkers(void)
 
 //
 // killough 8/29/98:
-//
+// 
 // We maintain separate threads of friends and enemies, to permit more
 // efficient searches.
 //
 
 void P_UpdateThinker(thinker_t *thinker)
 {
+  register thinker_t *th;
   // find the class the thinker belongs to
-
-  int class = thinker->function == P_MobjThinker &&
-    ((mobj_t *) thinker)->health > 0 &&
+  
+  int class =
+    thinker->function == P_RemoveThinkerDelayed ? th_delete :
+    thinker->function == P_MobjThinker && 
+    ((mobj_t *) thinker)->health > 0 && 
     (((mobj_t *) thinker)->flags & MF_COUNTKILL ||
      ((mobj_t *) thinker)->type == MT_SKULL) ?
     ((mobj_t *) thinker)->flags & MF_FRIEND ?
     th_friends : th_enemies : th_misc;
 
-  // Remove from current thread
-  thinker_t *th = thinker->cnext;
-  (th->cprev = thinker->cprev)->cnext = th;
-
+  {
+    /* Remove from current thread, if in one */
+    if ((th = thinker->cnext)!= NULL)
+      (th->cprev = thinker->cprev)->cnext = th;
+  }
+  
   // Add to appropriate thread
   th = &thinkerclasscap[class];
   th->cprev->cnext = thinker;
@@ -112,7 +120,7 @@ void P_AddThinker(thinker_t* thinker)
   thinker->references = 0;    // killough 11/98: init reference counter to 0
 
   // killough 8/29/98: set sentinel pointers, and then add to appropriate list
-  thinker->cnext = thinker->cprev = thinker;
+  thinker->cnext = thinker->cprev = NULL;
   P_UpdateThinker(thinker);
 }
 
@@ -139,8 +147,23 @@ void P_RemoveThinkerDelayed(thinker_t *thinker)
 {
   if (!thinker->references)
     {
-      thinker_t *next = thinker->next;
-      (next->prev = currentthinker = thinker->prev)->next = next;
+#ifdef COMPILE_VIDD    
+      if (VIDD_REC_inProgress()) VIDD_REC_registerElementDestruction(thinker); // POPE
+#endif
+    
+      { /* Remove from main thinker list */
+        thinker_t *next = thinker->next;
+        /* Note that currentthinker is guaranteed to point to us,
+         * and since we're freeing our memory, we had better change that. So
+         * point it to thinker->prev, so the iterator will correctly move on to
+         * thinker->prev->next = thinker->next */
+        (next->prev = currentthinker = thinker->prev)->next = next;
+      }
+      {
+        /* Remove from current thinker class list */
+        thinker_t *th = thinker->cnext;
+        (th->cprev = thinker->cprev)->cnext = th;
+      }
       Z_Free(thinker);
     }
 }
@@ -162,15 +185,25 @@ void P_RemoveThinker(thinker_t *thinker)
 {
   thinker->function = P_RemoveThinkerDelayed;
 
-  // killough 8/29/98: remove immediately from threaded list
-  (thinker->cnext->cprev = thinker->cprev)->cnext = thinker->cnext;
+  P_UpdateThinker(thinker);
+}
+
+/* cph 2002/01/13 - iterator for thinker list
+ * WARNING: Do not modify thinkers between calls to this functin
+ */
+thinker_t* P_NextThinker(thinker_t* th, th_class cl)
+{
+  thinker_t* top = &thinkerclasscap[cl];
+  if (!th) th = top;
+  th = cl == th_all ? th->next : th->cnext;
+  return th == top ? NULL : th;
 }
 
 /*
  * P_SetTarget
  *
  * This function is used to keep track of pointer references to mobj thinkers.
- * In Doom, objects such as lost souls could sometimes be removed despite
+ * In Doom, objects such as lost souls could sometimes be removed despite 
  * their still being referenced. In Boom, 'target' mobj fields were tested
  * during each gametic, and any objects pointed to by them would be prevented
  * from being removed. But this was incomplete, and was slow (every mobj was
@@ -231,25 +264,33 @@ void P_Ticker (void)
    * killough 9/29/98: note that this ties in with basetic,
    * since G_Ticker does the pausing during recording or
    * playback, and compenates by incrementing basetic.
-   *
+   * 
    * All of this complicated mess is used to preserve demo sync.
    */
 
   if (paused || (menuactive && !demoplayback && !netgame &&
-     players[consoleplayer].viewz != 1))
+		 players[consoleplayer].viewz != 1))
     return;
 
-  P_MapStart();
+#ifdef COMPILE_VIDD
+  if (!VIDD_PLAY_inProgress()) { // POPE
+#endif
+
                // not if this is an intermission screen
   if(gamestate==GS_LEVEL)
   for (i=0; i<MAXPLAYERS; i++)
     if (playeringame[i])
       P_PlayerThink(&players[i]);
 
+#ifdef COMPILE_VIDD
+  } // if (!VIDD_PLAY_inProgress()) { // POPE
+#endif
+
   P_RunThinkers();
   P_UpdateSpecials();
   P_RespawnSpecials();
-  P_MapEnd();
   leveltime++;                       // for par times
+
+  T_DelayedScripts();
 }
 
