@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -32,7 +32,7 @@
 #include "d_ticcmd.h"
 #include "m_swap.h"
 
-enum packet_type_e {
+enum packet_type_e { 
   PKT_INIT,    // initial packet to server
   PKT_SETUP,   // game information packet
   PKT_GO,      // game has started
@@ -49,12 +49,8 @@ enum packet_type_e {
 typedef struct {
   byte checksum;       // Simple checksum of the entire packet
   byte type;           /* Type of packet */
-  byte reserved[2];	/* Was random in prboom <=2.2.4, now 0 */
   unsigned tic;        // Timestamp
-} PACKEDATTR packet_header_t;
-
-static inline void packet_set(packet_header_t* p, enum packet_type_e t, unsigned long tic)
-{ p->tic = doom_htonl(tic); p->type = t; p->reserved[0] = 0; p->reserved[1] = 0; }
+} packet_header_t;
 
 #ifndef GAME_OPTIONS_SIZE
 // From g_game.h
@@ -83,8 +79,8 @@ inline static void RawToTic(ticcmd_t* dst, const void* src)
 
 inline static void TicToRaw(void* dst, const ticcmd_t* src)
 {
-  /* We have to make a copy of the source struct, then do byte swaps,
-   * and fnially copy to the destination (can't do the swaps in the
+  /* We have to make a copy of the source struct, then do byte swaps, 
+   * and fnially copy to the destination (can't do the swaps in the 
    * destination, because it might not be aligned).
    */
   ticcmd_t tmp = *src;
