@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -31,16 +31,13 @@
 
 #include "doomstat.h"
 #include "m_bbox.h"
-#include "i_system.h"
 #include "r_main.h"
 #include "r_segs.h"
 #include "r_plane.h"
 #include "r_things.h"
 #include "r_bsp.h" // cph - sanity checking
+#include "v_video.h"
 #include "lprintf.h"
-#ifdef GL_DOOM
-#include "gl_struct.h"
-#endif
 
 seg_t     *curline;
 side_t    *sidedef;
@@ -66,16 +63,16 @@ void R_ClearDrawSegs(void)
   ds_p = drawsegs;
 }
 
-// CPhipps -
-// Instead of clipsegs, let's try using an array with one entry for each column,
+// CPhipps - 
+// Instead of clipsegs, let's try using an array with one entry for each column, 
 // indicating whether it's blocked by a solid wall yet or not.
 
 byte solidcol[MAX_SCREENWIDTH];
 
-// CPhipps -
+// CPhipps - 
 // R_ClipWallSegment
 //
-// Replaces the old R_Clip*WallSegment functions. It draws bits of walls in those
+// Replaces the old R_Clip*WallSegment functions. It draws bits of walls in those 
 // columns which aren't solid, and updates the solidcol[] array appropriately
 
 void R_ClipWallSegment(int first, int last, boolean solid)
@@ -91,9 +88,9 @@ void R_ClipWallSegment(int first, int last, boolean solid)
       else to = p - solidcol;
       R_StoreWallRange(first, to-1);
       if (solid) {
-  memset(solidcol+first,1,to-first);
+	memset(solidcol+first,1,to-first);
       }
-  first = to;
+	first = to;
     }
   }
 }
@@ -110,7 +107,7 @@ void R_ClearClipSegs (void)
 // killough 1/18/98 -- This function is used to fix the automap bug which
 // showed lines behind closed doors simply because the door had a dropoff.
 //
-// cph - converted to R_RecalcLineFlags. This recalculates all the flags for
+// cph - converted to R_RecalcLineFlags. This recalculates all the flags for 
 // a line, including closure and texture tiling.
 
 static void R_RecalcLineFlags(void)
@@ -118,25 +115,25 @@ static void R_RecalcLineFlags(void)
   linedef->r_validcount = gametic;
 
   /* First decide if the line is closed, normal, or invisible */
-  if (!(linedef->flags & ML_TWOSIDED)
+  if (!(linedef->flags & ML_TWOSIDED) 
       || backsector->ceilingheight <= frontsector->floorheight
       || backsector->floorheight >= frontsector->ceilingheight
       || (
-    // if door is closed because back is shut:
-    backsector->ceilingheight <= backsector->floorheight
-
-    // preserve a kind of transparent door/lift special effect:
-    && (backsector->ceilingheight >= frontsector->ceilingheight ||
-        curline->sidedef->toptexture)
-
-    && (backsector->floorheight <= frontsector->floorheight ||
-        curline->sidedef->bottomtexture)
-
-    // properly render skies (consider door "open" if both ceilings are sky):
-    && (backsector->ceilingpic !=skyflatnum ||
-        frontsector->ceilingpic!=skyflatnum)
-    )
-      )
+	  // if door is closed because back is shut:
+	  backsector->ceilingheight <= backsector->floorheight
+	  
+	  // preserve a kind of transparent door/lift special effect:
+	  && (backsector->ceilingheight >= frontsector->ceilingheight ||
+	      curline->sidedef->toptexture)
+	  
+	  && (backsector->floorheight <= frontsector->floorheight ||
+	      curline->sidedef->bottomtexture)
+	  
+	  // properly render skies (consider door "open" if both ceilings are sky):
+	  && (backsector->ceilingpic !=skyflatnum ||
+	      frontsector->ceilingpic!=skyflatnum)
+	  )
+      ) 
     linedef->r_flags = RF_CLOSED;
   else {
     // Reject empty lines used for triggers
@@ -146,16 +143,16 @@ static void R_RecalcLineFlags(void)
     // and no middle texture.
     // CPhipps - recode for speed, not certain if this is portable though
     if (backsector->ceilingheight != frontsector->ceilingheight
-  || backsector->floorheight != frontsector->floorheight
-  || curline->sidedef->midtexture
-  || memcmp(&backsector->floor_xoffs, &frontsector->floor_xoffs,
-      sizeof(frontsector->floor_xoffs) + sizeof(frontsector->floor_yoffs) +
-      sizeof(frontsector->ceiling_xoffs) + sizeof(frontsector->ceiling_yoffs) +
-      sizeof(frontsector->ceilingpic) + sizeof(frontsector->floorpic) +
-      sizeof(frontsector->lightlevel) + sizeof(frontsector->floorlightsec) +
-      sizeof(frontsector->ceilinglightsec))) {
+	|| backsector->floorheight != frontsector->floorheight
+	|| curline->sidedef->midtexture
+	|| memcmp(&backsector->floor_xoffs, &frontsector->floor_xoffs,
+		  sizeof(frontsector->floor_xoffs) + sizeof(frontsector->floor_yoffs) +
+		  sizeof(frontsector->ceiling_xoffs) + sizeof(frontsector->ceiling_yoffs) +
+		  sizeof(frontsector->ceilingpic) + sizeof(frontsector->floorpic) + 
+		  sizeof(frontsector->lightlevel) + sizeof(frontsector->floorlightsec) + 
+		  sizeof(frontsector->ceilinglightsec))) {
       linedef->r_flags = 0; return;
-    } else
+    } else 
       linedef->r_flags = RF_IGNORE;
   }
 
@@ -165,21 +162,21 @@ static void R_RecalcLineFlags(void)
   /* Now decide on texture tiling */
   if (linedef->flags & ML_TWOSIDED) {
     int c;
-
+    
     /* Does top texture need tiling */
-    if ((c = frontsector->ceilingheight - backsector->ceilingheight) > 0 &&
-   (textureheight[texturetranslation[curline->sidedef->toptexture]] > c))
+    if ((c = frontsector->ceilingheight - backsector->ceilingheight) > 0 && 
+	 (textureheight[texturetranslation[curline->sidedef->toptexture]] > c))
       linedef->r_flags |= RF_TOP_TILE;
 
     /* Does bottom texture need tiling */
-    if ((c = frontsector->floorheight - backsector->floorheight) > 0 &&
-   (textureheight[texturetranslation[curline->sidedef->bottomtexture]] > c))
+    if ((c = frontsector->floorheight - backsector->floorheight) > 0 && 
+	 (textureheight[texturetranslation[curline->sidedef->bottomtexture]] > c))
       linedef->r_flags |= RF_BOT_TILE;
   } else {
     int c;
     /* Does middle texture need tiling */
-    if ((c = frontsector->ceilingheight - frontsector->floorheight) > 0 &&
-   (textureheight[texturetranslation[curline->sidedef->midtexture]] > c))
+    if ((c = frontsector->ceilingheight - frontsector->floorheight) > 0 && 
+	 (textureheight[texturetranslation[curline->sidedef->midtexture]] > c))
       linedef->r_flags |= RF_MID_TILE;
   }
 }
@@ -212,8 +209,7 @@ sector_t *R_FakeFlat(sector_t *sec, sector_t *tempsec,
   if (sec->heightsec != -1)
     {
       const sector_t *s = &sectors[sec->heightsec];
-      int heightsec = viewplayer->mo->subsector->sector->heightsec;
-      int underwater = heightsec!=-1 && viewz<=sectors[heightsec].floorheight;
+      int underwater = viewheightsec!=-1 && viewz<=sectors[viewheightsec].floorheight;
 
       // Replace sector being drawn, with a copy to be hacked
       *tempsec = *sec;
@@ -232,14 +228,14 @@ sector_t *R_FakeFlat(sector_t *sec, sector_t *tempsec,
 
           if (underwater) {
             if (s->ceilingpic == skyflatnum) {
-		tempsec->floorheight   = tempsec->ceilingheight+1;
-		tempsec->ceilingpic    = tempsec->floorpic;
+	      tempsec->floorheight   = tempsec->ceilingheight+1;
+	      tempsec->ceilingpic    = tempsec->floorpic;
                 tempsec->ceiling_xoffs = tempsec->floor_xoffs;
                 tempsec->ceiling_yoffs = tempsec->floor_yoffs;
 	    } else {
-		tempsec->ceilingpic    = s->ceilingpic;
-		tempsec->ceiling_xoffs = s->ceiling_xoffs;
-		tempsec->ceiling_yoffs = s->ceiling_yoffs;
+	      tempsec->ceilingpic    = s->ceilingpic;
+	      tempsec->ceiling_xoffs = s->ceiling_xoffs;
+	      tempsec->ceiling_yoffs = s->ceiling_yoffs;
 	    }
 	  }
 
@@ -254,7 +250,7 @@ sector_t *R_FakeFlat(sector_t *sec, sector_t *tempsec,
             sectors[s->ceilinglightsec].lightlevel; // killough 4/11/98
         }
       else
-        if (heightsec != -1 && viewz >= sectors[heightsec].ceilingheight &&
+        if (viewheightsec != -1 && viewz >= sectors[viewheightsec].ceilingheight &&
             sec->ceilingheight > s->ceilingheight)
           {   // Above-ceiling hack
             tempsec->ceilingheight = s->ceilingheight;
@@ -302,7 +298,6 @@ static void R_AddLine (seg_t *line)
   angle_t  span;
   angle_t  tspan;
   static sector_t tempsec;     // killough 3/8/98: ceiling/water hack
-  //  boolean solid = true;
 
   curline = line;
 
@@ -355,9 +350,10 @@ static void R_AddLine (seg_t *line)
   x2 = viewangletox[angle2];
 
 #ifdef GL_DOOM
+  // proff 11/99: we have to add these segs to avoid gaps in OpenGL
+  if (x1 >= x2)       // killough 1/31/98 -- change == to >= for robustness
   {
-    // proff 11/99: we have to add these segs to avoid gaps in OpenGL
-    if (x1 >= x2)       // killough 1/31/98 -- change == to >= for robustness
+    if (V_GetMode() == VID_MODEGL)
     {
       if (ds_p == drawsegs+maxdrawsegs)   // killough 1/98 -- fix 2s line HOM
       {
@@ -373,7 +369,10 @@ static void R_AddLine (seg_t *line)
       gld_AddWall(curline);
       return;
     }
+    else
+      return;
   }
+  else
 #else
   // Does not cross a pixel?
   if (x1 >= x2)       // killough 1/31/98 -- change == to >= for robustness
@@ -388,7 +387,7 @@ static void R_AddLine (seg_t *line)
     backsector = R_FakeFlat(backsector, &tempsec, NULL, NULL, true);
 
   /* cph - roll up linedef properties in flags */
-  if ((linedef = curline->linedef)->r_validcount != gametic)
+  if ((linedef = curline->linedef)->r_validcount != gametic) 
     R_RecalcLineFlags();
 
   if (linedef->r_flags & RF_IGNORE)
@@ -422,22 +421,22 @@ static const int checkcoord[12][4] = // killough -- static const
 };
 
 // killough 1/28/98: static // CPhipps - const parameter, reformatted
-static boolean R_CheckBBox(const fixed_t *bspcoord)
+static boolean R_CheckBBox(const fixed_t *bspcoord) 
 {
   angle_t angle1, angle2;
 
   {
     int        boxpos;
     const int* check;
-
+    
     // Find the corners of the box
     // that define the edges from current viewpoint.
     boxpos = (viewx <= bspcoord[BOXLEFT] ? 0 : viewx < bspcoord[BOXRIGHT ] ? 1 : 2) +
       (viewy >= bspcoord[BOXTOP ] ? 0 : viewy > bspcoord[BOXBOTTOM] ? 4 : 8);
-
+    
     if (boxpos == 5)
       return true;
-
+    
     check = checkcoord[boxpos];
     angle1 = R_PointToAngle (bspcoord[check[0]], bspcoord[check[1]]) - viewangle;
     angle2 = R_PointToAngle (bspcoord[check[2]], bspcoord[check[3]]) - viewangle;
@@ -446,7 +445,7 @@ static boolean R_CheckBBox(const fixed_t *bspcoord)
   // cph - replaced old code, which was unclear and badly commented
   // Much more efficient code now
   if ((signed)angle1 < (signed)angle2) { /* it's "behind" us */
-    /* Either angle1 or angle2 is behind us, so it doesn't matter if we
+    /* Either angle1 or angle2 is behind us, so it doesn't matter if we 
      * change it to the corect sign
      */
     if ((angle1 >= ANG180) && (angle1 < ANG270))
@@ -474,7 +473,7 @@ static boolean R_CheckBBox(const fixed_t *bspcoord)
     if (sx1 == sx2)
       return false;
 
-    if (!memchr(solidcol+sx1, 0, sx2-sx1)) return false;
+    if (!memchr(solidcol+sx1, 0, sx2-sx1)) return false; 
     // All columns it covers are already solidly covered
   }
 
@@ -497,10 +496,6 @@ static void R_Subsector(int num)
   sector_t    tempsec;              // killough 3/7/98: deep water hack
   int         floorlightlevel;      // killough 3/16/98: set floor lightlevel
   int         ceilinglightlevel;    // killough 4/11/98
-#ifdef GL_DOOM
-  visplane_t dummyfloorplane;
-  visplane_t dummyceilingplane;
-#endif
 
 #ifdef RANGECHECK
   if (num>=numsubsectors)
@@ -524,8 +519,8 @@ static void R_Subsector(int num)
     (frontsector->heightsec != -1 &&
      sectors[frontsector->heightsec].ceilingpic == skyflatnum) ?
     R_FindPlane(frontsector->floorheight,
-                frontsector->floorpic == skyflatnum &&  // kilough 10/98
-                frontsector->sky & PL_SKYFLAT ? frontsector->sky :
+            		frontsector->floorpic == skyflatnum &&  // kilough 10/98
+		            frontsector->sky & PL_SKYFLAT ? frontsector->sky :
                 frontsector->floorpic,
                 floorlightlevel,                // killough 3/16/98
                 frontsector->floor_xoffs,       // killough 3/7/98
@@ -537,17 +532,20 @@ static void R_Subsector(int num)
     (frontsector->heightsec != -1 &&
      sectors[frontsector->heightsec].floorpic == skyflatnum) ?
     R_FindPlane(frontsector->ceilingheight,     // killough 3/8/98
-                frontsector->ceilingpic == skyflatnum &&  // kilough 10/98
-                frontsector->sky & PL_SKYFLAT ? frontsector->sky :
+            		frontsector->ceilingpic == skyflatnum &&  // kilough 10/98
+            		frontsector->sky & PL_SKYFLAT ? frontsector->sky :
                 frontsector->ceilingpic,
                 ceilinglightlevel,              // killough 4/11/98
                 frontsector->ceiling_xoffs,     // killough 3/7/98
                 frontsector->ceiling_yoffs
                 ) : NULL;
-#ifdef GL_DOOM
+
   // check if the sector is faked
-  if (frontsector==sub->sector)
+  if ((frontsector==sub->sector) && (V_GetMode() == VID_MODEGL))
   {
+    visplane_t dummyfloorplane;
+    visplane_t dummyceilingplane;
+
     // if the sector has bottomtextures, then the floorheight will be set to the
     // highest surounding floorheight
     if ((frontsector->no_bottomtextures) || (!floorplane))
@@ -604,9 +602,8 @@ static void R_Subsector(int num)
         ceilingplane=&dummyceilingplane;
     }
   }
-#endif
 
-  // killough 9/18/98: Fix underwater slowdown, by passing real sector
+  // killough 9/18/98: Fix underwater slowdown, by passing real sector 
   // instead of fake one. Improve sprite lighting by basing sprite
   // lightlevels on floor & ceiling lightlevels in the surrounding area.
   //
@@ -616,7 +613,7 @@ static void R_Subsector(int num)
   // That is part of the 242 effect!!!  If you simply pass sub->sector to
   // the old code you will not get correct lighting for underwater sprites!!!
   // Either you must pass the fake sector and handle validcount here, on the
-  // real sector, or you must account for the lighting in some other way,
+  // real sector, or you must account for the lighting in some other way, 
   // like passing it as an argument.
 
   R_AddSprites(sub, (floorlightlevel+ceilinglightlevel)/2);
@@ -625,9 +622,11 @@ static void R_Subsector(int num)
     if (line->miniseg == false)
       R_AddLine (line);
     line++;
+    curline = NULL; /* cph 2001/11/18 - must clear curline now we're done with it, so R_ColourMap doesn't try using it for other things */
   }
 #ifdef GL_DOOM
-  gld_AddPlane(num, floorplane, ceilingplane);
+  if (V_GetMode() == VID_MODEGL)
+    gld_AddPlane(num, floorplane, ceilingplane);
 #endif
 }
 
