@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -6,9 +6,9 @@
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2001 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -29,6 +29,7 @@
  *
  *-----------------------------------------------------------------------------*/
 
+#include "z_zone.h"
 #include "doomstat.h"
 #include "w_wad.h"
 #include "r_main.h"
@@ -36,10 +37,8 @@
 #include "r_segs.h"
 #include "r_draw.h"
 #include "r_things.h"
+#include "v_video.h"
 #include "lprintf.h"
-#ifdef GL_DOOM
-#include "gl_struct.h"
-#endif
 
 #define MINZ        (FRACUNIT*4)
 #define BASEYCENTER 100
@@ -65,13 +64,12 @@ fixed_t pspriteiscale;
 // proff 11/06/98: Added for high-res
 fixed_t pspriteyscale;
 
-static lighttable_t **spritelights;        // killough 1/25/98 made static
-
 // constant arrays
 //  used for psprite clipping and initializing clipping
 
 short negonearray[MAX_SCREENWIDTH];        // killough 2/8/98:
 short screenheightarray[MAX_SCREENWIDTH];  // change to MAX_*
+int lefthanded=0;
 
 //
 // INITIALIZATION FUNCTIONS
@@ -79,7 +77,7 @@ short screenheightarray[MAX_SCREENWIDTH];  // change to MAX_*
 
 // variables used to look up and range check thing_t sprites patches
 
-spritedef_t *sprites;
+spritedef_t *sprites = NULL;
 int numsprites;
 
 #define MAX_SPRITE_FRAMES 29          /* Macroized -- killough 1/25/98 */
@@ -124,6 +122,22 @@ static void R_InstallSpriteLump(int lump, unsigned frame,
     }
 }
 
+void R_FreeSpriteDefs(void)
+{
+  int i;
+
+  if (!sprites)
+    return;
+  for (i=0 ; i<numsprites ; i++)
+    if (sprites[i].spriteframes)
+    {
+      Z_Free(sprites[i].spriteframes);
+      sprites[i].spriteframes = NULL;
+    }
+  Z_Free(sprites);
+  sprites = NULL;
+}
+
 //
 // R_InitSpriteDefs
 // Pass a null terminated list of sprite names
@@ -159,6 +173,8 @@ void R_InitSpriteDefs(const char * const * namelist)
   if (!numentries || !*namelist)
     return;
 
+  R_FreeSpriteDefs();
+
   // count the number of sprite names
   for (i=0; namelist[i]; i++)
     ;
@@ -166,6 +182,7 @@ void R_InitSpriteDefs(const char * const * namelist)
   numsprites = i;
 
   sprites = Z_Malloc(numsprites *sizeof(*sprites), PU_STATIC, NULL);
+  memset(sprites, 0, numsprites *sizeof(*sprites));
 
   // Create hash table based on just the first four letters of each sprite
   // killough 1/31/98
@@ -196,7 +213,7 @@ void R_InitSpriteDefs(const char * const * namelist)
           maxframe = -1;
           do
             {
-              register lumpinfo_t *lump = lumpinfo + j + firstspritelump;
+              register lumpinfo_t *lump = &lumpinfo[j + firstspritelump];
 
               // Fast portable comparison -- killough
               // (using int pointer cast is nonportable):
@@ -315,41 +332,59 @@ short   *mceilingclip;
 fixed_t spryscale;
 fixed_t sprtopscreen;
 
-void R_DrawMaskedColumn(const column_t *column)
-{
+//---------------------------------------------------------------------------
+void R_DrawMaskedColumn(
+  const TPatch *patch,
+  const TPatchColumn *column, 
+  const TPatchColumn *prevColumn, 
+  const TPatchColumn *nextColumn
+) {
   int     topscreen;
   int     bottomscreen;
-  fixed_t basetexturemid = dc_texturemid;
-
-  dc_texheight = 0; // killough 11/98
-  while (column->topdelta != 0xff)
-    {
+  fixed_t basetexturemid = dcvars.texturemid;
+  
+  int i;
+  
+  dcvars.texheight = patch->height;//0; // killough 11/98
+  for (i=0; i<column->numPosts; i++) {
+    const TPatchPost *post = &column->posts[i]; 
+  
+  //while (column->topdelta != 0xff)
+  //  {
       // calculate unclipped screen coordinates for post
-      topscreen = sprtopscreen + spryscale*column->topdelta;
-      bottomscreen = topscreen + spryscale*column->length;
+      topscreen = sprtopscreen + spryscale*post->startY;
+      bottomscreen = sprtopscreen + spryscale*(post->startY+post->length-1);
 
-      dc_yl = (topscreen+FRACUNIT-1)>>FRACBITS;
-      dc_yh = (bottomscreen-1)>>FRACBITS;
+      dcvars.yl = (topscreen+FRACUNIT-1)>>FRACBITS;
+      dcvars.yh = (bottomscreen-1)>>FRACBITS;
 
-      if (dc_yh >= mfloorclip[dc_x])
-        dc_yh = mfloorclip[dc_x]-1;
+      if (dcvars.yh >= mfloorclip[dcvars.x])
+        dcvars.yh = mfloorclip[dcvars.x]-1;
 
-      if (dc_yl <= mceilingclip[dc_x])
-        dc_yl = mceilingclip[dc_x]+1;
+      if (dcvars.yl <= mceilingclip[dcvars.x])
+        dcvars.yl = mceilingclip[dcvars.x]+1;
 
-      // killough 3/2/98, 3/27/98: Failsafe against overflow/crash:
-      if (dc_yl <= dc_yh && dc_yh < viewheight)
+      if (dcvars.yl <= dcvars.yh && dcvars.yh < viewheight)
         {
-          dc_source = (const byte *)column + 3;
-          dc_texturemid = basetexturemid - (column->topdelta<<FRACBITS);
+          dcvars.source = column->pixels + post->startY;
+          
+          dcvars.nextsource = nextColumn->pixels + post->startY;
+          dcvars.prevsource = prevColumn->pixels + post->startY;
+          dcvars.texturemid = basetexturemid - (post->startY<<FRACBITS);
 
+          if (rdrawvars.maskedColumnEdgeType == RDRAW_MASKEDCOLUMNEDGE_SLOPED) { // POPE
+            dcvars.edgeSlope = post->edgeSloping;
+          }
           // Drawn by either R_DrawColumn
           //  or (SHADOW) R_DrawFuzzColumn.
+          dcvars.drawingmasked = 1; // POPE
           colfunc ();
+          dcvars.drawingmasked = 0; // POPE
         }
-      column = (const column_t *)(  (byte *)column + column->length + 4);
+
+      //column = (const column_t *)(  (byte *)column + column->length + 4);
     }
-  dc_texturemid = basetexturemid;
+  dcvars.texturemid = basetexturemid;
 }
 
 //
@@ -359,55 +394,60 @@ void R_DrawMaskedColumn(const column_t *column)
 // CPhipps - new wad lump handling, *'s to const*'s
 void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
 {
-  const column_t *column;
   int      texturecolumn;
   fixed_t  frac;
-  const patch_t *patch = W_CacheLumpNum (vis->patch+firstspritelump);
-
-  dc_colormap = vis->colormap;
+  const TPatch *patch = R_GetPatch(vis->patch+firstspritelump);
+  
+  dcvars.colormap = vis->colormap;
+  dcvars.nextcolormap = dcvars.colormap; // POPE
 
   // killough 4/11/98: rearrange and handle translucent sprites
   // mixed with translucent/non-translucenct 2s normals
 
-  if (!dc_colormap)   // NULL colormap = shadow draw
-    colfunc = R_DrawFuzzColumn;    // killough 3/14/98
+  if (!dcvars.colormap)   // NULL colormap = shadow draw
+    colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_FUZZ); // POPE
   else
     if (vis->mobjflags & MF_TRANSLATION)
       {
-        colfunc = R_DrawTranslatedColumn;
-        dc_translation = translationtables - 256 +
+        colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_TRANSLATED); // POPE
+        dcvars.translation = translationtables - 256 +
           ((vis->mobjflags & MF_TRANSLATION) >> (MF_TRANSSHIFT-8) );
       }
     else
       if (vis->mobjflags & MF_TRANSLUCENT && general_translucency) // phares
         {
-          colfunc = R_DrawTLColumn;
+          colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_TRANSLUCENT); // POPE
           tranmap = main_tranmap;       // killough 4/11/98
         }
       else
-        colfunc = R_DrawColumn;         // killough 3/14/98, 4/11/98
+        colfunc =R_GetDrawFunc(RDRAW_PIPELINE_COL_STANDARD); // POPE
 
 // proff 11/06/98: Changed for high-res
-  dc_iscale = FixedDiv (FRACUNIT, vis->scale);
-  dc_texturemid = vis->texturemid;
+  dcvars.iscale = FixedDiv (FRACUNIT, vis->scale);
+  dcvars.texturemid = vis->texturemid;
   frac = vis->startfrac;
   spryscale = vis->scale;
-  sprtopscreen = centeryfrac - FixedMul(dc_texturemid,spryscale);
+  sprtopscreen = centeryfrac - FixedMul(dcvars.texturemid,spryscale);
 
-  for (dc_x=vis->x1 ; dc_x<=vis->x2 ; dc_x++, frac += vis->xiscale)
+  for (dcvars.x=vis->x1 ; dcvars.x<=vis->x2 ; dcvars.x++, frac += vis->xiscale)
     {
       texturecolumn = frac>>FRACBITS;
+      dcvars.texu = frac;
 
 #ifdef RANGECHECK
       if (texturecolumn < 0 || texturecolumn >= SHORT(patch->width))
         I_Error ("R_DrawSpriteRange: Bad texturecolumn");
 #endif
 
-      column = (const column_t *)((const byte *) patch +
-                            LONG(patch->columnofs[texturecolumn]));
-      R_DrawMaskedColumn (column);
+      R_DrawMaskedColumn(
+        patch, 
+        R_GetPatchColumnClamped(patch, texturecolumn),
+        R_GetPatchColumnClamped(patch, texturecolumn-1),
+        R_GetPatchColumnClamped(patch, texturecolumn+1)
+      );
     }
-  colfunc = R_DrawColumn;         // killough 3/14/98
+  
+  colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_STANDARD); // POPE
   W_UnlockLumpNum(vis->patch+firstspritelump); // cph - release lump
 }
 
@@ -416,7 +456,7 @@ void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
 // Generates a vissprite for a thing if it might be visible.
 //
 
-void R_ProjectSprite (mobj_t* thing)
+void R_ProjectSprite (mobj_t* thing, int lightlevel)
 {
   fixed_t   gzt;               // killough 3/27/98
   fixed_t   tx;
@@ -428,9 +468,6 @@ void R_ProjectSprite (mobj_t* thing)
   int       lump;
   boolean   flip;
   vissprite_t *vis;
-#ifndef GL_DOOM
-  fixed_t   iscale;
-#endif
   int heightsec;      // killough 3/27/98
 
   // transform the origin point
@@ -441,6 +478,7 @@ void R_ProjectSprite (mobj_t* thing)
   fixed_t gyt = -FixedMul(tr_y,viewsin);
 
   fixed_t tz = gxt-gyt;
+  int width;
 
     // thing is behind view plane?
   if (tz < MINZ)
@@ -472,6 +510,11 @@ void R_ProjectSprite (mobj_t* thing)
 
   sprframe = &sprdef->spriteframes[thing->frame & FF_FRAMEMASK];
 
+  // If sprframe is NULL the sprite is not in the IWAD.
+  if (!sprframe)
+    I_Error ("R_ProjectSprite: Missing spriteframes %i : %i", thing->sprite,
+             thing->frame);
+
   if (sprframe->rotate)
     {
       // choose a different rotation based on player view
@@ -487,25 +530,24 @@ void R_ProjectSprite (mobj_t* thing)
       flip = (boolean) sprframe->flip[0];
     }
 
-  /* calculate edges of the shape
-   * cph 2003/08/1 - fraggle points out that this offset must be flipped if the
-   * sprite is flipped; e.g. FreeDoom imp is messed up by this. */
-  tx -= flip ? spritewidth[lump] - spriteoffset[lump] : spriteoffset[lump];
-  x1 = (centerxfrac + FixedMul(tx,xscale)) >>FRACBITS;
+  {
+    const TPatch *patch = R_GetPatch(lump+firstspritelump);
 
-    // off the right side?
-  if (x1 > viewwidth)
+    // calculate edges of the shape
+    tx -= SHORT(patch->leftOffset)<<FRACBITS;
+    x1 = (centerxfrac + FixedMul(tx,xscale)) >>FRACBITS;
+
+    tx += SHORT(patch->width)<<FRACBITS;
+    x2 = ((centerxfrac + FixedMul (tx,xscale) ) >>FRACBITS) - 1;
+
+    gzt = thing->z + (SHORT(patch->topOffset)<<FRACBITS);
+    width = SHORT(patch->width);
+  }
+
+  // off the side?
+  if (x1 > viewwidth || x2 < 0)
     return;
 
-  tx +=  spritewidth[lump];
-  x2 = ((centerxfrac + FixedMul (tx,xscale) ) >>FRACBITS) - 1;
-
-    // off the left side
-  if (x2 < 0)
-    return;
-
-  gzt = thing->z + spritetopoffset[lump];
-
   // killough 4/9/98: clip things which are out of view due to height
   if (thing->z > viewz + FixedDiv(centeryfrac, xscale) ||
       gzt      < viewz - FixedDiv(centeryfrac-viewheight, xscale))
@@ -519,12 +561,11 @@ void R_ProjectSprite (mobj_t* thing)
 
   if (heightsec != -1)   // only clip things which are in special sectors
     {
-      int phs = viewplayer->mo->subsector->sector->heightsec;
-      if (phs != -1 && viewz < sectors[phs].floorheight ?
+      if (viewheightsec != -1 && viewz < sectors[viewheightsec].floorheight ?
           thing->z >= sectors[heightsec].floorheight :
           gzt < sectors[heightsec].floorheight)
         return;
-      if (phs != -1 && viewz > sectors[phs].ceilingheight ?
+      if (viewheightsec != -1 && viewz > sectors[viewheightsec].ceilingheight ?
           gzt < sectors[heightsec].ceilingheight &&
           viewz >= sectors[heightsec].ceilingheight :
           thing->z >= sectors[heightsec].ceilingheight)
@@ -535,59 +576,64 @@ void R_ProjectSprite (mobj_t* thing)
   vis = R_NewVisSprite ();
 
 #ifdef GL_DOOM
-  // proff 11/99: add sprite for OpenGL
-  vis->thing = thing;
-  vis->flip = flip;
-  vis->scale = FixedDiv(projectiony, tz);
-  vis->patch = lump;
-  gld_AddSprite(vis);
-  return;
-#else
-  // killough 3/27/98: save sector for special clipping later
-  vis->heightsec = heightsec;
+  if (V_GetMode() == VID_MODEGL)
+  {
+    // proff 11/99: add sprite for OpenGL
+    vis->thing = thing;
+    vis->flip = flip;
+    vis->scale = FixedDiv(projectiony, tz);
+    vis->patch = lump;
+    gld_AddSprite(vis);
 
-  vis->mobjflags = thing->flags;
-// proff 11/06/98: Changed for high-res
-  vis->scale = FixedDiv(projectiony, tz);
-  vis->gx = thing->x;
-  vis->gy = thing->y;
-  vis->gz = thing->z;
-  vis->gzt = gzt;                          // killough 3/27/98
-  vis->texturemid = vis->gzt - viewz;
-  vis->x1 = x1 < 0 ? 0 : x1;
-  vis->x2 = x2 >= viewwidth ? viewwidth-1 : x2;
-  iscale = FixedDiv (FRACUNIT, xscale);
-
-  if (flip)
-    {
-      vis->startfrac = spritewidth[lump]-1;
-      vis->xiscale = -iscale;
-    }
+    return;
+  }
   else
-    {
-      vis->startfrac = 0;
-      vis->xiscale = iscale;
-    }
-
-  if (vis->x1 > x1)
-    vis->startfrac += vis->xiscale*(vis->x1-x1);
-  vis->patch = lump;
-
-  // get light level
-  if (thing->flags & MF_SHADOW)
-      vis->colormap = NULL;             // shadow draw
-  else if (fixedcolormap)
-    vis->colormap = fixedcolormap;      // fixed map
-  else if (thing->frame & FF_FULLBRIGHT)
-    vis->colormap = fullcolormap;     // full bright  // killough 3/20/98
-  else
-    {      // diminished light
-      int index = xscale>>LIGHTSCALESHIFT;
-      if (index >= MAXLIGHTSCALE)
-        index = MAXLIGHTSCALE-1;
-      vis->colormap = spritelights[index];
-    }
 #endif
+  {
+    fixed_t   iscale;
+
+    // killough 3/27/98: save sector for special clipping later
+    vis->heightsec = heightsec;
+
+    vis->mobjflags = thing->flags;
+    // proff 11/06/98: Changed for high-res
+    vis->scale = FixedDiv(projectiony, tz);
+    vis->gx = thing->x;
+    vis->gy = thing->y;
+    vis->gz = thing->z;
+    vis->gzt = gzt;                          // killough 3/27/98
+    vis->texturemid = vis->gzt - viewz;
+    vis->x1 = x1 < 0 ? 0 : x1;
+    vis->x2 = x2 >= viewwidth ? viewwidth-1 : x2;
+    iscale = FixedDiv (FRACUNIT, xscale);
+
+    if (flip)
+      {
+        vis->startfrac = (width<<FRACBITS)-1;
+        vis->xiscale = -iscale;
+      }
+    else
+      {
+        vis->startfrac = 0;
+        vis->xiscale = iscale;
+      }
+
+    if (vis->x1 > x1)
+      vis->startfrac += vis->xiscale*(vis->x1-x1);
+    vis->patch = lump;
+
+    // get light level
+    if (thing->flags & MF_SHADOW)
+        vis->colormap = NULL;             // shadow draw
+    else if (fixedcolormap)
+      vis->colormap = fixedcolormap;      // fixed map
+    else if (thing->frame & FF_FULLBRIGHT)
+      vis->colormap = fullcolormap;     // full bright  // killough 3/20/98
+    else
+      {      // diminished light
+        vis->colormap = R_ColourMap(lightlevel,xscale);
+      }
+  }
 }
 
 //
@@ -599,7 +645,6 @@ void R_AddSprites(subsector_t* subsec, int lightlevel)
 {
   sector_t* sec=subsec->sector;
   mobj_t *thing;
-  int    lightnum;
 
   // BSP is traversed by subsector.
   // A sector might have been split into several
@@ -612,28 +657,18 @@ void R_AddSprites(subsector_t* subsec, int lightlevel)
   // Well, now it will be done.
   sec->validcount = validcount;
 
-  lightnum = (lightlevel >> LIGHTSEGSHIFT)+extralight;
-
-  if (lightnum < 0)
-    spritelights = scalelight[0];
-  else if (lightnum >= LIGHTLEVELS)
-    spritelights = scalelight[LIGHTLEVELS-1];
-  else
-    spritelights = scalelight[lightnum];
-
   // Handle all things in sector.
 
   for (thing = sec->thinglist; thing; thing = thing->snext)
-    R_ProjectSprite(thing);
+    R_ProjectSprite(thing, lightlevel);
 }
 
 //
 // R_DrawPSprite
 //
 
-void R_DrawPSprite (pspdef_t *psp)
+void R_DrawPSprite (pspdef_t *psp, int lightlevel)
 {
-  fixed_t       tx;
   int           x1, x2;
   spritedef_t   *sprdef;
   spriteframe_t *sprframe;
@@ -641,6 +676,8 @@ void R_DrawPSprite (pspdef_t *psp)
   boolean       flip;
   vissprite_t   *vis;
   vissprite_t   avis;
+  int           spritewidth;
+  fixed_t       topoffset;
 
   // decide which patch to use
 
@@ -662,21 +699,26 @@ void R_DrawPSprite (pspdef_t *psp)
   lump = sprframe->lump[0];
   flip = (boolean) sprframe->flip[0];
 
-  // calculate edges of the shape
-  tx = psp->sx-160*FRACUNIT;
+  {
+    const TPatch *patch = R_GetPatch(lump+firstspritelump);
+    
+    // calculate edges of the shape
+    fixed_t       tx;
+    int           spriteoffset = SHORT(patch->leftOffset)<<FRACBITS;
+    tx = psp->sx-160*FRACUNIT;
+    spritewidth = SHORT(patch->width)<<FRACBITS;
 
-  tx -= spriteoffset[lump];
-  x1 = (centerxfrac + FixedMul (tx,pspritescale))>>FRACBITS;
+    tx -= flip ? spritewidth - spriteoffset : spriteoffset;
+    x1 = (centerxfrac + FixedMul (tx,pspritescale))>>FRACBITS;
 
-  // off the right side
-  if (x1 > viewwidth)
-    return;
+    tx += spritewidth;
+    x2 = ((centerxfrac + FixedMul (tx, pspritescale) ) >>FRACBITS) - 1;
 
-  tx +=  spritewidth[lump];
-  x2 = ((centerxfrac + FixedMul (tx, pspritescale) ) >>FRACBITS) - 1;
+    topoffset = SHORT(patch->topOffset)<<FRACBITS;
+  }
 
-  // off the left side
-  if (x2 < 0)
+  // off the side
+  if (x2 < 0 || x1 > viewwidth)
     return;
 
   // store information in a vissprite
@@ -684,7 +726,7 @@ void R_DrawPSprite (pspdef_t *psp)
   vis->mobjflags = 0;
    // killough 12/98: fix psprite positioning problem
   vis->texturemid = (BASEYCENTER<<FRACBITS) /* +  FRACUNIT/2 */ -
-                    (psp->sy-spritetopoffset[lump]);
+                    (psp->sy-topoffset);
   vis->x1 = x1 < 0 ? 0 : x1;
   vis->x2 = x2 >= viewwidth ? viewwidth-1 : x2;
 // proff 11/06/98: Added for high-res
@@ -693,7 +735,7 @@ void R_DrawPSprite (pspdef_t *psp)
   if (flip)
     {
       vis->xiscale = -pspriteiscale;
-      vis->startfrac = spritewidth[lump]-1;
+      vis->startfrac = spritewidth-1;
     }
   else
     {
@@ -714,12 +756,15 @@ void R_DrawPSprite (pspdef_t *psp)
   else if (psp->state->frame & FF_FULLBRIGHT)
     vis->colormap = fullcolormap;            // full bright // killough 3/20/98
   else
-    vis->colormap = spritelights[MAXLIGHTSCALE-1];  // local light
+    vis->colormap = R_ColourMap(lightlevel,pspritescale);  // local light
 
   // proff 11/99: don't use software stuff in OpenGL
-#ifndef GL_DOOM
-  R_DrawVisSprite(vis, vis->x1, vis->x2);
-#else
+  if (V_GetMode() != VID_MODEGL)
+  {
+    R_DrawVisSprite(vis, vis->x1, vis->x2);
+  }
+#ifdef GL_DOOM
+  else
   {
     int lightlevel;
     sector_t tmpsec;
@@ -729,7 +774,7 @@ void R_DrawPSprite (pspdef_t *psp)
       lightlevel=255;
     else
     {
-//      lightlevel = (viewplayer->mo->subsector->sector->lightlevel) + (extralight << LIGHTSEGSHIFT);
+      //lightlevel = (viewplayer->mo->subsector->sector->lightlevel) + (extralight << LIGHTSEGSHIFT);
       R_FakeFlat( viewplayer->mo->subsector->sector, &tmpsec,
                   &floorlightlevel, &ceilinglightlevel, false);
       lightlevel = ((floorlightlevel+ceilinglightlevel) >> 1) + (extralight << LIGHTSEGSHIFT);
@@ -741,7 +786,7 @@ void R_DrawPSprite (pspdef_t *psp)
     }
     gld_DrawWeapon(lump,vis,lightlevel);
   }
-#endif
+#endif	
 }
 
 //
@@ -750,20 +795,11 @@ void R_DrawPSprite (pspdef_t *psp)
 
 void R_DrawPlayerSprites(void)
 {
-  int i, lightnum;
+  int i, lightlevel = viewplayer->mo->subsector->sector->lightlevel;
   pspdef_t *psp;
 
-  // get light level
-  lightnum = (viewplayer->mo->subsector->sector->lightlevel >> LIGHTSEGSHIFT)
-    + extralight;
-
-  if (lightnum < 0)
-    spritelights = scalelight[0];
-  else if (lightnum >= LIGHTLEVELS)
-    spritelights = scalelight[LIGHTLEVELS-1];
-  else
-    spritelights = scalelight[lightnum];
-
+  if (viewcamera) return;
+  
   // clip to screen bounds
   mfloorclip = screenheightarray;
   mceilingclip = negonearray;
@@ -771,7 +807,7 @@ void R_DrawPlayerSprites(void)
   // add all active psprites
   for (i=0, psp=viewplayer->psprites; i<NUMPSPRITES; i++,psp++)
     if (psp->state)
-      R_DrawPSprite (psp);
+      R_DrawPSprite (psp, lightlevel);
 }
 
 //
@@ -931,7 +967,7 @@ void R_DrawSprite (vissprite_t* spr)
   // Clip the sprite against deep water and/or fake ceilings.
   // killough 4/9/98: optimize by adding mh
   // killough 4/11/98: improve sprite clipping for underwater/fake ceilings
-  // killough 11/98: fix disappearing sprites
+  // killough 11/98: fix disappearing sprites 
 
   if (spr->heightsec != -1)  // only things in specially marked sectors
     {
@@ -947,10 +983,10 @@ void R_DrawSprite (vissprite_t* spr)
                 clipbot[x] = h;
           }
         else                        // clip top
-    if (phs != -1 && viewz <= sectors[phs].floorheight) // killough 11/98
-      for (x=spr->x1 ; x<=spr->x2 ; x++)
-        if (cliptop[x] == -2 || h > cliptop[x])
-    cliptop[x] = h;
+	  if (phs != -1 && viewz <= sectors[phs].floorheight) // killough 11/98
+	    for (x=spr->x1 ; x<=spr->x2 ; x++)
+	      if (cliptop[x] == -2 || h > cliptop[x])
+		cliptop[x] = h;
       }
 
       if ((mh = sectors[spr->heightsec].ceilingheight) < spr->gzt &&
@@ -976,7 +1012,7 @@ void R_DrawSprite (vissprite_t* spr)
   for (x = spr->x1 ; x<=spr->x2 ; x++) {
     if (clipbot[x] == -2)
       clipbot[x] = viewheight;
-
+    
     if (cliptop[x] == -2)
       cliptop[x] = -1;
   }
