@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -45,6 +45,7 @@
 #include "sounds.h"
 #include "dstrings.h"
 #include "r_draw.h"
+#include "c_runcmd.h"
 
 //
 // STATUS BAR DATA
@@ -284,11 +285,6 @@ static boolean st_armson;
 // !deathmatch
 static boolean st_fragson;
 
-// main bar left
-// CPhipps - convert to a bitmap
-static byte *sbar;
-static unsigned short sbar_width, sbar_height;
-
 // 0-9, tall numbers
 static patchnum_t tallnum[10];
 
@@ -300,7 +296,7 @@ static patchnum_t shortnum[10];
 
 // 3 key-cards, 3 skulls, 3 card/skull combos
 // jff 2/24/98 extend number of patches by three skull/card combos
-static patchnum_t keys[NUMCARDS+3];
+patchnum_t keys[NUMCARDS+3];
 
 // face status patches
 static patchnum_t faces[ST_NUMFACES];
@@ -318,14 +314,14 @@ static patchnum_t arms[6][2];
 static st_number_t w_ready;
 
 //jff 2/16/98 status color change levels
-int ammo_red;      // ammo percent less than which status is red
-int ammo_yellow;   // ammo percent less is yellow more green
-int health_red;    // health amount less than which status is red
-int health_yellow; // health amount less than which status is yellow
-int health_green;  // health amount above is blue, below is green
-int armor_red;     // armor amount less than which status is red
-int armor_yellow;  // armor amount less than which status is yellow
-int armor_green;   // armor amount above is blue, below is green
+int ammo_red=25;      // ammo percent less than which status is red
+int ammo_yellow=50;   // ammo percent less is yellow more green
+int health_red=25;    // health amount less than which status is red
+int health_yellow=50; // health amount less than which status is yellow
+int health_green=100; // health amount above is blue, below is green
+int armor_red=25;     // armor amount less than which status is red
+int armor_yellow=50;  // armor amount less than which status is yellow
+int armor_green=100;  // armor amount above is blue, below is green
 
  // in deathmatch only, summary of frags stats
 static st_number_t w_frags;
@@ -390,18 +386,17 @@ void ST_refreshBackground(void)
 
   if (st_statusbaron)
     {
-#ifdef GL_DOOM
       // proff 05/17/2000: draw to the frontbuffer in OpenGL
-      y=ST_Y;
-#endif
+      if (V_GetMode() == VID_MODEGL)
+        y=ST_Y;
       V_DrawNamePatch(ST_X, y, screen, "STBAR", CR_DEFAULT, VPT_STRETCH);
 
       // killough 3/7/98: make face background change with displayplayer
       if (netgame)
       {
         V_DrawNumPatch(ST_FX, y, BG, faceback.lumpnum,
-           displayplayer ? CR_LIMIT+displayplayer : CR_DEFAULT,
-           displayplayer ? (VPT_TRANS | VPT_STRETCH) : VPT_STRETCH);
+		       displayplayer ? CR_LIMIT+displayplayer : CR_DEFAULT, 
+		       displayplayer ? (VPT_TRANS | VPT_STRETCH) : VPT_STRETCH);
       }
 
       V_CopyRect(ST_X, y, screen, ST_SCALED_WIDTH, ST_SCALED_HEIGHT, ST_X, ST_SCALED_Y, FG, VPT_NONE);
@@ -625,7 +620,7 @@ void ST_updateFaceWidget(void)
 
 }
 
-int sts_traditional_keys; // killough 2/28/98: traditional status bar keys
+int sts_traditional_keys = 0; // killough 2/28/98: traditional status bar keys
 
 void ST_updateWidgets(void)
 {
@@ -738,8 +733,13 @@ void ST_doPaletteStuff(void)
       else
         palette = 0;
 
-  if (palette != st_palette)
+  if (palette != st_palette) {
     V_SetPalette(st_palette = palette); // CPhipps - use new palette function
+    
+    // have to redraw the entire status bar when the palette changes
+    // in truecolor modes - POPE
+    if (V_GetMode() == VID_MODE16 || V_GetMode() == VID_MODE32) st_firsttime = true;
+  }
 }
 
 void ST_drawWidgets(boolean refresh)
@@ -823,7 +823,7 @@ void ST_diffDraw(void)
 
 void ST_Drawer(boolean st_statusbaron, boolean refresh)
 {
-  /* cph - let status bar on be controlled
+  /* cph - let status bar on be controlled 
    * completely by the call from D_Display
    * proff - really do it
    */
@@ -831,20 +831,12 @@ void ST_Drawer(boolean st_statusbaron, boolean refresh)
 
   ST_doPaletteStuff();  // Do red-/gold-shifts from damage/items
 
-#ifdef GL_DOOM
-  /* proff 05/17/2000: always draw everything in OpenGL, because there
-   * is no backbuffer
-   */
-  if (st_statusbaron)
-    ST_doRefresh();
-#else
   if (st_statusbaron) {
-    if (st_firsttime)
+    if (st_firsttime || (V_GetMode() == VID_MODEGL))
       ST_doRefresh();     /* If just after ST_Start(), refresh all */
     else
       ST_diffDraw();      /* Otherwise, update as little as possible */
   }
-#endif
 }
 
 
@@ -859,13 +851,8 @@ static void ST_loadGraphics(boolean doload)
 {
   int  i, facenum;
   char namebuf[9];
-  // cph - macro that either acquires a pointer and lock for a lump, or
+  // cph - macro that either acquires a pointer and lock for a lump, or 
   // unlocks it. var is referenced exactly once in either case, so ++ in arg works
-/*
-#define LOADORFREE(var,name) \
-if (!doload) { W_UnlockLumpName(name); var = NULL; } \
-else var = (const patch_t*)W_CacheLumpName(name)
-*/
 
   // Load the numbers, tall and short
   for (i=0;i<10;i++)
@@ -876,7 +863,7 @@ else var = (const patch_t*)W_CacheLumpName(name)
       R_SetPatchNum(&shortnum[i],namebuf);
     }
 
-  // Load percent key.
+  // Load percent key. 
   R_SetPatchNum(&tallpercent,"STTPRCNT");
 
   // key cards
@@ -906,14 +893,6 @@ else var = (const patch_t*)W_CacheLumpName(name)
   // player face backgrounds and using displayplayer to choose them:
   R_SetPatchNum(&faceback, "STFB0");
 
-  // status bar background bits
-  if (doload)
-    sbar = V_PatchToBlock("STBAR", CR_DEFAULT, VPT_NONE,
-      &sbar_width, &sbar_height);
-  else {
-    free(sbar); sbar=NULL;
-  }
-
   // face states
   facenum = 0;
   for (i=0;i<ST_NUMPAINFACES;i++)
@@ -1158,7 +1137,35 @@ void ST_Init(void)
 {
   veryfirsttime = 0;
   ST_loadData();
-// proff 08/18/98: Changed for high-res
-  screens[4] = Z_Malloc(SCREENWIDTH*(ST_SCALED_HEIGHT+1), PU_STATIC, 0);
-//  screens[4] = Z_Malloc(ST_WIDTH*ST_HEIGHT, PU_STATIC, 0);
+}
+
+CONSOLE_INT(ammo_red, ammo_red, NULL, 0, 100, NULL, 0) { }
+CONSOLE_INT(ammo_yellow, ammo_yellow, NULL, 0, 100, NULL, 0) { }
+CONSOLE_INT(health_red, health_red, NULL, 0, 200, NULL, 0) { }
+CONSOLE_INT(health_yellow, health_yellow, NULL, 0, 200, NULL, 0) { }
+CONSOLE_INT(health_green, health_green, NULL, 0, 200, NULL, 0) { }
+CONSOLE_INT(armor_red, armor_red, NULL, 0, 200, NULL, 0) { }
+CONSOLE_INT(armor_yellow, armor_yellow, NULL, 0, 200, NULL, 0) { }
+CONSOLE_INT(armor_green, armor_green, NULL, 0, 200, NULL, 0) { }
+
+CONSOLE_BOOLEAN(st_graypct, sts_pct_always_gray, NULL, yesno, 0) { }
+CONSOLE_BOOLEAN(st_rednum, sts_always_red, NULL, yesno, 0) {}
+CONSOLE_BOOLEAN(st_singlekey, sts_traditional_keys, NULL, yesno, 0) { }
+
+void ST_AddCommands()
+{
+  C_AddCommand(ammo_red);
+  C_AddCommand(ammo_yellow);
+  
+  C_AddCommand(health_red);
+  C_AddCommand(health_yellow);
+  C_AddCommand(health_green);
+  
+  C_AddCommand(armor_red);
+  C_AddCommand(armor_yellow);
+  C_AddCommand(armor_green);
+  
+  C_AddCommand(st_graypct);
+  C_AddCommand(st_rednum);
+  C_AddCommand(st_singlekey);
 }
