@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -42,8 +42,12 @@
 #include "lprintf.h"  // jff 08/03/98 - declaration of lprintf
 #include "r_draw.h"
 
+#ifdef COMPILE_VIDD
+#include "vidd/vidd.h"
+#endif
+
 // Ty 03/17/98: flag that new par times have been loaded in d_deh
-extern boolean deh_pars;
+extern boolean deh_pars;  
 
 //
 // Data needed to add patches to full screen intermission pics.
@@ -63,11 +67,11 @@ extern boolean deh_pars;
 
 // Not used
 // in tics
-//U #define PAUSELEN    (TICRATE*2)
+//U #define PAUSELEN    (TICRATE*2) 
 //U #define SCORESTEP    100
 //U #define ANIMPERIOD    32
 // pixel distance from "(YOU)" to "PLAYER N"
-//U #define STARDIST  10
+//U #define STARDIST  10 
 //U #define WK 1
 
 
@@ -87,7 +91,7 @@ extern boolean deh_pars;
 
 // NET GAME STUFF
 #define NG_STATSY     50
-#define NG_STATSX     (32 + V_NamePatchWidth(star)/2 + 32*!dofrags)
+#define NG_STATSX     (32 + R_NamePatchWidth(star)/2 + 32*!dofrags)
 
 #define NG_SPACINGX   64
 
@@ -149,12 +153,12 @@ typedef struct
   // ALWAYS: n/a,
   // RANDOM: random base period,
   // LEVEL: n/a
-  int   data2;
+  int   data2; 
 
   /* actual graphics for frames of animations
    * cphipps - const
    */
-  const patch_t*  p[3];
+  patchnum_t p[3]; 
 
   // following must be initialized to zero before use!
 
@@ -166,9 +170,9 @@ typedef struct
 
   // next frame number to animate
   int   ctr;
-
+  
   // used by RANDOM and LEVEL when animating
-  int   state;
+  int   state;  
 } anim_t;
 
 
@@ -186,7 +190,7 @@ static point_t lnodes[NUMEPISODES][NUMMAPS] =
     { 135, 29 },  // location of level 7 (CJ)
     { 71, 24 }    // location of level 8 (CJ)
   },
-
+  
   // Episode 1 World Map should go here
   {
     { 254, 25 },  // location of level 0 (CJ)
@@ -199,7 +203,7 @@ static point_t lnodes[NUMEPISODES][NUMMAPS] =
     { 148, 140 }, // location of level 7 (CJ)
     { 235, 158 }  // location of level 8 (CJ)
   },
-
+  
   // Episode 2 World Map should go here
   {
     { 156, 168 }, // location of level 0 (CJ)
@@ -286,8 +290,8 @@ static anim_t *anims[NUMEPISODES] =
 #define SP_KILLS    0
 #define SP_ITEMS    2
 #define SP_SECRET   4
-#define SP_FRAGS    6
-#define SP_TIME     8
+#define SP_FRAGS    6 
+#define SP_TIME     8 
 #define SP_PAR      ST_TIME
 
 #define SP_PAUSE    1
@@ -312,13 +316,13 @@ static wbstartstruct_t* wbs;
 static wbplayerstruct_t* plrs;  // wbs->plyr[]
 
 // used for general timing
-static int    cnt;
+static int    cnt;  
 
 // used for timing of background animation
 static int    bcnt;
 
 // signals to refresh everything for one frame
-static int    firstrefresh;
+static int    firstrefresh; 
 
 static int    cnt_time;
 static int    cnt_total_time;
@@ -331,7 +335,7 @@ static int    cnt_pause;
 
 // You Are Here graphic
 static const char* yah[2] = { "WIURH0", "WIURH1" };
-
+  
 // splat
 static const char* splat = "WISPLAT";
 
@@ -340,7 +344,7 @@ static const char percent[] = {"WIPCNT"};
 static const char colon[] = {"WICOLON"};
 
 // 0-9 graphic
-static const patch_t  * num[10];
+static patchnum_t num[10];
 
 // minus sign
 static const char wiminus[] = {"WIMINUS"};
@@ -367,7 +371,7 @@ static const char sucks[] = {"WISUCKS"};
 
 // "killers", "victims"
 static const char killers[] = {"WIKILRS"};
-static const char victims[] = {"WIVCTMS"};
+static const char victims[] = {"WIVCTMS"}; 
 
 // "Total", your face, your dead face
 static const char total[] = {"WIMSTT"};
@@ -381,6 +385,81 @@ static const char facebackp[] = {"STPB0"};
 // CODE
 //
 
+/**********************
+
+  INTERMISSION CAMERA
+
+***********************/
+
+// is this just some boring picture, or a view of the level?
+static int realbackdrop=1;
+camera_t        intercam;
+
+mapthing_t *camerathing[MAXCAMERAS];
+mapthing_t *wi_camera;
+int numcameraviews=0;
+
+void WI_StopCamera()
+{
+  numcameraviews = 0;
+  realbackdrop = 0;
+}
+
+void WI_TickerCamera()
+{
+  if(!realbackdrop) return;
+}
+
+void WI_AddCamera(mapthing_t *mthing)
+{
+  camerathing[numcameraviews] = mthing;
+  numcameraviews++;
+}
+
+// set up the intermissions camera
+
+void WI_StartCamera()
+{
+  int i;
+  
+  if(numcameraviews)
+    {
+      realbackdrop = 1;
+      // pick a camera at random
+      wi_camera = camerathing[M_Random() % numcameraviews];
+      
+      // centre the view
+      // players[displayplayer].updownangle = 0;
+      
+      // remove the player mobjs (look silly in camera view)
+      for(i=0; i<MAXPLAYERS; i++)
+	{
+	  if(!playeringame[i]) continue;
+	  // this is strange. the monsters can still
+	  // see the player Mobj, (and even kill it!)
+	  // even tho it has been removed from the
+	  // level. I make it unshootable first so
+	  // they lose interest.
+	  players[i].mo->flags &= ~MF_SHOOTABLE;
+	  P_RemoveMobj(players[i].mo);
+	}
+      
+      
+      intercam.x = wi_camera->x*FRACUNIT;
+      intercam.y = wi_camera->y*FRACUNIT;
+      intercam.angle = R_WadToAngle(wi_camera->angle);
+      intercam.updownangle = 0;
+      intercam.z = R_PointInSubsector(intercam.x, intercam.y)
+	->sector->floorheight + 41*FRACUNIT;
+      R_SetViewSize (11);     // force fullscreen
+    }
+  else            // no camera, boring interpic
+    {
+      realbackdrop = 0;
+      wi_camera = NULL;
+    }
+}
+
 static void WI_endDeathmatchStats(void);
 static void WI_endNetgameStats(void);
 void WI_unloadData(void);
@@ -388,7 +467,7 @@ void WI_unloadData(void);
 
 /* ====================================================================
  * WI_levelNameLump
- * Purpore: Returns the name of the graphic lump containing the name of
+ * Purpore: Returns the name of the graphic lump containing the name of 
  *          the given level.
  * Args:    Episode and level, and buffer (must by 9 chars) to write to
  * Returns: void
@@ -396,7 +475,7 @@ void WI_unloadData(void);
 void WI_levelNameLump(int epis, int map, char* buf)
 {
   if (gamemode == commercial) {
-    sprintf(buf, "CWILV%2.2d", map);
+    sprintf(buf, "CWILV%2.2d", map);    
   } else {
     sprintf(buf, "WILV%d%d", epis, map);
   }
@@ -414,7 +493,7 @@ static void WI_slamBackground(void)
 
   if (gamemode == commercial || (gamemode == retail && wbs->epsd == 3))
     strcpy(name, "INTERPIC");
-  else
+  else 
     sprintf(name, "WIMAP%d", wbs->epsd);
 
   // background
@@ -447,19 +526,19 @@ void WI_drawLF(void)
   int y = WI_TITLEY;
   char lname[9];
 
-  // draw <LevelName>
+  // draw <LevelName> 
   /* cph - get the graphic lump name and use it */
   WI_levelNameLump(wbs->epsd, wbs->last, lname);
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - V_NamePatchWidth(lname))/2, y,
-     FB, lname, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - R_NamePatchWidth(lname))/2, y, 
+		 FB, lname, CR_DEFAULT, VPT_STRETCH);
 
   // draw "Finished!"
-  y += (5*V_NamePatchHeight(lname))/4;
-
+  y += (5*R_NamePatchHeight(lname))/4;
+  
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - V_NamePatchWidth(finished))/2, y,
-     FB, finished, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - R_NamePatchWidth(finished))/2, y, 
+		 FB, finished, CR_DEFAULT, VPT_STRETCH);
 }
 
 
@@ -479,15 +558,15 @@ void WI_drawEL(void)
 
   // draw "Entering"
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - V_NamePatchWidth(entering))/2,
-      y, FB, entering, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - R_NamePatchWidth(entering))/2, 
+		  y, FB, entering, CR_DEFAULT, VPT_STRETCH);
 
   // draw level
-  y += (5*V_NamePatchHeight(lname))/4;
+  y += (5*R_NamePatchHeight(lname))/4;
 
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - V_NamePatchWidth(lname))/2, y, FB,
-     lname, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - R_NamePatchWidth(lname))/2, y, FB, 
+		 lname, CR_DEFAULT, VPT_STRETCH);
 }
 
 
@@ -514,18 +593,17 @@ WI_drawOnLnode  // draw stuff at a location by episode/map#
     int            right;
     int            bottom;
     int            lump = W_GetNumForName(c[i]);
-    const patch_t* p = W_CacheLumpNum(lump);
-
-    left = lnodes[wbs->epsd][n].x - SHORT(p->leftoffset);
-    top = lnodes[wbs->epsd][n].y - SHORT(p->topoffset);
-    right = left + SHORT(p->width);
-    bottom = top + SHORT(p->height);
-    W_UnlockLumpNum(lump);
+    const TPatch *patch = R_GetPatch(lump); // POPE
 
+    left = lnodes[wbs->epsd][n].x - SHORT(patch->leftOffset);
+    top = lnodes[wbs->epsd][n].y - SHORT(patch->topOffset);
+    right = left + SHORT(patch->width);
+    bottom = top + SHORT(patch->height);
+  
     if (left >= 0
-       && right < 320
+       && right < SCREENWIDTH
        && top >= 0
-       && bottom < 200)
+       && bottom < SCREENHEIGHT)
     {
       fits = true;
     }
@@ -539,7 +617,7 @@ WI_drawOnLnode  // draw stuff at a location by episode/map#
   {
     // CPhipps - patch drawing updated
     V_DrawNamePatch(lnodes[wbs->epsd][n].x, lnodes[wbs->epsd][n].y,
-       FB, c[i], CR_DEFAULT, VPT_STRETCH);
+		   FB, c[i], CR_DEFAULT, VPT_STRETCH);
   }
   else
   {
@@ -556,7 +634,7 @@ WI_drawOnLnode  // draw stuff at a location by episode/map#
 // Args:    none
 // Returns: void
 //
-void WI_initAnimatedBack(void)
+static void WI_initAnimatedBack(void)
 {
   int   i;
   anim_t* a;
@@ -573,14 +651,14 @@ void WI_initAnimatedBack(void)
 
     // init variables
     a->ctr = -1;
-
+  
     // specify the next time to draw it
     if (a->type == ANIM_ALWAYS)
       a->nexttic = bcnt + 1 + (M_Random()%a->period);
-    else
+    else 
       if (a->type == ANIM_RANDOM)
         a->nexttic = bcnt + 1 + a->data2+(M_Random()%a->data1);
-      else
+      else 
         if (a->type == ANIM_LEVEL)
           a->nexttic = bcnt + 1;
   }
@@ -593,7 +671,7 @@ void WI_initAnimatedBack(void)
 // Args:    none
 // Returns: void
 //
-void WI_updateAnimatedBack(void)
+static void WI_updateAnimatedBack(void)
 {
   int   i;
   anim_t* a;
@@ -624,10 +702,10 @@ void WI_updateAnimatedBack(void)
                a->ctr = -1;
                a->nexttic = bcnt+a->data2+(M_Random()%a->data1);
              }
-             else
+             else 
                a->nexttic = bcnt + a->period;
              break;
-
+    
         case ANIM_LEVEL:
              // gawd-awful hack for level anims
              if (!(state == StatCount && i == 7)
@@ -650,7 +728,7 @@ void WI_updateAnimatedBack(void)
 // Args:    none
 // Returns: void
 //
-void WI_drawAnimatedBack(void)
+static void WI_drawAnimatedBack(void)
 {
   int     i;
   anim_t*   a;
@@ -667,7 +745,7 @@ void WI_drawAnimatedBack(void)
 
     if (a->ctr >= 0)
       // CPhipps - patch drawing updated
-      V_DrawMemPatch(a->loc.x, a->loc.y, FB, a->p[a->ctr], CR_DEFAULT, VPT_STRETCH);
+      V_DrawNumPatch(a->loc.x, a->loc.y, FB, a->p[a->ctr].lumpnum, CR_DEFAULT, VPT_STRETCH);
   }
 }
 
@@ -683,7 +761,7 @@ void WI_drawAnimatedBack(void)
 // CPhipps - static
 static int WI_drawNum (int x, int y, int n, int digits)
 {
-  int   fontwidth = SHORT(num[0]->width);
+  int   fontwidth = SHORT(num[0].width);
   int   neg;
   int   temp;
 
@@ -721,7 +799,7 @@ static int WI_drawNum (int x, int y, int n, int digits)
   {
     x -= fontwidth;
     // CPhipps - patch drawing updated
-    V_DrawMemPatch(x, y, FB, num[ n % 10 ], CR_DEFAULT, VPT_STRETCH);
+    V_DrawNumPatch(x, y, FB, num[n % 10].lumpnum, CR_DEFAULT, VPT_STRETCH);
     n /= 10;
   }
 
@@ -736,7 +814,7 @@ static int WI_drawNum (int x, int y, int n, int digits)
 
 // ====================================================================
 // WI_drawPercent
-// Purpose: Draws a percentage, really just a call to WI_drawNum
+// Purpose: Draws a percentage, really just a call to WI_drawNum 
 //          after putting a percent sign out there
 // Args:    x, y   -- location
 //          p      -- the percentage value to be drawn, no negatives
@@ -765,7 +843,7 @@ static void WI_drawPercent(int x, int y, int p)
 //         - largely rewritten to display hours and use slightly better algorithm
 
 static void WI_drawTime(int x, int y, int t)
-{
+{ 
   int   n;
 
   if (t<0)
@@ -775,17 +853,17 @@ static void WI_drawTime(int x, int y, int t)
     for(;;) {
       n = t % 60;
       t /= 60;
-      x = WI_drawNum(x, y, n, (t || n>9) ? 2 : 1) - V_NamePatchWidth(colon);
+      x = WI_drawNum(x, y, n, (t || n>9) ? 2 : 1) - R_NamePatchWidth(colon);
 
       // draw
       if (t)
-  // CPhipps - patch drawing updated
+	// CPhipps - patch drawing updated
         V_DrawNamePatch(x, y, FB, colon, CR_DEFAULT, VPT_STRETCH);
       else break;
-    }
+    } 
   else // "sucks" (maybe should be "addicted", even I've never had a 100 hour game ;)
-    V_DrawNamePatch(x - V_NamePatchWidth(sucks),
-        y, FB, sucks, CR_DEFAULT, VPT_STRETCH);
+    V_DrawNamePatch(x - R_NamePatchWidth(sucks), 
+		    y, FB, sucks, CR_DEFAULT, VPT_STRETCH); 
 }
 
 
@@ -814,7 +892,7 @@ void WI_End(void)
 // Args:    none
 // Returns: void
 //
-void WI_initNoState(void)
+static void WI_initNoState(void)
 {
   state = NoState;
   acceleratestage = 0;
@@ -838,12 +916,12 @@ static void WI_drawTimeStats(int cnt_time, int cnt_total_time, int cnt_par)
   V_DrawNamePatch(SP_TIMEX, (SP_TIMEY+200)/2, FB, total, CR_DEFAULT, VPT_STRETCH);
   WI_drawTime(320/2 - SP_TIMEX, (SP_TIMEY+200)/2, cnt_total_time);
 
-  // Ty 04/11/98: redid logic: should skip only if with pwad but
+  // Ty 04/11/98: redid logic: should skip only if with pwad but 
   // without deh patch
   // killough 2/22/98: skip drawing par times on pwads
   // Ty 03/17/98: unless pars changed with deh patch
 
-  if (!(modifiedgame && !deh_pars))
+  if (!(modifiedgame && !deh_pars))   
   {
     if (wbs->epsd < 3)
     {
@@ -859,7 +937,7 @@ static void WI_drawTimeStats(int cnt_time, int cnt_total_time, int cnt_par)
 // Args:    none
 // Returns: void
 //
-void WI_updateNoState(void)
+void WI_updateNoState(void) 
 {
 
   WI_updateAnimatedBack();
@@ -873,11 +951,11 @@ static boolean    snl_pointeron = false;
 
 // ====================================================================
 // WI_initShowNextLoc
-// Purpose: Prepare to show the next level's location
+// Purpose: Prepare to show the next level's location 
 // Args:    none
 // Returns: void
 //
-void WI_initShowNextLoc(void)
+static void WI_initShowNextLoc(void)
 {
   if ((gamemode != commercial) && (gamemap == 8)) {
     G_WorldDone();
@@ -898,7 +976,7 @@ void WI_initShowNextLoc(void)
 // Args:    none
 // Returns: void
 //
-void WI_updateShowNextLoc(void)
+static void WI_updateShowNextLoc(void)
 {
   WI_updateAnimatedBack();
 
@@ -915,7 +993,7 @@ void WI_updateShowNextLoc(void)
 // Args:    none
 // Returns: void
 //
-void WI_drawShowNextLoc(void)
+static void WI_drawShowNextLoc(void)
 {
   int   i;
   int   last;
@@ -923,7 +1001,7 @@ void WI_drawShowNextLoc(void)
   WI_slamBackground();
 
   // draw animated background
-  WI_drawAnimatedBack();
+  WI_drawAnimatedBack(); 
 
   if ( gamemode != commercial)
   {
@@ -932,7 +1010,7 @@ void WI_drawShowNextLoc(void)
       WI_drawEL();  // "Entering..." if not E1 or E2
       return;
     }
-
+  
     last = (wbs->last == 8) ? wbs->next - 1 : wbs->last;
 
     // draw a splat on taken cities.
@@ -945,13 +1023,13 @@ void WI_drawShowNextLoc(void)
 
     // draw flashing ptr
     if (snl_pointeron)
-      WI_drawOnLnode(wbs->next, yah);
+      WI_drawOnLnode(wbs->next, yah); 
   }
 
   // draws which level you are entering..
   if ( (gamemode != commercial)
      || wbs->next != 30)  // check for MAP30 end game
-  WI_drawEL();
+  WI_drawEL();  
 }
 
 // ====================================================================
@@ -960,7 +1038,7 @@ void WI_drawShowNextLoc(void)
 // Args:    none
 // Returns: void
 //
-void WI_drawNoState(void)
+static void WI_drawNoState(void)
 {
   snl_pointeron = true;
   WI_drawShowNextLoc();
@@ -973,21 +1051,21 @@ void WI_drawNoState(void)
 // Args:    playernum -- the player to be calculated
 // Returns: the total frags for this player
 //
-int WI_fragSum(int playernum)
+static int WI_fragSum(int playernum)
 {
   int   i;
   int   frags = 0;
-
+    
   for (i=0 ; i<MAXPLAYERS ; i++)
   {
     if (playeringame[i]  // is this player playing?
        && i!=playernum) // and it's not the player we're calculating
     {
-      frags += plrs[playernum].frags[i];
+      frags += plrs[playernum].frags[i];  
     }
   }
 
-
+  
   // JDC hack - negative frags.
   frags -= plrs[playernum].frags[playernum];
 
@@ -1001,12 +1079,12 @@ static short int   *dm_totals;  // totals by player
 
 // ====================================================================
 // WI_initDeathmatchStats
-// Purpose: Set up to display DM stats at end of level.  Calculate
+// Purpose: Set up to display DM stats at end of level.  Calculate 
 //          frags for all players.
 // Args:    none
 // Returns: void
 //
-void WI_initDeathmatchStats(void)
+static void WI_initDeathmatchStats(void)
 {
   int   i; // looping variables
 
@@ -1023,7 +1101,7 @@ void WI_initDeathmatchStats(void)
   for (i=0 ; i<MAXPLAYERS ; i++)
   {
     if (playeringame[i])
-    {
+    { 
       // CPhipps - allocate frags line
       dm_frags[i] = calloc(MAXPLAYERS, sizeof(**dm_frags)); // set all counts to zero
 
@@ -1051,21 +1129,21 @@ void WI_endDeathmatchStats(void)
 
 // ====================================================================
 // WI_updateDeathmatchStats
-// Purpose: Advance Deathmatch stats screen animation.  Calculate
+// Purpose: Advance Deathmatch stats screen animation.  Calculate 
 //          frags for all players.  Lots of noise and drama around
 //          the presentation.
 // Args:    none
 // Returns: void
 //
-void WI_updateDeathmatchStats(void)
+static void WI_updateDeathmatchStats(void)
 {
   int   i;
   int   j;
-
+    
   boolean stillticking;
-
+  
   WI_updateAnimatedBack();
-
+  
   if (acceleratestage && dm_state != 4)  // still ticking
   {
     acceleratestage = 0;
@@ -1081,18 +1159,18 @@ void WI_updateDeathmatchStats(void)
         dm_totals[i] = WI_fragSum(i);
       }
     }
-
+  
 
     S_StartSound(0, sfx_barexp);  // bang
     dm_state = 4;  // we're done with all 4 (or all we have to do)
   }
 
-
+    
   if (dm_state == 2)
   {
     if (!(bcnt&3))
       S_StartSound(0, sfx_pistol);  // noise while counting
-
+  
     stillticking = false;
 
     for (i=0 ; i<MAXPLAYERS ; i++)
@@ -1114,7 +1192,7 @@ void WI_updateDeathmatchStats(void)
 
             if (dm_frags[i][j] < -999)
               dm_frags[i][j] = -999;
-
+      
             stillticking = true;
           }
         }
@@ -1122,7 +1200,7 @@ void WI_updateDeathmatchStats(void)
 
         if (dm_totals[i] > 999)
           dm_totals[i] = 999;
-
+    
         if (dm_totals[i] < -999)
           dm_totals[i] = -999;  // Ty 03/17/98 end 3-digit frag count
       }
@@ -1137,7 +1215,7 @@ void WI_updateDeathmatchStats(void)
   else if (dm_state == 4)
   {
     if (acceleratestage)
-    {
+    {   
       S_StartSound(0, sfx_slop);
 
       if ( gamemode == commercial)
@@ -1172,22 +1250,22 @@ void WI_drawDeathmatchStats(void)
   int   x;
   int   y;
   int   w;
-
+  
   int   lh; // line height
-  int   halfface = V_NamePatchWidth(facebackp)/2;
+  int   halfface = R_NamePatchWidth(facebackp)/2;
 
   lh = WI_SPACINGY;
 
   WI_slamBackground();
-
+  
   // draw animated background
-  WI_drawAnimatedBack();
+  WI_drawAnimatedBack(); 
   WI_drawLF();
 
   // draw stat titles (top line)
-  V_DrawNamePatch(DM_TOTALSX-V_NamePatchWidth(total)/2,
-     DM_MATRIXY-WI_SPACINGY+10, FB, total, CR_DEFAULT, VPT_STRETCH);
-
+  V_DrawNamePatch(DM_TOTALSX-R_NamePatchWidth(total)/2,
+		 DM_MATRIXY-WI_SPACINGY+10, FB, total, CR_DEFAULT, VPT_STRETCH);
+  
   V_DrawNamePatch(DM_KILLERSX, DM_KILLERSY, FB, killers, CR_DEFAULT, VPT_STRETCH);
   V_DrawNamePatch(DM_VICTIMSX, DM_VICTIMSY, FB, victims, CR_DEFAULT, VPT_STRETCH);
 
@@ -1199,19 +1277,19 @@ void WI_drawDeathmatchStats(void)
   {
     if (playeringame[i]) {
       //int trans = playernumtotrans[i];
-      V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY,
-         FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
-         VPT_STRETCH | (i ? VPT_TRANS : 0));
-      V_DrawNamePatch(DM_MATRIXX-halfface, y,
-         FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
-         VPT_STRETCH | (i ? VPT_TRANS : 0));
+      V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY, 
+		     FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
+		     VPT_STRETCH | (i ? VPT_TRANS : 0));
+      V_DrawNamePatch(DM_MATRIXX-halfface, y, 
+		     FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
+		     VPT_STRETCH | (i ? VPT_TRANS : 0));
 
       if (i == me)
       {
-        V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY,
-           FB, bstar, CR_DEFAULT, VPT_STRETCH);
-        V_DrawNamePatch(DM_MATRIXX-halfface, y,
-           FB, star, CR_DEFAULT, VPT_STRETCH);
+        V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY, 
+		       FB, bstar, CR_DEFAULT, VPT_STRETCH);
+        V_DrawNamePatch(DM_MATRIXX-halfface, y, 
+		       FB, star, CR_DEFAULT, VPT_STRETCH);
       }
     }
     x += DM_SPACINGX;
@@ -1220,7 +1298,7 @@ void WI_drawDeathmatchStats(void)
 
   // draw stats
   y = DM_MATRIXY+10;
-  w = SHORT(num[0]->width);
+  w = num[0].width;
 
   for (i=0 ; i<MAXPLAYERS ; i++)
   {
@@ -1272,7 +1350,7 @@ static void WI_endNetgameStats(void)
 // Args:    none
 // Returns: void
 //
-void WI_initNetgameStats(void)
+static void WI_initNetgameStats(void)
 {
   int i;
 
@@ -1305,11 +1383,11 @@ void WI_initNetgameStats(void)
 // Returns: void
 // Comment: This stuff sure is complicated for what it does
 //
-void WI_updateNetgameStats(void)
+static void WI_updateNetgameStats(void)
 {
   int   i;
   int   fsum;
-
+    
   boolean stillticking;
 
   WI_updateAnimatedBack();
@@ -1327,7 +1405,7 @@ void WI_updateNetgameStats(void)
       cnt_items[i] = (plrs[i].sitems * 100) / wbs->maxitems;
 
       // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
-      cnt_secret[i] = wbs->maxsecret ?
+      cnt_secret[i] = wbs->maxsecret ? 
                       (plrs[i].ssecret * 100) / wbs->maxsecret : 100;
       if (dofrags)
         cnt_frags[i] = WI_fragSum(i);  // we had frags
@@ -1355,10 +1433,10 @@ void WI_updateNetgameStats(void)
       else
         stillticking = true; // still got stuff to tally
     }
-
+  
     if (!stillticking)
     {
-      S_StartSound(0, sfx_barexp);
+      S_StartSound(0, sfx_barexp); 
       ng_state++;
     }
   }
@@ -1366,21 +1444,21 @@ void WI_updateNetgameStats(void)
   {
     if (!(bcnt&3))
       S_StartSound(0, sfx_pistol);
-
+  
     stillticking = false;
-
+  
     for (i=0 ; i<MAXPLAYERS ; i++)
     {
       if (!playeringame[i])
         continue;
-
+  
       cnt_items[i] += 2;
       if (cnt_items[i] >= (plrs[i].sitems * 100) / wbs->maxitems)
         cnt_items[i] = (plrs[i].sitems * 100) / wbs->maxitems;
       else
         stillticking = true;
     }
-
+  
     if (!stillticking)
     {
       S_StartSound(0, sfx_barexp);
@@ -1403,12 +1481,12 @@ void WI_updateNetgameStats(void)
 
       // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
 
-      if (cnt_secret[i] >= (wbs->maxsecret ? (plrs[i].ssecret * 100) / wbs->maxsecret : compatibility_level < lxdoom_1_compatibility ? 0 : 100))
+      if (cnt_secret[i] >= (wbs->maxsecret ? (plrs[i].ssecret * 100) / wbs->maxsecret : 100))
         cnt_secret[i] = wbs->maxsecret ? (plrs[i].ssecret * 100) / wbs->maxsecret : 100;
       else
         stillticking = true;
     }
-
+  
     if (!stillticking)
     {
       S_StartSound(0, sfx_barexp);
@@ -1434,7 +1512,7 @@ void WI_updateNetgameStats(void)
       else
         stillticking = true;
     }
-
+      
     if (!stillticking)
     {
       S_StartSound(0, sfx_pldeth);
@@ -1476,32 +1554,32 @@ void WI_drawNetgameStats(void)
   int   i;
   int   x;
   int   y;
-  int   pwidth = V_NamePatchWidth(percent);
-  int   fwidth = V_NamePatchWidth(facebackp);
+  int   pwidth = R_NamePatchWidth(percent);
+  int   fwidth = R_NamePatchWidth(facebackp);
 
   WI_slamBackground();
-
+  
   // draw animated background
-  WI_drawAnimatedBack();
+  WI_drawAnimatedBack(); 
 
   WI_drawLF();
 
   // draw stat titles (top line)
-  V_DrawNamePatch(NG_STATSX+NG_SPACINGX-V_NamePatchWidth(kills),
-     NG_STATSY, FB, kills, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch(NG_STATSX+NG_SPACINGX-R_NamePatchWidth(kills),
+		 NG_STATSY, FB, kills, CR_DEFAULT, VPT_STRETCH);
 
-  V_DrawNamePatch(NG_STATSX+2*NG_SPACINGX-V_NamePatchWidth(items),
-     NG_STATSY, FB, items, CR_DEFAULT, VPT_STRETCH);
-
-  V_DrawNamePatch(NG_STATSX+3*NG_SPACINGX-V_NamePatchWidth(secret),
-     NG_STATSY, FB, secret, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch(NG_STATSX+2*NG_SPACINGX-R_NamePatchWidth(items),
+		 NG_STATSY, FB, items, CR_DEFAULT, VPT_STRETCH);
 
+  V_DrawNamePatch(NG_STATSX+3*NG_SPACINGX-R_NamePatchWidth(secret),
+		 NG_STATSY, FB, secret, CR_DEFAULT, VPT_STRETCH);
+  
   if (dofrags)
-    V_DrawNamePatch(NG_STATSX+4*NG_SPACINGX-V_NamePatchWidth(frags),
-       NG_STATSY, FB, frags, CR_DEFAULT, VPT_STRETCH);
+    V_DrawNamePatch(NG_STATSX+4*NG_SPACINGX-R_NamePatchWidth(frags),
+		   NG_STATSY, FB, frags, CR_DEFAULT, VPT_STRETCH);
 
   // draw stats
-  y = NG_STATSY + V_NamePatchHeight(kills);
+  y = NG_STATSY + R_NamePatchHeight(kills);
 
   for (i=0 ; i<MAXPLAYERS ; i++)
   {
@@ -1510,9 +1588,9 @@ void WI_drawNetgameStats(void)
       continue;
 
     x = NG_STATSX;
-    V_DrawNamePatch(x-fwidth, y, FB, facebackp,
-       i ? CR_LIMIT+i : CR_DEFAULT,
-       VPT_STRETCH | (i ? VPT_TRANS : 0));
+    V_DrawNamePatch(x-fwidth, y, FB, facebackp, 
+		   i ? CR_LIMIT+i : CR_DEFAULT,
+		   VPT_STRETCH | (i ? VPT_TRANS : 0));
 
     if (i == me)
       V_DrawNamePatch(x-fwidth, y, FB, star, CR_DEFAULT, VPT_STRETCH);
@@ -1543,14 +1621,14 @@ static int  sp_state;
 // Comment: Seems like we could do all these stats in a more generic
 //          set of routines that weren't duplicated for dm, coop, sp
 //
-void WI_initStats(void)
+static void WI_initStats(void)
 {
   state = StatCount;
   acceleratestage = 0;
   sp_state = 1;
 
   // CPhipps - allocate (awful code, I know, but saves changing it all) and initialise
-  *(cnt_kills = malloc(sizeof(*cnt_kills))) =
+  *(cnt_kills = malloc(sizeof(*cnt_kills))) = 
   *(cnt_items = malloc(sizeof(*cnt_items))) =
   *(cnt_secret= malloc(sizeof(*cnt_secret))) = -1;
   cnt_time = cnt_par = cnt_total_time = -1;
@@ -1565,7 +1643,7 @@ void WI_initStats(void)
 // Args:    none
 // Returns: void
 //
-void WI_updateStats(void)
+static void WI_updateStats(void)
 {
   WI_updateAnimatedBack();
 
@@ -1576,7 +1654,7 @@ void WI_updateStats(void)
     cnt_items[0] = (plrs[me].sitems * 100) / wbs->maxitems;
 
     // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
-    cnt_secret[0] = (wbs->maxsecret ?
+    cnt_secret[0] = (wbs->maxsecret ? 
       (plrs[me].ssecret * 100) / wbs->maxsecret : 100);
 
     cnt_total_time = wbs->totaltimes / TICRATE;
@@ -1622,11 +1700,10 @@ void WI_updateStats(void)
       S_StartSound(0, sfx_pistol);
 
     // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
-    if ((!wbs->maxsecret && compatibility_level < lxdoom_1_compatibility) ||
-	cnt_secret[0] >= (wbs->maxsecret ?
+    if (cnt_secret[0] >= (wbs->maxsecret ? 
       (plrs[me].ssecret * 100) / wbs->maxsecret : 100))
     {
-      cnt_secret[0] = (wbs->maxsecret ?
+      cnt_secret[0] = (wbs->maxsecret ? 
         (plrs[me].ssecret * 100) / wbs->maxsecret : 100);
       S_StartSound(0, sfx_barexp);
       sp_state++;
@@ -1693,15 +1770,15 @@ void WI_updateStats(void)
 void WI_drawStats(void)
 {
   // line height
-  int lh;
+  int lh; 
 
-  lh = (3*SHORT(num[0]->height))/2;
+  lh = (3*num[0].height)/2;
 
   WI_slamBackground();
 
   // draw animated background
   WI_drawAnimatedBack();
-
+  
   WI_drawLF();
 
   V_DrawNamePatch(SP_STATSX, SP_STATSY, FB, kills, CR_DEFAULT, VPT_STRETCH);
@@ -1729,6 +1806,13 @@ void WI_checkForAccelerate(void)
   int   i;
   player_t  *player;
 
+#ifdef COMPILE_VIDD
+  if (sp_state != 10 && VIDD_PLAY_inProgress() || VIDD_MENU_getAdvanceIntermission()) { // POPE
+    acceleratestage = 1;
+    return;
+  }
+#endif
+
   // check for button presses to skip delays
   for (i=0, player = players ; i<MAXPLAYERS ; i++, player++)
   {
@@ -1765,15 +1849,15 @@ void WI_checkForAccelerate(void)
 void WI_Ticker(void)
 {
   // counter for general background animation
-  bcnt++;
+  bcnt++;  
 
   if (bcnt == 1)
   {
     // intermission music
     if ( gamemode == commercial )
-      S_ChangeMusic(mus_dm2int, true);
+      S_ChangeMusicNum(mus_dm2int, true);
     else
-      S_ChangeMusic(mus_inter, true);
+      S_ChangeMusicNum(mus_inter, true); 
   }
 
   WI_checkForAccelerate();
@@ -1785,11 +1869,11 @@ void WI_Ticker(void)
          else if (netgame) WI_updateNetgameStats();
          else WI_updateStats();
          break;
-
+  
     case ShowNextLoc:
          WI_updateShowNextLoc();
          break;
-
+  
     case NoState:
          WI_updateNoState();
          break;
@@ -1803,8 +1887,8 @@ void WI_Ticker(void)
  * Args:    none
  * Returns: void
  *
- * CPhipps - modified for new wad lump handling.
- *         - no longer preload most graphics, other funcs can use
+ * CPhipps - modified for new wad lump handling. 
+ *         - no longer preload most graphics, other funcs can use 
  *           them by name
  */
 
@@ -1825,16 +1909,16 @@ void WI_loadData(void)
         for (i=0;i<a->nanims;i++)
         {
           // MONDO HACK!
-          if (wbs->epsd != 1 || j != 8)
+          if (wbs->epsd != 1 || j != 8) 
           {
             // animations
             sprintf(name, "WIA%d%.2d%.2d", wbs->epsd, j, i);
-            a->p[i] = W_CacheLumpName(name);
+            R_SetPatchNum(&a->p[i], name);
           }
           else
           {
             // HACK ALERT!
-            a->p[i] = anims[1][4].p[i];
+            a->p[i] = anims[1][4].p[i]; 
           }
         }
       }
@@ -1845,7 +1929,7 @@ void WI_loadData(void)
   {
     // numbers 0-9
     sprintf(name, "WINUM%d", i);
-    num[i] = W_CacheLumpName(name);
+    R_SetPatchNum(&num[i], name);
   }
 }
 
@@ -1865,22 +1949,22 @@ void WI_unloadData(void)
   if (gamemode != commercial) {
     if (wbs->epsd < 3) {
       for (j=0;j<NUMANIMS[wbs->epsd];j++) {
-  anim_t* a = &anims[wbs->epsd][j];
-  for (i=0; i<a->nanims; i++) {
-    // MONDO HACK!
-    if (wbs->epsd != 1 || j != 8) {
-      // animations
-      sprintf(name, "WIA%d%.2d%.2d", wbs->epsd, j, i);
-      W_UnlockLumpName(name);
-    }
-  }
+	anim_t* a = &anims[wbs->epsd][j];
+	for (i=0; i<a->nanims; i++) {
+	  // MONDO HACK!
+	  if (wbs->epsd != 1 || j != 8) {
+	    // animations
+	    sprintf(name, "WIA%d%.2d%.2d", wbs->epsd, j, i);  
+	    W_UnlockLumpName(name);
+	  }
+	}
       }
     }
   }
 
   for (i=0;i<10;i++) {
     // numbers 0-9
-    sprintf(name, "WINUM%d", i);
+    sprintf(name, "WINUM%d", i);     
     W_UnlockLumpName(name);
   }
 }
@@ -1905,11 +1989,11 @@ void WI_Drawer (void)
          else
            WI_drawStats();
          break;
-
+  
     case ShowNextLoc:
          WI_drawShowNextLoc();
          break;
-
+  
     case NoState:
          WI_drawNoState();
          break;
@@ -1924,7 +2008,7 @@ void WI_Drawer (void)
 // Args:    wbstartstruct -- pointer to the structure with the data
 // Returns: void
 //
-void WI_initVariables(wbstartstruct_t* wbstartstruct)
+static void WI_initVariables(wbstartstruct_t* wbstartstruct)
 {
 
   wbs = wbstartstruct;
@@ -1958,6 +2042,12 @@ void WI_initVariables(wbstartstruct_t* wbstartstruct)
   if (!wbs->maxitems)
     wbs->maxitems = 1;
 
+  // killough 2/22/98: Keep maxsecret=0 if it's zero, so
+  // we can detect 0/0 as as a special case and print 100%.
+  //
+  //    if (!wbs->maxsecret)
+  //  wbs->maxsecret = 1;
+
   if ( gamemode != retail )
     if (wbs->epsd > 2)
       wbs->epsd -= 3;
@@ -1967,12 +2057,14 @@ void WI_initVariables(wbstartstruct_t* wbstartstruct)
 // WI_Start
 // Purpose: Call the various init routines
 //          Note: wbstartstruct_t is defined in d_player.h
-// Args:    wbstartstruct -- pointer to the structure with the
+// Args:    wbstartstruct -- pointer to the structure with the 
 //          intermission data
 // Returns: void
 //
 void WI_Start(wbstartstruct_t* wbstartstruct)
 {
+  WI_StartCamera();  //set up camera
+
   WI_initVariables(wbstartstruct);
   WI_loadData();
 
