@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C++ -*- 
  *-----------------------------------------------------------------------------
  *
  *
@@ -8,7 +8,7 @@
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *
+ *  
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +21,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -50,6 +50,7 @@
 #include "z_zone.h"
 #include "doomstat.h"
 #include "m_argv.h"
+#include "v_video.h"
 #include "lprintf.h"
 
 #ifdef DJGPP
@@ -70,16 +71,6 @@
 // How much RAM to leave aside for other libraries
 #define LEAVE_ASIDE (128*1024)
 
-// Minimum RAM machine is assumed to have
-  /* cph - Select zone size. 6megs is usable, but with the SDL version
-   * storing sounds in the zone, 8 is more sensible */
-#ifndef GL_DOOM
-#define MIN_RAM (8*1024*1024)
-#else
-  /* proff - OpenGL needs even more ram at least 16megs are allocated */
-#define MIN_RAM (16*1024*1024)
-#endif
-
 // Amount to subtract when retrying failed attempts to allocate initial pool
 #define RETRY_AMOUNT (256*1024)
 
@@ -118,7 +109,6 @@ static const size_t HEADER_SIZE = (sizeof(memblock_t)+CHUNK_SIZE-1) & ~(CHUNK_SI
 static memblock_t *rover;                // roving pointer to memory blocks
 static memblock_t *zone;                 // pointer to first block
 static memblock_t *zonebase;             // pointer to entire zone memory
-static size_t zonebase_size;             // zone memory allocated size
 static memblock_t *blockbytag[PU_MAX];
 
 #ifdef INSTRUMENTED
@@ -130,31 +120,53 @@ static size_t purgable_memory;
 static size_t inactive_memory;
 static size_t virtual_memory;
 
-static void Z_PrintStats(void)            // Print allocation statistics
+void Z_DrawStats(void)            // Print allocation statistics
 {
+  char buffer[1024];
+
   unsigned long total_memory = free_memory + active_memory +
                                purgable_memory + inactive_memory +
                                virtual_memory;
   double s = 100.0 / total_memory;
 
-  doom_printf("%-5u\t%6.01f%%\tstatic\n"
-          "%-5u\t%6.01f%%\tpurgable\n"
-          "%-5u\t%6.01f%%\tfree\n"
-          "%-5u\t%6.01f%%\tfragmentary\n"
-          "%-5u\t%6.01f%%\tvirtual\n"
-          "%-5lu\t\ttotal\n",
+  if (gamestate != GS_LEVEL)
+    return;
+
+  sprintf(buffer,FC_GRAY
+          "%-5u\n"
+          "%-5u\n"
+          "%-5u\n"
+          "%-5u\n"
+          "%-5u\n"
+          "%-5lu\n",
           active_memory,
-          active_memory*s,
           purgable_memory,
-          purgable_memory*s,
           free_memory,
-          free_memory*s,
           inactive_memory,
-          inactive_memory*s,
           virtual_memory,
-          virtual_memory*s,
           total_memory
           );
+  V_WriteTextXYGap(buffer, 0, 16, -1, -1);
+  sprintf(buffer,FC_GRAY
+          "%6.01f%%\n"
+          "%6.01f%%\n"
+          "%6.01f%%\n"
+          "%6.01f%%\n"
+          "%6.01f%%\n",
+          active_memory*s,
+          purgable_memory*s,
+          free_memory*s,
+          inactive_memory*s,
+          virtual_memory*s
+          );
+  V_WriteTextXYGap(buffer, 45, 16, -1, -1);
+  V_WriteTextXYGap(FC_GRAY
+                   "static\n"
+                   "purgable\n"
+                   "free\n"
+                   "fragmentary\n"
+                   "virtual\n"
+                   "total\n", 90, 16, -1, -1);
 }
 
 #ifdef HEAPDUMP
@@ -170,9 +182,10 @@ void Z_DumpMemory(void)
 
   sprintf(buf, "memdump.%d", dump++);
   fp = fopen(buf, "w");
+  if (block)
   do {
     switch (block->tag) {
-    case PU_FREE:
+    case PU_FREE: 
       fprintf(fp, "free %d\n", block->size);
       total_free += block->size;
       break;
@@ -187,20 +200,27 @@ void Z_DumpMemory(void)
     default:
       fprintf(fp, "malloc %s:%d:%d", block->file, block->line, block->size);
       total_malloc += block->size;
-      if (!strcmp(block->file,"w_wad.c")) W_PrintLump(fp, (char*)block + HEADER_SIZE);
+      if (block->file)
+        if (strstr(block->file,"w_wad.c"))
+	  W_PrintLump(fp, (char*)block + HEADER_SIZE);
       fputc('\n', fp);
       break;
     }
     block=block->next;
+    if (((int)block->file < 0x00001000) && (block->file != NULL) && (block->tag != 0)) {
+	    block->file = NULL;
+    }
   } while (block != zone);
   fprintf(fp, "malloc %d, cache %d, free %d, total %d\n",
-    total_malloc, total_cache, total_free,
-    total_malloc + total_cache + total_free);
+	  total_malloc, total_cache, total_free, 
+	  total_malloc + total_cache + total_free);
   fclose(fp);
 }
 #endif
 #endif
 
+size_t zone_size=8192;
+
 #ifdef INSTRUMENTED
 
 // killough 4/26/98: Add history information
@@ -241,7 +261,7 @@ void Z_DumpHistory(char *buf)
 
 #endif
 
-static void Z_Close(void)
+void Z_Close(void)
 {
   (free)(zonebase);
   zone = rover = zonebase = NULL;
@@ -249,48 +269,28 @@ static void Z_Close(void)
 
 void Z_Init(void)
 {
-#ifdef DJGPP
-  size_t size = _go32_dpmi_remaining_physical_memory();    // Get free RAM
-#else
-  size_t size = MIN_RAM;
-  {  /* cph - allow -heapsize or -heapkb parameters */
-    int p;
-    if ((p=M_CheckParm("-heapsize")))
-      if (++p < myargc) {
-  size = atol(myargv[p]) << 20;
-      }
+  size_t size = zone_size*1000;
 
-    if ((p=M_CheckParm("-heapkb")))
-      if (++p < myargc) {
-  size = atol(myargv[p]) << 10;
-      }
-  }
+#ifdef HAVE_MMAP
+  return; /* cphipps - if we have mmap, we don't need our own heap */
 #endif
-  if (size < MIN_RAM)         // If less than MIN_RAM, assume MIN_RAM anyway
-    size = MIN_RAM;
-
-  size -= LEAVE_ASIDE;        // Leave aside some for other libraries
 
 #ifdef INSTRUMENTED
-  if (!(HEADER_SIZE >= sizeof(memblock_t) && MIN_RAM > LEAVE_ASIDE))
+  if (!(HEADER_SIZE >= sizeof(memblock_t) && size > HEADER_SIZE)) 
     I_Error("Z_Init: Sanity check failed");
 #endif
 
-  atexit(Z_Close);            // exit handler
-
   size = (size+CHUNK_SIZE-1) & ~(CHUNK_SIZE-1);  // round to chunk size
+  size += HEADER_SIZE + CACHE_ALIGN;
 
-   // Allocate the memory
+  // Allocate the memory
 
-  while (!(zonebase=(malloc)(zonebase_size=size + HEADER_SIZE + CACHE_ALIGN)))
-    if (size < (MIN_RAM-LEAVE_ASIDE < RETRY_AMOUNT ? RETRY_AMOUNT :
-                                                     MIN_RAM-LEAVE_ASIDE))
-      I_Error("Z_Init: Failed on allocation of %lu bytes",(unsigned long)
-              zonebase_size);
-    else
-      size -= RETRY_AMOUNT;
+  zonebase=(malloc)(size);
+  if (!zonebase)
+    I_Error("Z_Init: Failed on allocation of %lu bytes", (unsigned long)size);
 
-  lprintf(LO_INFO,"Z_Init : Allocated %luKb zone memory\n", (long unsigned)size >> 10);
+  lprintf(LO_INFO,"Z_Init : Allocated %lukb zone memory\n",
+		  (long unsigned)size / 1000);
 
   // Align on cache boundary
 
@@ -309,32 +309,34 @@ void Z_Init(void)
 
 #ifdef INSTRUMENTED
   free_memory = size;
-  inactive_memory = zonebase_size - size;
-  active_memory = purgable_memory = virtual_memory = 0;
+  /* cph - remove unnecessary initialisations to 0 */
+#endif
+#ifdef HEAPDUMP
+  atexit(Z_DumpMemory);
 #endif
 }
 
 /* Z_Malloc
  * You can pass a NULL user if the tag is < PU_PURGELEVEL.
  *
- * cph - the algorithm here was a very simple first-fit round-robin
- *  one - just keep looping around, freeing everything we can until
+ * cph - the algorithm here was a very simple first-fit round-robin 
+ *  one - just keep looping around, freeing everything we can until 
  *  we get a large enough space
  *
- * This has been changed now; we still do the round-robin first-fit,
- * but we only free the blocks we actually end up using; we don't
+ * This has been changed now; we still do the round-robin first-fit, 
+ * but we only free the blocks we actually end up using; we don't 
  * free all the stuff we just pass on the way.
  */
 
 void *(Z_Malloc)(size_t size, int tag, void **user
 #ifdef INSTRUMENTED
-     , const char *file, int line
+		 , const char *file, int line
 #endif
-     )
+		 )
 {
   register memblock_t *block;
   memblock_t *start, *first_of_free;
-  register size_t contig_free;
+  register size_t contig_free = 0;
 
 #ifdef INSTRUMENTED
   size_t size_orig = size;
@@ -353,7 +355,7 @@ void *(Z_Malloc)(size_t size, int tag, void **user
 #ifdef INSTRUMENTED
              "Source: %s:%d", file, line
 #endif
-       );
+	     );
 #endif
 
   if (!size)
@@ -363,13 +365,17 @@ void *(Z_Malloc)(size_t size, int tag, void **user
 
   block = rover;
 
-  if (block->prev->tag == PU_FREE)
+  /* cph - allow memory allocation even before Z_Init;
+   *  just drop through to the malloc(3) fallback
+   */
+  if (block) {
+   if (block->prev->tag == PU_FREE)
     block = block->prev;
 
-  start = block;
-  first_of_free = NULL; contig_free = 0;
+   start = block;
+   first_of_free = NULL;
 
-  do {
+   do {
     /* If we just wrapped, we're not contiguous with the previous block */
     if (block == zone) contig_free = 0;
 
@@ -383,27 +389,28 @@ void *(Z_Malloc)(size_t size, int tag, void **user
 
       /* First fit */
       if (contig_free >= size)
-  break;
+	break;
     }
+   }
+   while ((block = block->next) != start);   // detect cycles as failure
   }
-  while ((block = block->next) != start);   // detect cycles as failure
 
   if (contig_free >= size) {
     /* We have a block of free(able) memory on the heap which will suffice */
     block = first_of_free;
 
     /* If the previous block is adjacent and free, step back and include it */
-    if (block != zone && block->prev->tag == PU_FREE)
+    if (block != zone && block->prev->tag == PU_FREE) 
       block = block->prev;
 
     /* Free current block if needed */
     if (block->tag != PU_FREE) Z_Free((char *) block + HEADER_SIZE);
 
-    /* Note: guaranteed that block->prev is either
-     * not free or not contiguous
+    /* Note: guaranteed that block->prev is either 
+     * not free or not contiguous 
      *
-     * At every step, block->next must be not free, else it would
-     *  have been merged with our block
+     * At every step, block->next must be not free, else it would 
+     *  have been merged with our block 
      * No range check needed because we know it works by the previous loop */
     while (block->size < size)
       Z_Free((char *)(block->next) + HEADER_SIZE);
@@ -412,44 +419,43 @@ void *(Z_Malloc)(size_t size, int tag, void **user
     {
       size_t extra = block->size - size;
       if (extra >= MIN_BLOCK_SPLIT + HEADER_SIZE) {
-  memblock_t *newb = (memblock_t *)((char *) block +
-            HEADER_SIZE + size);
-
-  (newb->next = block->next)->prev = newb;
-  (newb->prev = block)->next = newb;          // Split up block
-  block->size = size;
-  newb->size = extra - HEADER_SIZE;
-  newb->tag = PU_FREE;
-  newb->vm = 0;
-
+	memblock_t *newb = (memblock_t *)((char *) block +
+					  HEADER_SIZE + size);
+	
+	(newb->next = block->next)->prev = newb;
+	(newb->prev = block)->next = newb;          // Split up block
+	block->size = size;
+	newb->size = extra - HEADER_SIZE;
+	newb->tag = PU_FREE;
+	newb->vm = 0;
+	
 #ifdef INSTRUMENTED
-  inactive_memory += HEADER_SIZE;
-  free_memory -= HEADER_SIZE;
+	inactive_memory += HEADER_SIZE;
+	free_memory -= HEADER_SIZE;
 #endif
       }
-
+      
       rover = block->next;           // set roving pointer for next search
-
+      
 #ifdef INSTRUMENTED
       inactive_memory += block->extra = block->size - size_orig;
       if (tag >= PU_PURGELEVEL)
-  purgable_memory += size_orig;
+	purgable_memory += size_orig;
       else
-  active_memory += size_orig;
+	active_memory += size_orig;
       free_memory -= block->size;
 #endif
     }
   } else {
-    /* Allocate a vm block *
+    /* Allocate a vm block * 
      * We've run out of physical memory, or so we think.
      * Although less efficient, we'll just use ordinary malloc.
      * This will squeeze the remaining juice out of this machine
      * and start cutting into virtual memory if it has it.
      */
-
     
 #ifdef HAVE_LIBDMALLOC
-    while (!(block = dmalloc_malloc(file,line,size + HEADER_SIZE,DMALLOC_FUNC_MALLOC,0,0))) {
+    while (!(block = _malloc_leap(file,line,size + HEADER_SIZE))) {
 #else
     while (!(block = (malloc)(size + HEADER_SIZE))) {
 #endif
@@ -458,11 +464,11 @@ void *(Z_Malloc)(size_t size, int tag, void **user
 #ifdef INSTRUMENTED
                  "\nSource: %s:%d"
 #endif
-     ,(unsigned long) size
+		 ,(unsigned long) size
 #ifdef INSTRUMENTED
-     , file, line
+		 , file, line
 #endif
-     );
+		 );
       Z_FreeTags(PU_CACHE,PU_CACHE);
     }
 
@@ -471,20 +477,20 @@ void *(Z_Malloc)(size_t size, int tag, void **user
     blockbytag[tag] = block;
     block->prev = (memblock_t *) &blockbytag[tag];
     block->vm = 1;
-
+    
 #ifdef INSTRUMENTED
     virtual_memory += size + HEADER_SIZE;
 #endif
-    /* cph - the next line was lost in the #ifdef above, and also added an
+    /* cph - the next line was lost in the #ifdef above, and also added an 
      *  extra HEADER_SIZE to block->size, which was incorrect */
-    block->size = size;
+    block->size = size; 
   }
 
 #ifdef INSTRUMENTED
   block->file = file;
   block->line = line;
 #endif
-
+  
 #ifdef ZONEIDCHECK
   block->id = ZONEID;         // signature required in block header
 #endif
@@ -493,9 +499,8 @@ void *(Z_Malloc)(size_t size, int tag, void **user
   block = (memblock_t *)((char *) block + HEADER_SIZE);
   if (user)                   // if there is a user
     *user = block;            // set user to point to new block
-
+  
 #ifdef INSTRUMENTED
-  Z_PrintStats();           // print memory allocation stats
   // scramble memory -- weed out any bugs
   memset(block, gametic & 0xff, size);
 #endif
@@ -504,9 +509,9 @@ void *(Z_Malloc)(size_t size, int tag, void **user
 
 void (Z_Free)(void *p
 #ifdef INSTRUMENTED
-        , const char *file, int line
+	      , const char *file, int line
 #endif
-        )
+	      )
 {
 #ifdef INSTRUMENTED
 #ifdef CHECKHEAP
@@ -550,7 +555,7 @@ void (Z_Free)(void *p
           virtual_memory -= block->size + HEADER_SIZE;
 #endif
 #ifdef HAVE_LIBDMALLOC
-          dmalloc_free(file,line,block,DMALLOC_FUNC_MALLOC);
+          _free_leap(file,line,block);
 #else
           (free)(block);
 #endif
@@ -601,20 +606,16 @@ void (Z_Free)(void *p
 #endif
             }
         }
-
-#ifdef INSTRUMENTED
-      Z_PrintStats();           // print memory allocation stats
-#endif
     }
 }
 
 void (Z_FreeTags)(int lowtag, int hightag
 #ifdef INSTRUMENTED
-      , const char *file, int line
+		  , const char *file, int line
 #endif
-      )
+		  )
 {
-  /* cph - move rover to start of zone; we like to encourage static
+  /* cph - move rover to start of zone; we like to encourage static 
    * data to stay in one place, at the start of the heap
    */
   memblock_t *block = rover = zone;
@@ -626,6 +627,7 @@ void (Z_FreeTags)(int lowtag, int hightag
   if (lowtag <= PU_FREE)
     lowtag = PU_FREE+1;
 
+  if (block)
   do               // Scan through list, searching for tags in range
     if (block->tag >= lowtag && block->tag <= hightag)
       {
@@ -635,14 +637,14 @@ void (Z_FreeTags)(int lowtag, int hightag
 #else
         (Z_Free)((char *) block + HEADER_SIZE);
 #endif
-  /* cph - be more careful here, we were skipping blocks!
-   * If the current block was not merged with the previous,
-   *  cur is still a valid pointer, prev->next == cur, and cur is
-   *  already free so skip to the next.
-   * If the current block was merged with the previous,
-   *  the next block to analyse is prev->next.
-   * Note that the while() below does the actual step forward
-   */
+	/* cph - be more careful here, we were skipping blocks!
+	 * If the current block was not merged with the previous, 
+	 *  cur is still a valid pointer, prev->next == cur, and cur is 
+	 *  already free so skip to the next.
+	 * If the current block was merged with the previous, 
+	 *  the next block to analyse is prev->next.
+	 * Note that the while() below does the actual step forward 
+	 */
         block = (prev->next == cur) ? cur : prev;
       }
   while ((block=block->next) != zone);
@@ -683,9 +685,9 @@ void (Z_FreeTags)(int lowtag, int hightag
 
 void (Z_ChangeTag)(void *ptr, int tag
 #ifdef INSTRUMENTED
-       , const char *file, int line
+		   , const char *file, int line
 #endif
-       )
+		   )
 {
   memblock_t *block = (memblock_t *)((char *) ptr - HEADER_SIZE);
 
@@ -748,7 +750,7 @@ void *(Z_Realloc)(void *ptr, size_t n, int tag, void **user
 #ifdef INSTRUMENTED
                   , const char *file, int line
 #endif
-      )
+		  )
 {
   void *p = (Z_Malloc)(n, tag, user DA(file, line));
   if (ptr)
@@ -766,7 +768,7 @@ void *(Z_Calloc)(size_t n1, size_t n2, int tag, void **user
 #ifdef INSTRUMENTED
                  , const char *file, int line
 #endif
-     )
+		 )
 {
   return
     (n1*=n2) ? memset((Z_Malloc)(n1, tag, user DA(file, line)), 0, n1) : NULL;
@@ -776,19 +778,20 @@ char *(Z_Strdup)(const char *s, int tag, void **user
 #ifdef INSTRUMENTED
                  , const char *file, int line
 #endif
-     )
+		 )
 {
   return strcpy((Z_Malloc)(strlen(s)+1, tag, user DA(file, line)), s);
 }
 
 void (Z_CheckHeap)(
 #ifdef INSTRUMENTED
-       const char *file, int line
+		   const char *file, int line
 #endif
-       )
+		   )
 {
   memblock_t *block = zone;   // Start at base of zone mem
-  do                          // Consistency check (last node treated special)
+  if (block)
+  do {                        // Consistency check (last node treated special)
     if ((block->next != zone &&
          (memblock_t *)((char *) block+HEADER_SIZE+block->size) != block->next)
         || block->next->prev != block || block->prev->next != block)
@@ -799,5 +802,12 @@ void (Z_CheckHeap)(
               , file, line, block->file, block->line
 #endif
               );
-  while ((block=block->next) != zone);
+//#ifdef INSTRUMENTED
+// shouldn't be needed anymore, was just for testing
+#if 0
+    if (((int)block->file < 0x00001000) && (block->file != NULL) && (block->tag != 0)) {
+	    block->file = NULL;
+    }
+#endif
+  } while ((block=block->next) != zone);
 }
